// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.actions;

import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import static com.intellij.openapi.ui.Messages.showMessageDialog;
import static com.intellij.openapi.ui.Messages.getErrorIcon;
import com.telelogic.integration.intellij.cm.CMConfiguration;

import javax.swing.*;

/**
 * Superclass for all user actions requiring CLI session interaction.
 * @author Luke Stahlman
 */
abstract class CMAction extends AnAction {
    String actionName = "";
    Project project;
    CMConfiguration myConfig;
    VirtualFile[] virtualFiles;

    public CMAction(String text, String description, Icon icon) {
        super(text, description, icon);
    }

    public CMAction(String text) {
        super(text);
    }

    public CMAction() {
        super();
    }

    /**
     * Sets the visibility of actions in the IDEA popup menus.
     * If a session is present, all popup menu items whose actions extend this class will be enabled. Conversely, they
     * will be disabled if no session is present.
     * @param event The AnActionEvent object associated with the popup menu item
     */

    public void update(AnActionEvent event)
    {
        Presentation presentation = event.getPresentation();
        boolean state = false;

        Project project = (Project)event.getData(DataKeys.PROJECT);
        if( project != null)
        {
            // Check the VSS is enabled.
            //CMPlugin host = CMPlugin.getInstance(project);
            CMConfiguration config = CMConfiguration.getInstance(project);
            //ProjectLevelVcsManager pm = ProjectLevelVcsManager.getInstance(project);

            // Check for a session
            state = config.SESSION_STARTED;
        }
        presentation.setEnabled( state );
    }

    /**
     * Checks if the action can be performed without errors.
     * This method checks first for an active session, then for the size of the VirtualFile array. These are required
     * for nearly every action, however this method is overloaded when other validation parameters are required.
     * It is debatable whether validation code should be exclusively in the actions package or the cli package, or if
     * it's acceptable to have such code in both packages.  For now, it's in both.
     * @param config The plugin configuration, which is used to check for a session
     * @param files The VirtualFile array to be checked for objects
     * @return Whether the validation checks in succeeded
     */
    boolean validateAction(CMConfiguration config, VirtualFile[] files) {
        // Look for a CM session
        if(!config.SESSION_STARTED)
        {
            // Ask user to first start a CM Session
            showMessageDialog("No active CM session is present", "Error", getErrorIcon());
        }
        else{
            if(files.length == 0){
                showMessageDialog("No files or directories were selected", "Error", getErrorIcon());
            }
            else {
                return true;
            }
        }
        return false;
    }
}
