// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.actions;

import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.actionSystem.DataKeys;
//import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import static com.intellij.openapi.ui.Messages.showMessageDialog;
import static com.intellij.openapi.ui.Messages.getErrorIcon;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.vcs.VcsException;
import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.cm.CMUtil;
import com.telelogic.integration.intellij.cli.CheckOutCommand;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

/**
 * Handles user actions related to checking out objects
 * @author Luke Stahlman
 */
public class CheckOutAction extends CMAction implements ActionListener {

    /**
     * Default constrcutor
     */
    public CheckOutAction() {
        super();
    }

    /**
     * Constructor method used when setting this as an ActionListener
     * @param p The active project
     */
    public CheckOutAction(Project p) {
        super();
        project = p;
    }

    /**
     * Handles JComponent actions.
     * This method assumes the action was fired for the file currently selected (in focus) in the IDEA editor. Event
     * data can't be used to find the project, config and files involved in this action. It relies instead on the
     * Project instance passed to the constructor to gather file information and validate the action. On successful
     * validation, the CLI command is executed via com.telelogic.integration.intellij.cm.CheckOutCommand.
     * @param event Event data (not used)
     */
    public void actionPerformed(ActionEvent event) {
        myConfig = CMConfiguration.getInstance(project);
        //status = CMPlugin.getInstance(project).toolWindow;
       VirtualFile file = myConfig.getSelectedFile();

        ArrayList<VirtualFile> fileArray = new ArrayList<VirtualFile>();
        fileArray.add(file);

        virtualFiles = fileArray.toArray(new VirtualFile[fileArray.size()]);


        if(validateAction()) {
            // run command
            try {
                new CheckOutCommand(virtualFiles, project, true, false);
            } catch (VcsException e) {
                 Messages.showErrorDialog(e.getMessage(), "Check Out Error");
            }
        }
    }

    /**
     * Handles IDEA popup menu actions.
     * Project, config and file information is gathered from the event data.  On successful validation, the CLI command
     * is executed via com.telelogic.integration.intellij.cm.CheckOutCommand.
     * @param event Event data
     */
    public void actionPerformed(AnActionEvent event) {
        project = (Project)event.getData(DataKeys.PROJECT);
        myConfig = CMConfiguration.getInstance(project);
        virtualFiles = CMUtil.getVirtualFiles( event );

        if(validateAction()) {
            try {
                new CheckOutCommand(virtualFiles, project, true, true);
            }
            catch (VcsException e) {
                Messages.showErrorDialog(e.getMessage(), "Checkout Error");
            }
        }
    }

    /**
     * Overloads super method for additional validation.
     * Specifically, it checks whether a current/default task is set before allowing check out.
     * @return Whether the validation checks succeeded
     */
    private boolean validateAction() {
        if(!super.validateAction(myConfig, virtualFiles)) {
            return false;
        }
        else {
            // Task 117: because the checkout command checks for tasks now, this isn't needed
            /*if(!myConfig.isTaskSet()) {
                showMessageDialog("Please select a task before checking out files", "Error", getErrorIcon());
                return false;
            }*/
            return true;
        }
    }
}
