// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.actions;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.ui.Messages;
import com.intellij.psi.search.PsiShortNamesCache;
import com.intellij.psi.PsiManager;
import com.intellij.psi.PsiFile;
import com.telelogic.integration.intellij.cli.PropertiesCommand;
import com.telelogic.integration.intellij.ui.CMMessages;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.ArrayList;


/**
 * Looks for and displays files in the active IDEA project that match a 4-part Synergy object name. This class is linked
 * to the associated objects table in the "My Tasks" tool window tab.
 * @author Luke Stahlman
 */
public class ViewAssociatedObjectAction implements ActionListener {

    /**
     * Searches for and displays files in IDEA which match a Synergy object name. The project is first searched for
     * files matching the short name, then gets their respective full object names from Synergy via the
     * PropertiesCommand class. If a match is found, an editor in IDEA is opened with the appropriate project file.
     * @param project       The active IDEA project
     * @param objectName    The Synergy object name (1/java/myfile.java/1.2.1)
     * @param fileName      The short file name (myfile.java)
     */
    public ViewAssociatedObjectAction(Project project, String objectName, String fileName) {
        FileEditorManager editor        = FileEditorManager.getInstance(project);
        ArrayList<VirtualFile> matches  = new ArrayList<VirtualFile>();
        PsiShortNamesCache find         = PsiManager.getInstance(project).getShortNamesCache();
        PsiFile[] results               = find.getFilesByName(fileName);

        // Iterate through results
        for(PsiFile result : results) {
            VirtualFile file = result.getVirtualFile();
            HashMap<String, String> fileProperties = new PropertiesCommand(file, project).get("name:%fullname");

            if(fileProperties.get("name").equals(objectName)) {
                matches.add(file);
            }
        }

        //If 0 results, error. If 1 result, open file in editor. If more than 1 result, open file in editor, but


        if(matches.size() == 1) {
            if(!editor.isFileOpen(matches.get(0))) {
                // Open file with focus
                editor.openFile(matches.get(0),true);

            }
            else {
                // Place editor focus on file
                editor.openFile(matches.get(0),true);
            }
        }
        else if(matches.size() > 1) {
            String[] fileNames = new String[matches.size()];

            // TODO truncate the display strings to 70 chars, but do it intelligently (C:\...\...\sth\something)
            for(int i = 0; i < matches.size(); i++) {
                fileNames[i] = matches.get(i).getName() + " " + matches.get(i).getPath();
            }
            Integer choice = Messages.showChooseDialog("There are multiple files in your project that match this object. " +
                    "Which would you like to open?", "Choose file", fileNames, fileNames[0], null);
            
            if(choice != null) {
                if(!editor.isFileOpen(matches.get(0))) {
                    // Open file with focus
                    editor.openFile(matches.get(0),true);

                }
                else {
                    // Place editor focus on file
                    editor.openFile(matches.get(0),true);
                }
            }
        }
        else {
            // TODO: Allow users to view the file anyway
            CMMessages.generalMessage("This file does not exist in your IDEA project.");
        }
    }

    /**
     * Blank implementation for ActionListener. This is a placeholder in case future UI features fire this action
     * @param e Action
     */
    public void actionPerformed(ActionEvent e) {

    }

    /**
     * Intelligently shortens file paths for UI display. This method is still under development, but the idea is to
     * convert something like c:/documents and settings/user1/idea projects/project foo/com/bar/main.java to
     * c:/documents and settings/../../../bar/main.java
     * @param path The path to be shortened
     * @param maxLength The maximum length of the shortened path.
     */
    public void truncatePath(String path, int maxLength) {
        int currentLength   = 0;
        int chopLength      = currentLength - maxLength;

        if(chopLength > 0) {
            // Split directories
            String[] split = path.split("/");

            // What's the maximum amount of text we can remove
            int choppableText = 0;
            for(String val : split) {
               choppableText += val.length();
            }
            // TODO: Finish this code, although it is not truly that important


            // Each entry in split will need to have their directory slashes put back. Therefore we will count those /'s
            // As the current length of the outputted string
            currentLength += split.length - 1;

            //ignore first entries directories and last 2 entries (C:/project/../../parentDir/filename.java)
            if(split.length > 2) {
                currentLength += split[0].length() + split[1].length();
            }
            else {
                currentLength += split[0].length();
            }
        }

    }
}
