// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.fileEditor.FileDocumentManager;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.progress.Task;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.progress.EmptyProgressIndicator;
import com.intellij.vcsUtil.VcsRunnable;
import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.cm.CMPlugin;
import com.telelogic.integration.intellij.ui.CMToolWindow;
import com.telelogic.integration.intellij.ui.dialogs.CommentDialog;

import java.util.ArrayList;

/**
 * The base class on which all commands are built. Provides a super method to gather necessary environment information
 * used by all subclasses and provides file saving functionality.
 * @author Luke Stahlman
 */
public abstract class AbstractCommand {
    Project             project;
    CMConfiguration     myConfig;
    CMToolWindow        toolWindow;
    VirtualFile[]       virtualFiles;
    Process             process;
    ArrayList<String>   options;
    boolean             showSuccessMessage = true;
    private Task        taskIndicator;
    

    /**
     * Default constructor
     */
    public AbstractCommand() {}

    /**
     * Super method used by all subclasses to gather information specific to the IDEA project environment
     * @param p The active IDEA project
     */
    public AbstractCommand(Project p) {
        project = p;
        myConfig = CMConfiguration.getInstance(project);
        toolWindow = CMPlugin.getInstance(project).toolWindow;
    }

    public VcsRunnable getProgressIndicator(final String title, boolean modal, boolean canBeCancelled) {

        VcsRunnable runner = new VcsRunnable() {
            public void run() {

            }
        };

        return runner;
    }

    /**
     * Saves files before Synergy commands are executed.  This is useful when IDEA has cached but not written all
     * changes to an object before the user fires a Synergy command such as Check In. This method first checks if it
     * needs to run a save operation on the file.
     * @param files The files to be saved
     */
    void saveFiles(VirtualFile[] files) {
        FileDocumentManager fileManager = FileDocumentManager.getInstance();
        for (VirtualFile file : files){
            Document doc = fileManager.getDocument(file);
            // Refactoring change: files added to Synergy here may not be created yet, therefore IDEA will set doc as null
            if(doc != null) {
                if(fileManager.isDocumentUnsaved(doc)){
                    fileManager.saveDocument(doc);
                }
            }
        }
    }

    /**
     * Retries the command (if overridden)
     */
    public void retry() {

    }

    public String getComment(String title) throws Exception {

        CommentDialog dialog = new CommentDialog();

        dialog.setTitle(title);
        dialog.setRequired(true);
        dialog.showDialog();

        if(!dialog.wasCancelled()) {
            String comment = dialog.getComment();

            if(comment != null) {
                return comment;
            }
            else {
                return "";
            }
        }
        else {
            throw new Exception("Check in cancelled");    
        }


    }

    /**
     * String value of this class
     * @return Name of the class
     */
    public abstract String toString();
}
