// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VirtualFileSystem;
import com.intellij.openapi.vfs.VirtualFileManager;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.vcs.FilePath;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.vcsUtil.VcsUtil;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMUtil;
import com.telelogic.integration.intellij.cm.CMPlugin;

import javax.swing.*;
import java.util.ArrayList;
import java.util.LinkedList;
import java.io.File;

/**
 * Used to add objects to Synergy. Currently only able to add files, but support for directories will come soon
 * @author Luke Stahlman
 */
public class AddCommand extends AbstractCommand {
    // TODO: Add support for directories

    public static int TYPE_DIR      = 0;
    public static int TYPE_FILE     = 1;
    public static int TYPE_MIXED  = -1;

    private String  taskID;
    private boolean succeeded = false;
    public  boolean showSuccessMessage = false;


    /**
     * Used to add a number of objects to Synergy at once.  These are only for pre-existing objects whose type is unknown
     * @param existingFiles The objects to be added
     * @param p The active IntelliJ project
     * @param task The task this operation should be associated with
     */
    @SuppressWarnings({"SameParameterValue"})
    private AddCommand(Project p, VirtualFile[] existingFiles, String task) {
        super(p);
        setTask(task);
        saveFiles(existingFiles);
        try {
            for(VirtualFile file : existingFiles) {
                if(file.exists()) {
                    // Check for and attempt to control any uncontrolled ancestors
                    FindUncontrolledAncestors finder = new FindUncontrolledAncestors(file);

                    if(finder.findAndControl(false)) {
                        if(file.isDirectory()) {
                            execute(file.getPath(), TYPE_DIR);
                        }
                        else {
                            execute(file.getPath(), TYPE_FILE);
                        }
                        file.refresh(false, false);
                    }
                }
                else {
                    //TODO: Handle bad files?
                }

            }
        } catch (Exception e) {

        }
    }

    /**
     * This constructor is used to create controlled objects not yet in the file system
     * @param p
     * @param parent
     * @param name
     * @param type
     * @param task
     */
    private AddCommand(Project p, VirtualFile parent, String name, int type, String task) {
        super(p);
        boolean error = false;

        setTask(task);

        // Check if parent exists and is a directory.  Then look for file naming conflicts
        if(parent.exists() && parent.isDirectory()) {
            VirtualFile[] children = parent.getChildren();

            for(VirtualFile child : children) {
                if(child.getName().equals(name)) {
                    error = true;
                }
            }

            if(!error) {
                // Check for and attempt to control any uncontrolled ancestors
                try {
                    FindUncontrolledAncestors finder = new FindUncontrolledAncestors(parent);

                    if(finder.findAndControl(true)) {
                        execute(parent.getPresentableUrl() + File.separator + name, type);
                    }
                } catch (Exception e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }

            }
        }
    }

    public AddCommand(Project p, VirtualFile file, int type, String task) {
        super(p);

        setTask(task);
        // Check for and attempt to control any uncontrolled ancestors
        try {
            FindUncontrolledAncestors finder = new FindUncontrolledAncestors(file);
            if(finder.findAndControl(false)) {
                execute(file.getPath(), type);
            }
        } catch (Exception e) {

        }
    }

    private void setTask(String task) {
        if(task != null) {
            taskID = task;
        }
        else {
            taskID = myConfig.getDefaultTask();
        }
    }

    /**
     * Runs the command and sends output to the HelperThread
     * @param fileSpec
     * @param type
     */
    private void execute(final String fileSpec, int type) throws VcsException {
        options = new ArrayList<String>();

        options.add(myConfig.getPath());

        options.add("create");

        if(taskID != "") {
            options.add("-task");
            options.add(taskID);
        }
        else {
            if(!new Tasks(project).ensureTaskSet()) {
                JOptionPane.showMessageDialog(null, "This file will be created, but cannot be controlled by Synergy without a task.\n" +
                        "Please set a task, then manually add this file to Synergy.",
                        "Cannot add file to Synergy", JOptionPane.WARNING_MESSAGE);
                throw new VcsException("Error: No task is set.");
            }
        }

        if(type == TYPE_DIR) {
            options.add("-type");
            options.add("dir");
        }
        
        options.add(fileSpec);

        String[] commands = options.toArray(new String[options.size()]);

        try {
            toolWindow.appendConsole("Starting Add Operation...");
            // Run command
            process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
           
            //new HelperThread(this).start();

            new Consumer(project, process, true, true);

        } catch (Exception e) {
            new CMExceptionHandler(e);
        }

        // Now check if the file exists in Synergy
        //AttributeCommand attr = new AttributeCommand(project, fileSpec);

        final boolean[] exists = new boolean[1];
        final VirtualFile[] file = new VirtualFile[1];

        ApplicationManager.getApplication().runWriteAction(new Runnable() {
            public void run() {
                file[0] = VirtualFileManager.getInstance().refreshAndFindFileByUrl(fileSpec);
                exists[0] = (file[0] != null);
            }
        });

        // If the file exists, Synergy probably created it and is controlling it.  We test here whether it's controlled
        // As the success of the add command depends on whether the file is controlled, succeeded is assigned the result of
        // properties.isControlled()
        if(exists[0]) {
            PropertiesCommand properties = new PropertiesCommand(file[0], project);
            succeeded = properties.isControlled();
        }

    }

    public static boolean createDir(Project p, VirtualFile parent, String child, String task) {
        AddCommand executor = new AddCommand(p, parent, child, TYPE_DIR, task);

        return executor.succeeded;
    }

    public static boolean addDir(Project p, VirtualFile dir, String task) {
        AddCommand executor = new AddCommand(p, dir, TYPE_DIR, task);

        return executor.succeeded;
    }

    public static boolean createFile(Project p, VirtualFile parent, String child, String task) {
        AddCommand executor = new AddCommand(p, parent, child, TYPE_FILE, task);

        return executor.succeeded;
    }

    /*public static boolean createFiles(Project p, VirtualFile parent, String child, String task) {
        AddCommand executor = new AddCommand(p, parent, child, TYPE_FILE, task);

        return executor.succeeded;
    }

    public static boolean addFile(Project p, VirtualFile file, String task) {
        AddCommand executor = new AddCommand(p, file, TYPE_FILE, task);

        return executor.succeeded;
    }*/

    public static boolean addFiles(Project p, VirtualFile[] files, String task) {

        AddCommand executor = new AddCommand(p, files, task);

        return executor.succeeded;
    }

    // This is where a mix of files and directories are being created all in one shot
    /*public static boolean createMixed(Project p, VirtualFile[] files, String task) {
        return false;
    }*/

    /**
     * This is where a mix of files and directories are being added all in one shot
     * @param p
     * @param files
     * @param task
     * @return
     */
    public static boolean addMixed(Project p, VirtualFile[] files, String task) {
        // Split VirtualFiles into dirs and files
        // First add dirs, then file
        ArrayList<VirtualFile> notAdded = new ArrayList<VirtualFile>();

        ArrayList<VirtualFile> dirsToAdd = new ArrayList<VirtualFile>();
        ArrayList<VirtualFile> filesToAdd = new ArrayList<VirtualFile>();

        for(VirtualFile file : files){
            if(file.isDirectory()) {
                dirsToAdd.add(file);
            }
            else {
                filesToAdd.add(file);
            }
        }

        for(VirtualFile dir : dirsToAdd) {
            if(!addDir(p, dir, task)) {
                notAdded.add(dir);
            }
        }

        boolean filesAdded = false;

        if(filesToAdd.size() == 0) {
            if(notAdded.size() == 0) {
                return true;
            }
        }
        else {
            boolean fileAddSucceeded = addFiles(p, filesToAdd.toArray(VirtualFile.EMPTY_ARRAY), task);

            return (fileAddSucceeded && notAdded.size() == 0);
        }

        return false;
    }

    /**
     * @see com.telelogic.integration.intellij.cli.AbstractCommand#toString()
     */
    public String toString() { return "Add"; }

    /**
     * Finds any uncontrolled ancestors to a given object
     * This class checks the parent dir(s) of an object to be placed under Synergy control.  The method is as follows:
     * -Check if the parent dir is controlled.  If yes, stop.  If not, go to the next parent and do the same
     * -On stopping, return the directories that need to be placed under Synergy control before the object can be added
     */
    private class FindUncontrolledAncestors {

        VirtualFile _object;

        public FindUncontrolledAncestors(VirtualFile object) throws Exception {
            _object = object;
            eval();
        }

        public FindUncontrolledAncestors(FilePath objectPath) throws Exception {
            _object = objectPath.getVirtualFile();
            eval();
        }

        public FindUncontrolledAncestors(String objectPath) throws Exception {
            _object = VcsUtil.getFilePath(objectPath).getVirtualFile();
            eval();
        }

        public LinkedList<VirtualFile> find() {
            /*
                It is important this list is ordered from highest to lowest in the hierarchy, otherwise there will be
                trouble adding the list entries to Synergy
             */
            LinkedList<VirtualFile> uncontrolled = new LinkedList<VirtualFile>();

            if(!new PropertiesCommand(_object, project).isControlled()) {
                uncontrolled.addLast(_object);  // Last because this is the end of the hierarchy.
                                                // It doesn't actually matter here, but it's supposed to add clarity
                // Call a recursive method to get the full hierarchy that needs to be added to Synergy before the 'object'
                // can be added
                return getUncontrolledAncestors(uncontrolled);
            }
            else {
                return uncontrolled;
            }

        }

        /**
         * This is a recursive method that will call itself until it finds a controlled dir or gets to the top of the
         * project.  Presumably the top of the project is under Synergy control, but this cannot be assumed
         * @param list The list of uncontrolled objects.  This starts as just the object but can expand as far as
         * the project's uppermost level in the filesystem hierarchy
         * @return The full hierarchy of uncontrolled files (including the object)
         */
        private LinkedList<VirtualFile> getUncontrolledAncestors(LinkedList<VirtualFile> list) {

            VirtualFile current = list.getFirst();  // Gets "first" as that's the top of the hierarchy

            VirtualFile parent = current.getParent();

            if(!new PropertiesCommand(parent, project).isControlled()) {

                // We may need to loop again.  First check if we're in the project still and we're in the portion of
                // the hierarchy under Synergy control (just in case only one module of many is Synergy-controlled)

                if(     parent != null &&
                        parent.isValid() &&
                        VcsUtil.isPathUnderProject(project, parent) &&
                        VcsUtil.isFileForVcs(parent, project, CMPlugin.getInstance(project))
                   ) {

                    list.addFirst(parent); // Put the uncontrolled dir on the top of the list (top of the hierarchy)
                    getUncontrolledAncestors(list); // Loop through the code
                }
            }

            // We get to this return statement only when the list hits:
            // - A controlled directory
            // - The top of the Synergy-controlled portion of the project
            // - The top of the project

            return list;
        }

        /**
         *  Looks at _object and throws an Exception if there's a problem (e.g. it's null)
         * @throws Exception File error
         */
        private void eval()throws Exception {
            if(_object == null) {
                throw new Exception("File error");
            }
        }

        /**
         * Looks for uncontrolled ancestors and attempts to control them
         * @param controlLast Control the object passed to the constructor (example: when adding a new file to Synergy,
         * the parent is the object passed to this class and therefore needs to be considered)
         * @return Whether all ancestors are controlled
         */
        public boolean findAndControl(boolean controlLast) {
            // Get the list of uncontrolled
            LinkedList<VirtualFile> uncontrolled = find();

            if(uncontrolled.size() > 1 || (uncontrolled.size() > 0 && controlLast)) {
                // Ask the user if these should be controlled.
                String message = "The following parent directories will be added to Synergy as a part of this operation:";
                for(VirtualFile file : uncontrolled) {
                    if(!file.equals(_object) || controlLast) {
                        message += "\n  " + file.getName();
                    }
                }

                message += "\nWould you like to continue?";

                int answer = Messages.showYesNoDialog(project, message, "Control parent directories?", null);

                //  Return false if user rejects (as that means the original file cannot be controlled)
                //  Send list to AddCommand.addMixed if true
                if(answer == 0) {

                    // Remove the object passed to the class if it's not to be controlled in this operation
                    if(!controlLast) {
                        uncontrolled.remove(_object);
                    }
                    // Return the success or failure of controlling the ancestors
                    return AddCommand.addMixed(project, uncontrolled.toArray(VirtualFile.EMPTY_ARRAY), null);
                }
                // Cancel the control operation
                else {
                    return false;
                }
            }
            else {
                // No ancestors need to be controlled
                return true;
            }
        }
    }

    private class Consumer extends AbstractOutputConsumer {
        String objectName;
        boolean inWarning = false;
        String failureMessage;

        public Consumer(Project project, Process process, boolean console, boolean redirectedErrorStream){
            super(project, process, console, redirectedErrorStream);
        }

        protected void findErrors(String line) {

            /*
            Added 'release' attribute with value '<release>' to object <object>
            Warning: A task must be specified to check out '<object>'.
            Unable to add object directory entry to directory.
            */

            if(inWarning) {

            }
            // No task set
            else if(line.contains("Warning: A task must be specified to check out")) {
                failed = true;
                failureMessage = line;
            }
            // Also no task set, but caught anyways
            else if(line.contains("Unable to add object directory entry to directory")) {
                failed = true;
                failureMessage = line;
            }
            // Probably not in a project controlled by the Synergy DB
            else if(line.contains("Invalid object specification")) {
                failed = true;
                failureMessage = line;
            }
            // File already exists
            else if(line.contains("Object version or directory entry already exists with this name:")) {
                failed = true;
                failureMessage = line;
            }
        }

        protected void finished(ArrayList<String> result) {
            String message = "";

            toolWindow.updateCurrentFileTab();

            for(String line : result) {
                if(line.contains("Member") && line.contains("added to project")) {
                    succeeded = true;
                    message = line;
                }
            }

            if(succeeded) {
                Messages.showMessageDialog(project, message, "Member added to Synergy", null);
            }
            else if (failed) {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        Messages.showMessageDialog(project, failureMessage, "Member not added to Synergy", null);
                    }
                });
            }
        }
    }

}