// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vcs.FilePath;

import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * Implementation of the ccm attribute command.  Includes constant vars for the base DB model
 */
public class AttributeCommand extends AbstractCommand {

    private String objectName = null;
    private String filePath = null;
    
    // Default attributes in the base DB model
    public static final String COMMENT              = "comment";
    public static final String CREATE_TIME          = "create_time";
    public static final String CVTYPE               = "cvtype";
    public static final String HANDLE_SOURCE_AS     = "handle_source_as";
    public static final String IS_ASM               = "is_asm";
    public static final String IS_MODEL             = "is_model";
    public static final String MODIFY_TIME          = "modify_time";
    public static final String NAME                 = "name";
    public static final String OWNER                = "owner";
    public static final String PROJECT              = "project";
    public static final String RELEASE              = "release";
    public static final String SOURCE               = "source";
    public static final String SOURCE_CREATE_TIME   = "source_create_time";
    public static final String SOURCE_MODIFY_TIME   = "source_modify_time";
    public static final String STATUS               = "status";
    public static final String STATUS_LOG           = "status_log";
    public static final String SUBSYSTEM            = "subsystem";
    public static final String VERSION              = "version";

    public AttributeCommand(Project p, String objectName) {
        super(p);
        this.objectName = objectName;
    }

    public AttributeCommand(Project p, FilePath path) {
        super(p);
        filePath = path.getPath();
    }

    /**
     * ccm attr -s [attribute] [objectName]
     * This is a very basic CLI command that does not require much parsing.  Output is for just one attribute,
     * so that output is simply read into a String.  Any processing errors return null
     * @param attribute The desired attribute
     * @return The attribute value (if any)
     */
    public String get(String attribute) {
        Process p;
        BufferedReader br;
        ArrayList<String> options = new ArrayList<String>();
        String result = null;

        options.add(myConfig.getPath());
        options.add("attr");
        options.add("-show");
        options.add(attribute);
        if(objectName == null && filePath != null) {
            options.add(filePath);
        }
        else {
            options.add(objectName);
        }

        String[] command = options.toArray(new String[options.size()]);

        try {
            // Run command
            p = CommandHelper.runCommand(command, myConfig.getCmEnvPath());
            br = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String l;
            int lineCount = 0;
            
            while ((l = br.readLine()) != null) {
                // If this isn't the first line of output, add a line break
                if(lineCount > 0) {
                    result += "\n";
                }

                // Add the output to the result string
                if(result == null) {
                    result = l;
                }
                else {
                    result += l;
                }

                lineCount++;
            }
            
            br.close();

        } catch (IOException e) { new CMExceptionHandler(e); }

        return result;
    }

    public byte[] getAsByte(String attribute) {
        Process p;
        InputStreamReader reader;
        ByteArrayOutputStream bytes = new ByteArrayOutputStream();
        ArrayList<String> options       = new ArrayList<String>();
        String result = null;

        options.add(myConfig.getPath());
        options.add("attr");
        options.add("-show");
        options.add(attribute);
        if(objectName == null && filePath != null) {
            options.add(filePath);
        }
        else {
            options.add(objectName);
        }

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Run command
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
            reader = new InputStreamReader(p.getInputStream());

            int c;

            while ((c = reader.read()) != -1) {
                bytes.write(c);
            }


        } catch (IOException e) { new CMExceptionHandler(e); }

        return bytes.toByteArray();

    }

    /**
     * String value of this class
     *
     * @return Name of the class
     */
    public String toString() {
        return "Attribute";
    }
}
