// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VirtualFileManager;
import com.intellij.openapi.ui.Messages;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMPlugin;
import com.telelogic.integration.intellij.ui.dialogs.CommentDialog;
import com.telelogic.integration.intellij.cli.helpers.ParallelHelper;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * Handles Check-in functions
 * @author Luke Stahlman
 */
public class CheckInCommand extends AbstractCommand {
    private boolean showConsole = false;
    //private ArrayList<VirtualFile> filesToCheckIn;
    private VirtualFile[]   filesToCheckIn;
    private String          comment;
    private ArrayList<String> parallelConflicts = new ArrayList<String>();
    
    // TODO: Save file(s) first before checkin
    /**
     * Special method to bypass normal verification of objects before they're checked in. Used when a properties
     * command has already been run on the file (this currently happens when the "Check In" button is used in the
     * current file tab
     * @param vf The objects to be checked in
     * @param p The active IDEA project
     * @param isVerified Whether the objects ready to be checked in
     * @param showConsole Whether the output of the check in command should be automatically displayed to the user
     */
    public CheckInCommand(VirtualFile[] vf, Project p, boolean isVerified, boolean showConsole) {
        super(p);
        this.showConsole = showConsole;

        // If verification hasn't been done, call the constructor that will check the files
        if(!isVerified) {
            new CheckInCommand(p, vf, null);
        }
        else {
            try {
                getComment();
                if(showConsole) {
                    toolWindow.focusOnConsole();
                }
                filesToCheckIn = vf;

                checkIn();
            } catch (Exception e) {
                Messages.showWarningDialog(e.getMessage(), "Check In Failed");
            }
        }
    }

    /**
     * Constructor used to take objects which the user wants to check in but have not yet verified whether the objects
     * are ready to be checked in. The verification process is done via the PropertiesCommand class
     * @param p The active IDEA project
     * @param vf The objects to be checked in
     * @param c The comment (can be null)
     */
    public CheckInCommand(Project p, VirtualFile[] vf, String c) {
        super(p);

        VirtualFile[] toCheckIn = removeInvalidFiles(vf);

        if(toCheckIn.length > 0) {
            try {
                // Check for comments
                if(c != null) {
                    comment = c;
                }
                else {
                    getComment();
                }

                filesToCheckIn = toCheckIn;

                checkIn();
            } catch (Exception e) {
                Messages.showWarningDialog(e.getMessage(), "Check In Failed");
            }
        }
        else {
            Messages.showInfoMessage(p, "No file(s) to check in", "Check In Message");
        }
    }

    /**
     * Gathers a comment from the user to place in the database along with the checked-in object(s)
     */
    private void getComment() throws Exception {
        
        /*CommentDialog dialog = new CommentDialog();
        dialog.setTitle("Check In Comments");
        dialog.showDialog();
        String c = dialog.getComment();*/

        String c = super.getComment("Check in file comments");

        if(c != null) {
            comment = c;
        }
        else {
            comment = "";
        }
    }

    /**
     * Runs the CLI command
     */
    private void checkIn(){
        //BufferedReader br;
        ArrayList<String> options = new ArrayList<String>();

        // Make sure all files are saved first
        saveFiles(filesToCheckIn);

        // build checkin command
        // ccm ci -c "comment"  <file> <file> <file>...
        options.add(myConfig.getPath());
        options.add("ci");
        options.add("-nc"); // Prevent comment prompt
        options.add("-c");
        //options.add("\"");
        options.add(comment);
        //options.add("\"");
        /* TODO: Code to activate later... this is to make sure the files are checked into the right state
        options.add("-state");
        options.add("integrate");
         */

        for(VirtualFile file : filesToCheckIn) {
            options.add(file.getPath());
        }

        String[] commands = options.toArray(new String[options.size()]);

        try {
            HashMap<String,String> envVars = new HashMap<String,String>();
            envVars.put("CCM_ADDR", myConfig.getCmEnvPath());
            
            toolWindow.appendConsole("Starting Check In Operation...");

            // Run command
            process = CommandHelper.runCommand(commands, envVars);

            //br = new BufferedReader(new InputStreamReader(p.getInputStream()));
            //new DisplayThread(messages).start();
            //new HelperThread(project, toolWindow, br, HelperThread.CHECKIN, files).start();

            new Consumer(project, process, true, true);
            //new HelperThread(this).start();
            VirtualFileManager.getInstance().refresh(false);

            //myConfig.updateSelectedFileProperties();
            //toolWindow.updateCurrentFileTab(myConfig.getSelectedFile());
            toolWindow.updateCurrentFileTab();


        } catch (Exception e) {
            new CMExceptionHandler(e);
        }

    }

    public VirtualFile[] removeInvalidFiles(VirtualFile[] files) {
        ArrayList<VirtualFile> goodFiles = new ArrayList<VirtualFile>();

        for (VirtualFile tmp : files) {
            boolean result = new PropertiesCommand(tmp, project).isCheckedOutToUser();
            if(result) {
                goodFiles.add(tmp);
            }
        }

        return goodFiles.toArray(new VirtualFile[goodFiles.size()]);
    }

    public void processErrors() {
        
    }

    public void warningOnParallel() {
        new ParallelHelper(project, parallelConflicts, this);
    }

    /**
     * Called by a conflict resolver if the first check in failed.
     * Simple for now... just creates a new CheckInCommand instance
     */
    public void retry() {
        new CheckInCommand(project, filesToCheckIn, comment);
    }

    /**
     * @see com.telelogic.integration.intellij.cli.AbstractCommand#toString()
     */
    public String toString() { return "Check In"; }

    private class Consumer extends AbstractOutputConsumer {
        String objectName;
        boolean inWarning = false;

        public Consumer(Project project, Process process, boolean console, boolean redirectedErrorStream){
            super(project, process, console, redirectedErrorStream);
        }

        protected void findErrors(String line) {

            // Warning: Object "<object>" cannot be checked in because release "<release>"
            // does not allow parallel checkin within the release.
            // Could not change status of object '<object>' to 'integrate'
            if(inWarning) {
                if(line.contains("does not allow parallel checkin within the release.")) {
                    parallelConflicts.add(objectName);
                    objectName = null;
                }
                else if(line.contains("Could not change status of object")) {
                    // We're at the end of the error
                    inWarning = false;
                }
            }
            else if(line.contains("Warning")) {

                if(line.contains("cannot be checked in because release")) {
                    inWarning = true;
                    String[] tokens = line.split("\"");
                    objectName = tokens[1];
                }
                else if(line.startsWith("Warning: Parallel branches exist for ")) {
                    String object = line;
                    object = object.replaceAll("Warning: Parallel branches exist for ","");
                    parallelConflicts.add(object);
                }
            }
        }

        protected void finished(ArrayList<String> result) {
            if(parallelConflicts.size() > 0) {
                warningOnParallel();
            }
            else {
                String message = "Check In Operation Complete\nSummary:\n" + getSummary(result);
                Messages.showInfoMessage(project, message, "Check In Summary");
            }
        }
    }
}
