// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VirtualFileManager;
import com.intellij.openapi.vcs.changes.VcsDirtyScopeManager;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.progress.Task;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.progress.EmptyProgressIndicator;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ModalityState;
import com.intellij.vcsUtil.VcsUtil;
import com.intellij.util.text.StringTokenizer;
import com.intellij.util.concurrency.SwingWorker;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMPlugin;
import com.telelogic.integration.intellij.cm.CMUtil;
import com.telelogic.integration.intellij.ui.dialogs.CommentDialog;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Arrays;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.awt.*;
import java.lang.reflect.Array;

/**
 * Handles Check-out functions
 * @author Luke Stahlman
 */
public class CheckOutCommand extends AbstractCommand {
    private VcsDirtyScopeManager mgr;
    private boolean showConsole = true;

    /**
     * Special method to bypass normal verification of objects before they're checked out. Used when a properties
     * command has already been run on the file (this currently happens when the "Check Out" button is used in the
     * current file tab
     * @param vf The objects to be checked out
     * @param p The active IDEA project
     * @param isVerified Whether the objects ready to be checked out
     * @param showConsole Whether the output of the check out command should be automatically displayed to the user
     */
    @SuppressWarnings({"SameParameterValue"})
    public CheckOutCommand(VirtualFile[] vf, Project p, boolean isVerified, boolean showConsole) throws VcsException {
        super(p);

        this.showConsole = showConsole;


        if(!isVerified) {
            new CheckOutCommand(vf, p);
        }
        else {
            mgr = VcsDirtyScopeManager.getInstance( project );
            VirtualFile[] toCheckOut = removeInvalidFiles(vf);
            if(toCheckOut.length > 0) {
                checkOut(vf,"");
            }
            else {
                Messages.showInfoMessage(p, "No file(s) to check out", "Check out message");
            }
        }
    }

    /**
     * Special method to bypass normal verification of objects before they're checked out. Used when a properties
     * command has already been run on the file (this currently happens when the "Check Out" button is used in the
     * current file tab
     * @param files The objects to be checked out
     * @param p The active IDEA project
     * @param isVerified Whether the objects ready to be checked out
     * @param showConsole Whether the output of the check out command should be automatically displayed to the user
     */
    @SuppressWarnings({"SameParameterValue"})
    public CheckOutCommand(Project p, boolean isVerified, boolean showConsole, VirtualFile... files) throws VcsException {
        super(p);

        this.showConsole = showConsole;

        if(!isVerified) {
            new CheckOutCommand(files, p);
        }
        else {
            mgr = VcsDirtyScopeManager.getInstance( project );

            VirtualFile[] toCheckOut = removeInvalidFiles(files);
            if(toCheckOut.length > 0) {
                checkOut(toCheckOut,"");
            }
            else {
                Messages.showInfoMessage(p, "No file(s) to check out", "Check out message");
            }

        }
    }

    /**
     * Constructor used to take objects which the user wants to check out but have not yet verified whether the objects
     * are ready to be checked out. The verification process is done via the PropertiesCommand class
     * @param files The objects to be checked in
     * @param p The active IDEA project
     */
    public CheckOutCommand(final VirtualFile[] files, Project p) throws VcsException {
        super(p);
        mgr         = VcsDirtyScopeManager.getInstance( project );
        final ArrayList<VirtualFile> filesToCheckOut = new ArrayList<VirtualFile>();

// If checked in, ask to check file out (or check out automatically if selected in the option tab)
        //boolean isCheckedIn = new PropertiesCommand(file, project).isCheckedIn();
        final boolean[] doCheckout = new boolean[]{false};
        boolean anyCheckedOutFiles = false;
        final int fileCount = files.length;

        final SwingWorker propertiesWorker = new SwingWorker() {
            public ArrayList<VirtualFile> construct() {
                VirtualFile[] toCheckOut = removeInvalidFiles(files);
                filesToCheckOut.addAll(Arrays.asList(toCheckOut));

                return filesToCheckOut;
            }
        };

        final Runnable propertiesRunner = new Runnable() {
            ProgressManager progress = ProgressManager.getInstance();
            ProgressIndicator indicator = null;
            boolean indicatorStarted = false;

            public void run() {
                startProgress();
                propertiesWorker.start();
                propertiesWorker.get();
                endProgress();
            }

            private void startProgress() {
                if(progress.hasProgressIndicator()) {
                    indicator = progress.getProgressIndicator();
                    indicator.setIndeterminate(true);
                    indicator.setText("Getting information for " + fileCount + " objects");
                    if(!indicator.isRunning()) {
                        indicatorStarted = true;
                        indicator.start();
                    }
                }
            }

            private void endProgress() {
                if(indicator != null && !indicatorStarted) {
                    indicator.stop();
                }
            }
        };

        ProgressManager.getInstance().runProcessWithProgressSynchronously(propertiesRunner, "Gathering object information",
                false, project);

        if (filesToCheckOut.size() > 0 ) {
            try {
                new CheckOutCommand(filesToCheckOut.toArray(VirtualFile.EMPTY_ARRAY), project, true, true);
            } catch (VcsException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
            VirtualFileManager.getInstance().refresh(false);
        }

        // Update IntelliJ's file listing
        /*for (VirtualFile file : filesToCheckOut) {
            mgr.fileDirty(file);
        }*/
    }

    /**
     * Empty method used as a placeholder for future functionality
     */
    public void verifyFiles() {

    }

    /**
     * Same as the special constructor, as files are assumed ready for check out.
     * @param files The objects to be checked out
     */
    public void autoCheckOut(VirtualFile[] files) {
        //Process p;
        // BufferedReader br;
        ArrayList<String> options = new ArrayList<String>();

        // build checkin command
        // ccm co -c "comment"  <file> <file> <file>...
        options.add(myConfig.getPath());
        options.add("co");
        options.add("-task");
        options.add(myConfig.getDefaultTask());
        for(VirtualFile file : files) {
            options.add(file.getPath());
        }

        String[] commands = options.toArray(new String[options.size()]);

        try {
            toolWindow.appendConsole("Automatic Check Out Operation...");
            // Run command
            process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
            //br = new BufferedReader(new InputStreamReader(p.getInputStream()));

            //new HelperThread(project, toolWindow, br, HelperThread.CHECKOUT).start();
            new HelperThread(this).start();

        } catch (Exception e) { new CMExceptionHandler(e); }
    }

    /**
     * Runs the CLI command
     * @param files The object(s) to be checked out
     * @param comment Check out comments from user
     * @throws com.intellij.openapi.vcs.VcsException If anything goes wrong
     */
    private synchronized void checkOut2(final VirtualFile[] files, String comment) throws VcsException {
        //Process p;
        //BufferedReader br;
        ArrayList<String> options = new ArrayList<String>();

        // build checkin command
        // ccm co -c "comment"  <file> <file> <file>...
        options.add(myConfig.getPath());
        options.add("co");
        options.add("-task");
        options.add(myConfig.getDefaultTask());

        if(comment != null) {
            options.add("-c");
            options.add("\"");
            options.add(comment);
            options.add("\"");
        }

        for(VirtualFile file : files) {
            options.add(file.getPath());
        }

        final String[] commands = options.toArray(new String[options.size()]);
        final int numFiles = files.length;
        toolWindow.appendConsole("Starting Check Out Operation...");


        final Task task = new Task.Modal(project, "Running Check Out Operation", false) {

            ArrayList<String> parallelObjects = new ArrayList<String>(); // Stores object names of files with parallel conflicts
            ArrayList<String> warnings = new ArrayList<String>();

            public void run(ProgressIndicator indicator) {

                // Set up the incrementer for the progress indicator
                double incrementer = 1 / (double)numFiles;
                indicator.setIndeterminate(false);
                indicator.setText("Checking out " + Integer.toString(numFiles) + " objects");
                indicator.setFraction(0);

                try {
                    process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
                } catch (IOException e) {
                    String message = e.getMessage();
                    VcsUtil.showErrorMessage(myProject, "A filesystem error occured on checkout:\n" + message, "Check Out Error");
                }

                BufferedReader output = new BufferedReader(new InputStreamReader(process.getInputStream()));
                String line;

                try {
                    boolean encounteredParallel = false;

                    while((line = output.readLine()) != null) {

                        // Display the message in the progress indicator and the console
                        CMPlugin.getInstance(myProject).sendToConsole("\t" + line);
                        indicator.setText2(line);

                        // Evaluate code
                        if(line.startsWith("Checked out new version")) {
                            // If a previous message was a parallel warning, catch the object that went parallel
                            if(encounteredParallel) {
                                //Checked out new version 'CMGroup.java,3.1.1' from 'CMGroup.java,3'
                                // This should result in 4 values.  We want the second
                                String[] tokens = line.split("\'");
                                parallelObjects.add(tokens[1]);

                                // Parallel data captured.  Reset the toggle
                                encounteredParallel = false;
                            }

                            // Update the progress indicator
                            indicator.setFraction(indicator.getFraction() + incrementer);
                        }
                        // If there is a warning
                        else if (line.startsWith("Warning:")) {
                            warnings.add(line);
                        }
                        else if (line.startsWith("Parallel versions exist")) {
                            encounteredParallel = true;
                        }
                    }
                } catch (IOException e) {
                    VcsUtil.showErrorMessage(myProject, "Unknown file system error caught during check out: \n\n"
                            + e.getMessage(), "Uknown error");
                }

                // Wait until the process finishes. If it sends a non-zero exit code, something went wrong
                try {
                    if(process.waitFor() != 0) {
                        VcsUtil.showErrorMessage(myProject, "Check Out failed on one or more files", "Check Out Error");
                    }
                } catch (Exception e) {
                    // Catch an interrupted exception
                    VcsUtil.showErrorMessage(myProject, "Check Out failed due to the following issue: \n\n" + e.getMessage(), "Check Out Error");
                }
                indicator.stop();

                if(warnings.size() > 0) {
                    VcsUtil.showStatusMessage(project, "Synergy: Check out encountered warnings");
                    String message = "The following warnings were caught during check out:\n";
                    for(String warning : warnings) {
                        message += "\n\t" + warning;
                    }

                    VcsUtil.showErrorMessage(myProject, message, "Check Out Error");
                }
                if(parallelObjects.size() > 0) {
                    String message = "The following objects have parallel conflicts: \n\n";
                    for(String object : parallelObjects) {
                        message += object + "\n";
                    }
                    message += "\nPlease be sure to resolve these conflicts before check in";

                    final String text = message;

                    VcsUtil.showErrorMessage(myProject, text, "Parallel Warning");
                }

                for(VirtualFile file : files) {

                    VcsUtil.markFileAsDirty(project, file);
                }
                //VirtualFileManager.getInstance().refresh(false);

            }
        };

        if(ApplicationManager.getApplication().isDispatchThread()) {
            task.queue();
        }
        else {
            EventQueue.invokeLater(new Runnable() {
                public void run() {
                    task.queue();
                }
            });
        }
        //ProgressManager.getInstance().run(task);

       /*EventQueue.invokeLater(new Runnable() {
            public void run() {
                task.queue();
                //ProgressManager.getInstance().run(task);
            }
        });*/
        //task.queue();
    }

    /**
     * Runs the CLI command
     * @param files The object(s) to be checked out
     * @param comment Check out comments from user
     * @throws com.intellij.openapi.vcs.VcsException If anything goes wrong
     */
    private synchronized void checkOut(final VirtualFile[] files, String comment) throws VcsException {
        //Process p;
        //BufferedReader br;
        if(!new Tasks(project).ensureTaskSet()) {
            throw new VcsException("Error: No task is set.");
        }

        ArrayList<String> options = new ArrayList<String>();

        // build checkin command
        // ccm co -c "comment"  <file> <file> <file>...
        options.add(myConfig.getPath());
        options.add("co");
        options.add("-task");
        options.add(myConfig.getDefaultTask());

        if(comment != null) {
            options.add("-c");
            options.add("\"");
            options.add(comment);
            options.add("\"");
        }

        for(VirtualFile file : files) {
            options.add(file.getPath());
        }

        final String[] commands = options.toArray(new String[options.size()]);
        final int numFiles = files.length;
        toolWindow.appendConsole("Starting Check Out Operation...");


        final Runnable task = new CMRunnable(1/(double)numFiles) {

            ArrayList<String>   parallelObjects = new ArrayList<String>(); // Stores object names of files with parallel conflicts
            ArrayList<String>   warnings        = new ArrayList<String>();

            public void run() {
                setText("Checking out " + Integer.toString(numFiles) + " objects");

                try {
                    process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
                } catch (IOException e) {
                    String message = e.getMessage();
                    VcsUtil.showErrorMessage(project, "A filesystem error occured on checkout:\n" + message, "Check Out Error");
                }

                BufferedReader output = new BufferedReader(new InputStreamReader(process.getInputStream()));
                String line;

                try {
                    boolean encounteredParallel = false;

                    while((line = output.readLine()) != null) {

                        // Display the message in the progress indicator and the console
                        CMPlugin.getInstance(project).sendToConsole("\t" + line);
                        setText2(line);

                        // Evaluate code
                        if(line.startsWith("Checked out new version")) {
                            // If a previous message was a parallel warning, catch the object that went parallel
                            if(encounteredParallel) {
                                //Checked out new version 'CMGroup.java,3.1.1' from 'CMGroup.java,3'
                                // This should result in 4 values.  We want the second
                                String[] tokens = line.split("\'");
                                parallelObjects.add(tokens[1]);

                                // Parallel data captured.  Reset the toggle
                                encounteredParallel = false;
                            }

                            // Update the progress indicator
                            increment();
                        }
                        // If there is a warning
                        else if (line.startsWith("Warning:")) {
                            warnings.add(line);
                        }
                        else if (line.startsWith("Parallel versions exist")) {
                            encounteredParallel = true;
                        }
                    }
                } catch (IOException e) {
                    VcsUtil.showErrorMessage(project, "Unknown file system error caught during check out: \n\n"
                            + e.getMessage(), "Uknown error");
                }

                // Wait until the process finishes. If it sends a non-zero exit code, something went wrong
                try {
                    if(process.waitFor() != 0) {
                        VcsUtil.showErrorMessage(project, "Check Out failed on one or more files", "Check Out Error");
                    }
                } catch (Exception e) {
                    // Catch an interrupted exception
                    VcsUtil.showErrorMessage(project, "Check Out failed due to the following issue: \n\n" + e.getMessage(), "Check Out Error");
                }
                stopIndicator();

                if(warnings.size() > 0) {
                    VcsUtil.showStatusMessage(project, "Synergy: Check out encountered warnings");
                    String message = "The following warnings were caught during check out:\n";
                    for(String warning : warnings) {
                        message += "\n\t" + warning;
                    }

                    VcsUtil.showErrorMessage(project, message, "Check Out Error");
                }
                if(parallelObjects.size() > 0) {
                    String message = "The following objects have parallel conflicts: \n\n";
                    for(String object : parallelObjects) {
                        message += object + "\n";
                    }
                    message += "\nPlease be sure to resolve these conflicts before check in";

                    final String text = message;

                    VcsUtil.showErrorMessage(project, text, "Parallel Warning");
                }

                for(VirtualFile file : files) {
                    file.refresh(false, false);
                    //VcsUtil.markFileAsDirty(project, file);
                }
                //VirtualFileManager.getInstance().refresh(false);
                toolWindow.updateCurrentFileTab();
            }
        };

        if(ApplicationManager.getApplication().isDispatchThread()) {

            if(!ProgressManager.getInstance().runProcessWithProgressSynchronously(task, "Running Check Out Operation", false, project)) {
                throw new VcsException("Check out operation aborted");
            }
        }
        else {
            ApplicationManager.getApplication().invokeLater(task);
        }
    }

    /**
     * Gathers a comment from the user to place in the database along with the checked-out object(s)
     * This method is not currently used
     * @return The comments from the user
     */
    public String getComment() {

        CommentDialog dialog = new CommentDialog();
        dialog.setTitle("Check Out Comments");
        dialog.showDialog();
        return dialog.getComment();

    }

    public VirtualFile[] removeInvalidFiles(VirtualFile[] files) {
        ArrayList<VirtualFile> goodFiles = new ArrayList<VirtualFile>();

        for (VirtualFile tmp : files) {
            boolean result = new PropertiesCommand(tmp, project).isCheckedIn();
            if(result) {
                goodFiles.add(tmp);
            }
        }

        return goodFiles.toArray(new VirtualFile[goodFiles.size()]);
    }

    /**
     * @see com.telelogic.integration.intellij.cli.AbstractCommand#toString()
     */
    public String toString() { return "Check Out"; }
}
