// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMConstants;
import com.telelogic.integration.intellij.cm.CMPlugin;
import com.telelogic.integration.intellij.ui.CMToolWindow;
import com.telelogic.integration.intellij.cli.helpers.ReaderThread;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.ToolWindow;

import java.util.Map;
import java.util.HashMap;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * Handles CLI command execution. It is also meant to parse certain buffered reader output
 */
public class CommandHelper {

    /**
     * Default constructor
     */
    private CommandHelper() {}

    /**
     * Used to run commands that don't require a Synergy session to be started.
     * @param args Command details
     * @return The executed command
     * @throws IOException If there is a problem originating from the ProcessBuilder operation
     */
    public static Process runCommand(String args[]) throws IOException {

        ProcessBuilder pb = new ProcessBuilder(args);
        pb.redirectErrorStream(true);
        return pb.start();
    }

    /**
     * Used to run commands that don't require a Synergy session to be started.
     * @param args Command details
     * @param envVars Environment variables to pass
     * @return The executed command
     * @throws IOException If there is a problem originating from the ProcessBuilder operation
     */
    public static Process runCommand(String args[], HashMap<String,String> envVars) throws IOException {

        ProcessBuilder pb = new ProcessBuilder(args);
        pb.redirectErrorStream(true);
        Map<String,String> env = pb.environment();
        env.putAll(envVars);
        return pb.start();
    }

    /**
     * Used to run commands requiring a Synergy session to be started. The session is linked to the command via ccmAddr.
     * @param args Command details
     * @param ccmAddr CCM_ADDR environment variable
     * @return The executed command
     * @throws IOException If there is a problem originating from the ProcessBuilder operation
     */
    public static Process runCommand(String args[], String ccmAddr) throws IOException {

        ProcessBuilder pb = new ProcessBuilder(args);
        Map<String, String> env = pb.environment();
        env.put("CCM_ADDR", ccmAddr);

        return pb.start();
    }

    /**
     * Iterates through the BufferedReader passed to it and extracts property values.
     * @param br The buffered reader to be processed
     * @return The result of the parsing process
     */
    public static HashMap<String,String> parseProperties(BufferedReader br) {

        String l;
        final HashMap<String,String> properties= new HashMap<String,String>();

        try {
            while ((l = br.readLine()) != null) {
                // If the first line begins with an error, this object does not exist in Synergy
                if(l.startsWith(CMConstants.DOES_NOT_EXIST) && (properties.size() < 1)) {
                    return null;
                }
                String[] keyval = l.split(":",2);

                // This helps remove the chance that lines without values we want are ignored
                if(keyval.length > 1) {
                    properties.put(keyval[0].trim(),keyval[1].trim());
                }
            }
        } catch (IOException e) { new CMExceptionHandler(e); }

        return properties;

    }

    /**
     * Special parser meant to handle the more complex output of "ccm task -show i". Iterates through the BufferedReader
     * passed to it and extracts property values.
     * @param br The buffered reader to be processed
     * @return The result of the parsing process
     */
    public static HashMap<String,String> parseTask(BufferedReader br) {

        String l;
        final HashMap<String,String> properties= new HashMap<String,String>();
        boolean processOutput = false;

        try {
            while ((l = br.readLine()) != null) {
                if(!processOutput && l.contains("Waiting, CM Synergy command in progress")) {
                    processOutput = true;
                    continue;
                }
                else {
                    processOutput = true;
                }
                // Check if line is empty
                if(!l.trim().equals("")) {

                    // Task-specific parsing
                    if(l.contains("Task Description:")) {

                        String descriptionText = "";
                        String descriptionLine;

                        while ((descriptionLine = br.readLine()) != null) {
                            if(!descriptionLine.contains("Status Log:")) {
                                descriptionText += descriptionLine + "\n";
                            }
                            else {
                                String logText = "";
                                String logLine;

                                while((logLine = br.readLine()) != null) {
                                    logText += logLine + "\n";
                                }

                                properties.put("log",logText);
                            }
                        }
                        properties.put("Description",descriptionText);
                    }
                    else {

                        String[] keyval = l.split(":",2);

                        // This helps remove the chance that lines without values we want are ignored
                        if(keyval.length > 1) {
                            if(keyval[0].contains("Associated Change")) {
                                keyval[0] = "CR";
                            }

                            properties.put(keyval[0].trim(),keyval[1].trim());

                        }
                    }
                }

            }
        } catch (IOException e) { new CMExceptionHandler(e); }

        return properties;

    }

    /**
     * This goes through the input stream and error stream.  Any information is outputted to the CMToolWindow's console,
     * while any error will trigger the termination of the process
     * @param process
     */
    public static void processStartCommand(Project project, Process process) {
        BufferedReader output = new BufferedReader(new InputStreamReader(process.getInputStream()));
        BufferedReader errors = new BufferedReader(new InputStreamReader(process.getErrorStream()));

        CMToolWindow tw = CMPlugin.getInstance(project).toolWindow;
        tw.startProgressBar();

        new ReaderThread(project, process, output, "Output").start();
        new ReaderThread(project, process, errors, "Errors").start();

        // Look through the output lines and check for errors.  Send output to toolwindow

    }

}

