// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.vfs.VirtualFileManager;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMPlugin;
import com.telelogic.integration.intellij.cli.helpers.ParallelHelper;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * Completes the desired Synergy task. When a task is completed, all associated objects are checked in.
 * @author Luke Stahlman
 */
public class CompleteTaskCommand extends AbstractCommand {

    private ArrayList<String> options = new ArrayList<String>();
    private String task;
    private String comment;
    private ArrayList<String> parallelConflicts = new ArrayList<String>();
    private CompleteTaskCommand command;

    /**
     * Primary constructor. This gathers the information needed to complete the task, then runs the command via
     * complete()
     * @param taskID    The task to be completed
     * @param p         The active IDEA project
     * @param comment   Comment to be placed in the Synergy DB along with the completed task
     */
    public CompleteTaskCommand(String taskID, Project p, String comment) {
        super(p);
        task = taskID;
        this.comment = comment;

        toolWindow.focusOnConsole();
        complete();
    }

    /**
     * Executes the "ccm task -complete" command using the information gathered by the constructor
     */
    private void complete() {
        // Task 112: Save all documents in IDEA
        ApplicationManager.getApplication().saveAll();

        //  Build command
        options.add(myConfig.getPath());
        options.add("task");
        options.add("-complete");
        options.add("-comment");
        options.add(comment);
        options.add("-y");
        options.add(task);

        String[] commands = options.toArray(new String[options.size()]);

        try {
            HashMap<String,String> envVars = new HashMap<String,String>();
            envVars.put("CCM_ADDR", myConfig.getCmEnvPath());

            toolWindow.appendConsole("Starting Complete Task Operation...");
            // Run command
            process = CommandHelper.runCommand(commands, envVars);

            //br = new BufferedReader(new InputStreamReader(pr.getInputStream()));
            //new HelperThread(project, toolWindow, br, HelperThread.COMPLETETASK).start();
            new Consumer(project, process, true, true);

            //new HelperThread(this).start();

        } catch (Exception e) { new CMExceptionHandler(e); }

        // If the task being completed was set as the current task, then it needs to be cleared
        if(Tasks.parseTaskID(myConfig.getDefaultTask()).equals(task)) {
            Tasks.setCurrentTask("None", project);
        }
    }

    private void failedOnParallel() {
        new ParallelHelper(project, parallelConflicts, this);
    }

    private void warningOnParallel() {
        String message = "Parallel warnings were encountered on task completion for the following objects:\n";
        for(String object : parallelConflicts) {
            message += "\n" + object;
        }
        Messages.showInfoMessage(project, message, "Complete Task Message");
    }

    public void retry() {
        new CompleteTaskCommand(task, project, comment);
    }

    /**
     * @see com.telelogic.integration.intellij.cli.AbstractCommand#toString()
     */
    public String toString() { return "Complete Task"; }

    private class Consumer extends AbstractOutputConsumer {
        String objectName;
        boolean inError = false;

        public Consumer(Project project, Process process, boolean console, boolean redirectedErrorStream){
            super(project, process, console, redirectedErrorStream);
        }

        protected void findErrors(String line) {
            // Warning: Object "<object>" cannot be checked in because release "<release>"
            // does not allow parallel checkin within the release.
            // Could not change status of object '<object>' to 'integrate'
            if(inError) {
                if(line.contains("does not allow parallel checkin within the release.")) {
                    parallelConflicts.add(objectName);
                    objectName = null;
                }
                else if(line.contains("Could not change status of object")) {
                    // We're at the end of the error
                    inError = false;
                }
            }
            if(line.contains("Warning")) {

                if(line.contains("cannot be checked in because release")) {
                    inError = true;
                    String[] tokens = line.split("\"");
                    objectName = tokens[1];
                }
                else if(line.startsWith("Warning: Parallel branches exist for ")) {
                    String object = line;
                    object = object.replaceAll("Warning: Parallel branches exist for ","");
                    parallelConflicts.add(object);
                }
            }
            // Check for task completion failure
            if(line.startsWith("Complete") && line.endsWith(" failed.")) {
                // The task failed
                failed = true;
            }
        }

        protected void finished(ArrayList<String> result) {
            if(failed) {
                if(parallelConflicts.size() > 0) {
                    // The failure probably occurred due to parallel conflicts.  We need to resolve them
                    failedOnParallel();
                    return;
                }
            }
            else if(parallelConflicts.size() > 0) {
                warningOnParallel();
            }
            else {
                String message = "Complete Task Operation Complete\nSummary:\n" + getSummary(result);
                Messages.showInfoMessage(project, message, "Check In Summary");
            }
            VirtualFileManager.getInstance().refresh(false);
            toolWindow.updateCurrentFileTab();
        }
    }
}
