// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.telelogic.integration.intellij.cli.AbstractCommand;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vcs.FilePath;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.io.File;

/**
 * Handles delete operations in Synergy
 * @author Luke Stahlman
 */
public class Delete extends AbstractCommand {

    private String              taskID;
    private ArrayList<String> filePaths;

    /**
     * Main Constructor
     * @param file The object to be deleted
     * @param p The active IntelliJ project
     * @param task The task this operation should be associated with
     */
    public Delete(VirtualFile file, Project p, String task) {
        super(p);
        filePaths = new ArrayList<String>();

        if(task != null) {
            taskID = task;
        }
        else {
            taskID = myConfig.getDefaultTask();
        }

        if(new PropertiesCommand(file, project).isCheckedOutToUser()) {
            filePaths.add(file.getPath());
        }


        execute();
    }

    public Delete(List<FilePath> files, Project p, String task) {
        super(p);
        filePaths = new ArrayList<String>();

        // Go through all the files.  First check if they're under Synergy control
        for(FilePath file : files){
            if(new PropertiesCommand(file.getVirtualFile(), project).isCheckedOutToUser()) {
                filePaths.add(file.getPath());
            }
        }

        if(task != null) {
            taskID = task;
        }
        else {
            taskID = myConfig.getDefaultTask();
        }

        execute();
    }


    private void execute() {
        // Set the working directory path to the IntelliJ project base
        String workingDirectory = project.getBaseDir().getPath();

        // If there aren't any files to process, stop
        if(filePaths.size() < 1) {
            return;
        }

        ArrayList<String> options = new ArrayList<String>();

        options.add(myConfig.getPath());
        options.add("delete");
        options.add("-replace");

        for(String filePath : filePaths) {
            // Remove the base directory so the path is relative to the project.
            String expression = workingDirectory + "/";
            String relativePath = filePath.replaceFirst(expression, "");
            options.add(relativePath);
        }

        String[] commands = options.toArray(new String[options.size()]);

        try {
            toolWindow.appendConsole("Running delete operation...");
            // Run command
            //process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            ProcessBuilder pb = new ProcessBuilder(commands);
            
            // Set the working directory
            pb = pb.directory(new File(workingDirectory));

            Map<String, String> env = pb.environment();
            env.put("CCM_ADDR", myConfig.getCmEnvPath());

            process = pb.start();

            new HelperThread(this).start();

        } catch (Exception e) { new CMExceptionHandler(e); }
    }

    /** @see com.telelogic.integration.intellij.cli.AbstractCommand#toString() */
    public String toString() {
        return "Delete";
    }
}
