// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VirtualFileManager;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.fileEditor.FileDocumentManager;
import com.intellij.openapi.project.Project;
import com.telelogic.integration.intellij.ui.CMToolWindow;
import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;

import javax.swing.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;

/**
 * Processes text within buffered readers sent to it by CLI command classes  and displays the desired output in a dialog
 * (if required).
 */
public class HelperThread extends Thread {
    private CMToolWindow    toolWindow;
    private BufferedReader  messages;
    private BufferedReader  errors;
    private int             commandType;
    private VirtualFile[]   files;
    private Project         project;
    private AbstractCommand command;
    private boolean         showSuccessMessage = true;

    private static final int CHECKIN         = 1;
    private static final int CHECKOUT        = 2;
    private static final int COMPLETETASK    = 3;
    private static final int PROPERTIES      = 4;
    private static final int UPDATE          = 5;
    private static final int UNDOCHECKOUT    = 6;
    private static final int ADD             = 7;
    private static final int USE             = 8;

    /**
     * Constructor used by classes which extend AbstractCommand.
     * @param command The AbstractCommand instance asking for HelperThread processing
     */
    public HelperThread(AbstractCommand command)  {
        this.command    = command;
        toolWindow      = command.toolWindow;
        project         = command.project;
        Process process = command.process;
        messages        = new BufferedReader(new InputStreamReader(process.getInputStream()));
        errors          = new BufferedReader(new InputStreamReader(process.getErrorStream()));
        files           = command.virtualFiles;

        commandType = determineType();
    }

    /**
     * Constructor used by classes which don't extend AbstractCommand. In their case, all information required to
     * run HelperThread operations must be explicitly passed to this constructor.
     * @param p The active IDEA project
     * @param tw The project tool window
     * @param reader The output to be processed
     * @param type The command type
     * @throws Exception For invalid command types (the class doesn't know what to do for unknown commands)
     * @deprecated
     */
    public HelperThread(Project p, CMToolWindow tw, BufferedReader reader, int type) throws Exception {
        toolWindow      = tw;
        messages        = reader;
        this.project    = p;

        if(checkType(type)) { commandType = type; }
        else { throw new Exception("Invalid command type"); }
    }

    /**
     * Constructor used by classes which don't extend AbstractCommand. In their case, all information required to
     * run HelperThread operations must be explicitly passed to this constructor. This constructor also accepts
     * an array of VirtualFile objects for command-specific processing
     * @param p The active IDEA project
     * @param tw The project tool window
     * @param reader The output to be processed
     * @param type The command type
     * @param f The array of virtual files to be included in HelperThread operations
     * @throws Exception For invalid command types (the class doesn't know what to do for unknown commands)
     * @deprecated
     */
    public HelperThread(Project p, CMToolWindow tw, BufferedReader reader, int type, VirtualFile[] f) throws Exception {
        toolWindow  = tw;
        messages = reader;
        files       = f;
        project     = p;

        if(checkType(type)) { commandType = type; }
        else { throw new Exception("Invalid command type"); }
    }

    /**
     * Validate whether the type passed in one of the constructors is recognized by this class
     * @param type The command type to be validated
     * @return Whether it's a valid command type
     */
    private boolean checkType(int type) {
        switch(type) {
            case CHECKIN:
            case CHECKOUT:
            case COMPLETETASK:
            case PROPERTIES:
            case UPDATE:
            case UNDOCHECKOUT:
            case ADD:
            case USE:
                               return true;
            default:           return false;
        }
    }

    /**
     * The main method executed by the start() method. This does the bulk of all HelperThread processing.
     */
    public void run() {
        String l;
        final ArrayList<String> message= new ArrayList<String>();
        boolean warning = false;
        boolean processOutput = false;
        String warningType = "";
        String warningMessage = "";


        try {
            while ((l = messages.readLine()) != null) {

                /**
                 * This section of code is meant to look at the first line of the buffer and determine whether it should
                 * be displayed. If a command takes a long time to execute, Synergy will send a message to the buffer
                 * stating it's waiting.  This is not relevant to the plugin, therefore it should be ignored. Once it
                 * is ignored, the rest of the buffer should be processed, even if the output includes another "waiting"
                 * message, as it may be part of a description (eg when this plugin is being developed under Synergy
                 * control. This is why both if and else statements include "processOutput = true"
                 */
                if(!processOutput && l.contains("Waiting, CM Synergy command in progress")) {
                    processOutput = true;
                    continue;
                }
                else {
                    processOutput = true;
                }

                // Replace colons with tabs for properties display
                if(commandType == PROPERTIES) {
                    l = l.replace(" : ", "\t");
                }
                if(l.contains("Warning:")) {
                    if(l.contains("Parallel")) {
                        // Invoke parallel warning
                        warning = true;
                        warningType = "Parallel Conflict";
                        warningMessage = l;
                    }
                }

                final String line = l;

                SwingUtilities.invokeLater(new Runnable() {
                    /**
                     * Add text to the console. This must be done within a thread or IDEA will have problems.
                     */
                    public void run() {
                        toolWindow.appendConsole(line);
                    }
                });
                message.add(line);
            }
        } catch (IOException e) { new CMExceptionHandler(e); }

        if(commandType == CHECKIN) {
            if(warning) {
                showWarningMessage(warningType, warningMessage);
            }
        }

        String formattedMessage = formatMessage(message.toArray(new String[message.size()]));
        if(showSuccessMessage) {
            showMessage(formattedMessage);
        }

        /* This checks if the error buffer exists, and if so invokes displayErrors(). This code is executed even if
         * an error message does not exist. It resides in this location because there is an issue with reading the
         * error buffer during checkout operations involving more than 2 files. */
        if(errors != null) {
            displayErrors();
        }
        
        // Refresh the files' statuses in IDEA
        // If individual files were modified, refresh those.  Otherwise, refresh all project files
        if(files != null) {
            CMConfiguration config = CMConfiguration.getInstance(project);
            for (VirtualFile file : files) {
                file.refresh(true, false);

                if(file == config.getSelectedFile()) {
                    config.updateSelectedFileProperties();
                     toolWindow.updateCurrentFileTab(file);
                }
            }
        }
        else {
            VirtualFileManager.getInstance().refresh(true);
        }

        yield();
    }

    /**
     * Adds command-specific formatting to the message to be shown once processing is complete
     * @param text The current message text
     * @return The message text after formatting
     */
    private String formatMessage(String[] text) {

        String formattedMessage = "";

        if (text.length > 0) {
            switch(commandType) {
                case CHECKIN:
                                        formattedMessage = "Check In Operation Complete\nSummary:\n";
                                        break;
                case CHECKOUT:
                                        formattedMessage = "Check Out Operation Complete\n";
                                        break;
                case COMPLETETASK:
                                        formattedMessage = "Complete Task Operation Complete\nSummary:\n";
                                        break;
                case UPDATE:
                                        //formattedMessage = "Update Operation Complete\nSummary:\n";
                                        break;
                case UNDOCHECKOUT:
                                        formattedMessage = "Undo Check Out Complete\n";
                                        break;
                case ADD:               formattedMessage = "File Added to Synergy\n";
                                        break;
                case USE:               formattedMessage = "Use Version Complete\n";
                                        break;
                default:                break;
            }

            boolean atSummary = false;
            for (String aText : text) {
                if(commandType == UPDATE && aText.matches(".*Update for.*")) {
                    formattedMessage = formattedMessage.concat(aText);
                    break;
                }
                /*if(commandType == UPDATE && aText.matches(".*Update complete.*")) {
                    break;
                } */
                else if (atSummary || commandType == PROPERTIES) {
                    formattedMessage = formattedMessage.concat(aText);
                    formattedMessage = formattedMessage.concat("\n");
                } else {
                    if (aText.matches(".*Summary:.*")) {
                        atSummary = true;
                    }
                }
            }
        }
        return formattedMessage;
    }

    /**
     * Displays the completion message dialog.
     * @param message The message to be shown
     */
    private void showMessage(String message) {
        if(commandType != PROPERTIES && commandType != USE && commandType != ADD && commandType != CHECKIN) {
            message = message.concat("Please check the console window for more information.");
        }
        final String msg = message;

        SwingUtilities.invokeLater(new Runnable() {
            /**
             * Show the message dialog. This must be done within a thread or IDEA will have problems.
             */
            public void run() {

                Messages.showMessageDialog(msg, "Synergy Message", Messages.getInformationIcon());
            }

        });
    }

    /**
     * Displays any warnings processed by the thread
     * @param warningType The type of warning, which is displayed as the dialog's title
     * @param message The warning message
     */
    private void showWarningMessage(String warningType, String message) {
        if(commandType != PROPERTIES) {
            message = message.concat("\n Please check the console window for more information.");
        }
        final String msg = message;
        final String title = warningType;

        SwingUtilities.invokeLater(new Runnable() {

            /**
             * Show the message dialog. This must be done within a thread or IDEA will have problems.
             */
            public void run() {
                Messages.showMessageDialog(msg, title, Messages.getWarningIcon());
            }

        });
    }

    /**
     * Checks error stream for any messages.  If the messages are serious, the errors are displayed to the user. At
     * this time, all errors are considered serious and the operation is assumed to have failed.
     */
    private void displayErrors() {
        String l;
        String msg = "";
        boolean errorsExist = false;

        try {
            while((l = errors.readLine()) != null) {
                if(!l.equals("")) {
                    final String line = l;
                    errorsExist = true;
                    msg += line + "\n";

                    SwingUtilities.invokeLater(new Runnable() {
                        /**
                         * Add text to the console. This must be done within a thread or IDEA will have problems.
                         */
                        public void run() {
                            toolWindow.appendConsole(line);

                        }
                    });
                }
            }
        } catch (Exception e) {
            new CMExceptionHandler(e);
        }
        if(errorsExist) {
            final String message    = msg;
            showSuccessMessage      = false;
            SwingUtilities.invokeLater(new Runnable() {
                    /**
                     * Show the message dialog. This must be done within a thread or IDEA will have problems.
                     */
                public void run() {
                    Messages.showMessageDialog(message + "Please check the console for more information.",
                            "Synergy Error", Messages.getInformationIcon());
                }

            });
        }
    }

    /**
     * This determines which AbstractCommand subclass called the HelperThread and assigns the corresponding command type
     * @return The command type
     */
    private int determineType() {
        if(command instanceof AddCommand) {
            return ADD;
        }
        else if(command instanceof CheckInCommand) {
            return CHECKIN;
        }
        else if(command instanceof CheckOutCommand) {
            return CHECKOUT;
        }
        else if(command instanceof CompleteTaskCommand) {
            return COMPLETETASK;
        }
        else if(command instanceof PropertiesCommand) {
            return PROPERTIES;
        }
        else if(command instanceof UndoCheckoutCommand) {
            return UNDOCHECKOUT;
        }
        else if(command instanceof UseVersionCommand) {
            return USE;
        }
        return 0;
    }
}
