// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vcs.FilePath;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vcs.history.VcsFileRevision;
import com.intellij.vcsUtil.VcsUtil;
import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMConstants;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.*;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

/**
 * Handles history commands
 * @author Luke Stahlman
 */
public class History {

    /**
     * Default constructor
     */
    public void History(){ }

    /**
     * Runs the "ccm history" command to gather general history information about an object. The returned ArrayList is
     * the raw output from the CLI
     * @param project The active IDEA project
     * @param file The file used in the history command
     * @return Raw buffered reader output
     */
    public ArrayList getHistory(Project project, VirtualFile file) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);
        ArrayList<String> results = new ArrayList<String>();

        Process p;
        BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("history");
        options.add("-f");
        options.add("%objectname");
        options.add("%status");
        options.add("%owner");
        options.add("%task");
        options.add(file.getPath());

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line;
            while ((line = output.readLine()) != null) {
                results.add(line);
            }
        } catch (IOException e) { new CMExceptionHandler(e); }

        return results;
    }

    /**
     * Gets a specific history object (instead of all revision data) for a file in the project.  If no object name is
     * specified, use the one corresponding to the object in the project work area
     * @param project The IDEA project
     * @param virtualFile The virtual file
     * @param objectName 
     * @return The desired history object
     */
    public static HistoryObject getHistoryObject(Project project, VirtualFile virtualFile, String objectName) {
        LinkedHashMap<History.HistoryObject.Key, History.HistoryObject> historyData;
        HistoryObject historyObject;
        FilePath path = VcsUtil.getFilePath(virtualFile.getPath());

        // If no objectName was specified, assign it with the objectName corresponding to the file in the work area
        if(objectName == null) {
            objectName = new PropertiesCommand(virtualFile, project).get("%objectname", path.getPath());
        }

        // Get a list of revisions for this object
        historyData = History.getRevisionData(project, path);

        // Get the specific object
        historyObject = historyData.get(new History.HistoryObject.Key(objectName));

        return historyObject;
    }



    /**
     * Parses the history output and returns a list of object versions.
     * @param project The active IDEA project
     * @param file The file involved in the history command
     * @return Output format: {"Object name",{"status","owner","task"}}
     */
    public static HashMap<String,HashMap<String,String>> getVersions(Project project, VirtualFile file) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);

        ArrayList<String> rawOutput = new ArrayList<String>();
        HashMap<String,HashMap<String,String>> results = new HashMap<String,HashMap<String,String>>();

        String attrDelimiter = ";;;;";
        String valDelimiter = "::::";

        Process p;
        BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("history");
        options.add("-f");
        options.add(" objectname " + valDelimiter + " %objectname " +
                attrDelimiter +
                " name " + valDelimiter + " %name " +
                attrDelimiter +
                " version " + valDelimiter + " %version " +
                attrDelimiter +
                " state " + valDelimiter + " %status " +
                attrDelimiter +
                " owner " + valDelimiter + " %owner ");
        options.add(file.getPath());

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line;
            boolean readNextLine = true;

            while ((line = output.readLine()) != null) {
                if(!readNextLine) {
                    if(line.contains("*****")) {
                        readNextLine = true;
                    }
                }
                else {
                    if(line.contains("Predecessors:") || line.contains("Successors:")) {
                        readNextLine = false;
                    }
                    else {
                        rawOutput.add(line);
                    }
                }
            }
        } catch (IOException e) { new CMExceptionHandler(e); }

        if(rawOutput.size() > 0) {
            // Parse the output
            String[] keyVal;

            String objectname;
            HashMap<String,String> attrs;

            for(String line : rawOutput) {
                // If name was already assigned, then that means an iteration of the for loop has
                // been completed for a version of the object.  Therefore add the object name and
                // attrs into the result before overriding the name and attrs variables
                /*if(!objectname.equals("")) {
                    results.put(objectname, attrs);
                    objectname = "";
                    attrs = new HashMap<String,String>();
                }*/

                objectname = "";
                attrs = new HashMap<String,String>();
                keyVal = line.split(attrDelimiter);

                if(keyVal.length > 0) {
                    for(String attribute : keyVal) {
                        String[] splitAttribute = attribute.split(valDelimiter,2);

                        // Make sure the split did what we wanted
                        if(splitAttribute.length == 2) {
                            // Catch the name attribute and add the other attributes to a hashmap
                            if(splitAttribute[0].contains("objectname")) {

                                objectname = splitAttribute[1].trim();
                            }
                            else {
                                attrs.put(splitAttribute[0].trim(), splitAttribute[1].trim());
                            }
                        }
                        results.put(objectname, attrs);
                    }
                }
            }
        }

        return results;
    }

    /**
     * Retreives history data for CMFileRevision
     * @param project The active IDEA project
     * @param filePath The file involved in the history command
     * @return A LinkedList containing HistoryData objects (one for each version of the object)
     */
    public static LinkedHashMap<HistoryObject.Key,HistoryObject> getRevisionData(Project project, FilePath filePath) {
        CMConfiguration         myConfig = CMConfiguration.getInstance(project);
        ArrayList<VcsFileRevision> historyData = new ArrayList<VcsFileRevision>();

        LinkedHashMap<HistoryObject.Key,HistoryObject> objects = new LinkedHashMap<HistoryObject.Key,HistoryObject>();

        Process p;
        BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("history");
        options.add("-f");
        options.add("%objectname");
        options.add(filePath.getPath());

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute the history command and set up the buffered reader
            p       = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));

            // Define the objects to be used in history processing
            String              line                = null;
            String              objectName          = null;
            ArrayList<String>   predecessors        = new ArrayList<String>();
            ArrayList<String>   successors          = new ArrayList<String>();
            boolean             predecessorOutput   = false;  // Indicates whether we're processing predecessors
            boolean             successorOutput     = false;  // Indicates whether we're processing successors


            while ((line = output.readLine()) != null) {
                if(line.contains("******")) {
                    if(objectName != null) {
                        // A new object is being processed. Place the previous object data into the 'objects' HashMap,
                        // then reset all applicable vars
                        @SuppressWarnings("unchecked")
                        final ArrayList<String> pre = (ArrayList<String>) predecessors.clone();
                        @SuppressWarnings("unchecked")
                        final ArrayList<String> suc = (ArrayList<String>) successors.clone();

                        HistoryObject tempHistObj = new HistoryObject(project, objectName, pre, suc);
                        //tempHistObj.populate();
                        objects.put(tempHistObj.getKey(), tempHistObj);

                        objectName          = null;
                        predecessorOutput   = false;
                        successorOutput     = false;
                        predecessors.clear();
                        successors.clear();
                    }
                }
                else if(line.contains("Predecessors:")) {
                    // Read next line, but use different parsing rules
                    predecessorOutput = true;
                }
                else if(line.contains("Successors:")) {
                    predecessorOutput = false;
                    successorOutput = true;
                }
                else {
                    if(predecessorOutput) {
                        predecessors.add(line.replaceFirst("\t",""));
                    }
                    else if(successorOutput) {
                        successors.add(line.replaceFirst("\t",""));
                    }
                    else {
                        objectName = line;
                    }
                }

            }
        } catch (IOException e) { new CMExceptionHandler(e); }

        return objects;
    }

    /**
     * Searches through the history data to find a common predecessor
     * @param first
     * @param second
     * @param history
     * @return
     */
    public static HistoryObject getSimpleCommonPredecessor(HistoryObject first, HistoryObject second, LinkedHashMap<HistoryObject.Key, HistoryObject> history) {

        ArrayList<String> ancestorsFirst = new ArrayList<String>();
        ArrayList<String> ancestorsSecond = new ArrayList<String>();
        ArrayList<String> commonAncestors = new ArrayList<String>();

        // First get the immediate predecessors to the two objects
        ancestorsFirst.addAll(first.getPredecessors());
        ancestorsSecond.addAll(second.getPredecessors());

        // Run a comparison between the two arraylists and add common ancestors
        for(String ancestor : ancestorsFirst) {
            if(ancestorsSecond.contains(ancestor)) {
                commonAncestors.add(ancestor);
            }
        }

        if(commonAncestors.size() == 1) {
            // Even better, we don't need to choose which is the proper ancestor to return
            return history.get(new HistoryObject.Key(commonAncestors.get(0)));
        }

        return null;
    }

    /**
     *
     * @param project
     * @param file
     * @param historyData
     * @return
     * @throws VcsException
     */
    synchronized public static LinkedList<HistoryObject> getMergeCandidates(Project project, VirtualFile file, LinkedHashMap<HistoryObject.Key, HistoryObject> historyData) throws VcsException {

        if(historyData == null) {
            historyData = History.getRevisionData(project, VcsUtil.getFilePath(file.getPath()));
        }

        final HistoryObject             originalObject;
        final ArrayList<String>         candidatesToProcess;
        final ArrayList<String>         alreadyProcessed = new ArrayList<String>();
        final LinkedList<HistoryObject> candidates  = new LinkedList<History.HistoryObject>();


        final String                    currentObjectName   = new PropertiesCommand(file, project).get("%objectname", file.getPath());
        final HistoryObject             currentObject       = historyData.get(new HistoryObject.Key(currentObjectName));
        final ArrayList<String>         predecessors        = currentObject.getPredecessors();

        // We have the history information for the current object. Now we need to load the predecessor. We will get its
        // source and also its successors

        if (predecessors.size() > 0) {
            // TODO: Sloppy
            originalObject  = historyData.get(new HistoryObject.Key(predecessors.get(0)));
            candidatesToProcess = originalObject.getSuccessors();

            // Insert all non-working successors into the candidateData list
            // Also remove the current object if it shows up in the candidate list.  This will happen if the current
            // object is not in the working state. We will eventually check this file out, but only after the user has
            // selected a candidate to merge with. If it was checked out now, code would be needed to undo the checkout
            // in the event the user cancels the merge before making a candidate selection.

            // This loops over the original object's successors.  The point is to capture the end nodes of the history
            // tree.  This means if a successor has descendants, we drop that successor from the list of candidates and
            // add its successors to candidatesToProcess.  This repeats until there are no more successors left to process
            while(candidatesToProcess.size() > 0) {
                ArrayList<String> foundInLoop         = new ArrayList<String>();
                ArrayList<String> processedInLoop   = new ArrayList<String>();

                for(String candidate : candidatesToProcess) {
                    if(!alreadyProcessed.contains(candidate)) {
                        //Check for descendants
                        HistoryObject       candidateObject = historyData.get(new HistoryObject.Key(candidate));
                        ArrayList<String>   newPredecessors   = new ArrayList<String>();
                        ArrayList<String>   newSuccessors   = new ArrayList<String>();

                        newSuccessors.addAll(candidateObject.getSuccessors());
                        newPredecessors.addAll(candidateObject.getPredecessors());

                        /*for(String s : candidateObject.getSuccessors()) {
                            if(!alreadyProcessed.contains(s)) {
                                newSuccessors.add(s);
                            }
                        }

                        for(String s : candidateObject.getPredecessors()) {
                            if(!alreadyProcessed.contains(s)) {
                                newPredecessors.add(s);
                            }
                        }*/

                        // If we're at an end node in the history tree, then the successor is a candidate for merge...
                        // unless it's in a working state or is the current object itself
                        if(newSuccessors.size() == 0) {
                            if(!candidateObject.getStatus().equals("working") && !candidateObject.getObjectName().equals(currentObjectName)) {
                                if(!candidates.contains(candidateObject)) {
                                    candidates.add(candidateObject);
                                }
                            }
                        }

                        processedInLoop.add(candidate);
                        foundInLoop.addAll(newSuccessors);
                        foundInLoop.addAll(newPredecessors);
                    }
                }

                alreadyProcessed.addAll(processedInLoop);
                candidatesToProcess.addAll(foundInLoop);

                for(String s : alreadyProcessed) {
                    candidatesToProcess.remove(s);
                }

            }

        }

        return candidates;
    }

    public static class HistoryObject {

        private Project             project;
        private String              objectName, name, version, owner, comment, status, release, task, modifyTime, createTime = null;
        private ArrayList<String>   predecessors, successors = null;
        private Date                createdDate, modifiedDate = null;
        private Key                 key = null;
        private boolean             populated = false;
        private byte[]              source = null;

        private AttributeCommand    attr;

        public HistoryObject(Project project, String objectName, ArrayList<String> predecessors, ArrayList<String> successors) {
            this.project        = project;
            this.objectName     = objectName;
            this.predecessors   = predecessors;
            this.successors     = successors;
            key                 = new Key(objectName);
            this.attr = new AttributeCommand(project, objectName);
        }

        public Key getKey()                          {  return key; }

        public String getObjectName()                {  return objectName; }

        public void setObjectName(String objectname) {  this.objectName = objectname; }

        public String getName(){
            if(name == null){
                loadName();
            }

            return name;
        }

        public void loadName() {
            name = attr.get(AttributeCommand.NAME);
        }

        public String getVersion(){
            if(version == null) {
                loadVersion();
            }

            return version;
        }

        public void loadVersion(){
            version = attr.get(AttributeCommand.VERSION);
        }

        public String getOwner(){
            if(owner == null) {
                loadOwner();
            }

            return owner;
        }

        public void loadOwner(){
            owner = attr.get(AttributeCommand.OWNER);
        }

        public String getComment(){
            if(comment == null) {
                loadComment();
            }
            return comment;
        }

        public void loadComment(){
            comment = attr.get(AttributeCommand.COMMENT);
        }

        /*
        public Date getCreated(){
            if(created == null) {
                loadCreated();
            }

            return created;
        }

        public void loadCreated(){
            String createTime = attr.get(AttributeCommand.CREATE_TIME);

            try {
                this.created = DateFormat.getDateInstance().parse(createTime);
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }

        public Date getMofidied(){
            if(modified == null) {
                loadModified();
            }

            return modified;
        } */

        public String getCreated(){
            if(createTime == null) {
                loadCreated();
            }

            return createTime;
        }

        public void loadCreated(){
            createTime = attr.get(AttributeCommand.CREATE_TIME);
        }

        public String getMofidied(){
            if(modifyTime == null) {
                loadModified();
            }

            return modifyTime;
        }

        public void loadModified(){
            modifyTime = attr.get(AttributeCommand.MODIFY_TIME);
            //assert(modifyTime != null);
            /*
            try {
                DateFormat formatInstance = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.MEDIUM);
                this.created = DateFormat.getDateTimeInstance().parse(modifyTime);
            } catch (ParseException e) {
                e.printStackTrace();
            } */

        }

        public Date getCreatedDate() {
            if(createdDate == null) {
                loadCreatedDate();
            }

            return createdDate;
        }

        public void loadCreatedDate() {
           String rawDate = attr.get(AttributeCommand.CREATE_TIME);

            try {
                SimpleDateFormat formatter = new SimpleDateFormat(CMConstants.SessionVars.JAVA_DATE_FORMAT);
                this.createdDate = formatter.parse(rawDate);
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }

        public Date getModifiedDate() {
            if(modifiedDate == null) {
                loadModifiedDate();
            }

            return modifiedDate;
        }

        public void loadModifiedDate() {
            String rawDate = attr.get(AttributeCommand.MODIFY_TIME);

            try {
                SimpleDateFormat formatter = new SimpleDateFormat(CMConstants.SessionVars.JAVA_DATE_FORMAT);
                this.modifiedDate = formatter.parse(rawDate);
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }

        public String getRelease(){
            if(release == null) {
               loadRelease();
            }

            return release;
        }

        public void loadRelease()       {  release = attr.get(AttributeCommand.RELEASE); }

        public String getStatus(){
            if(status == null) {
                loadStatus();
            }

            return status;
        }

        public void loadStatus() {
            status = attr.get(AttributeCommand.STATUS);
        }
/*
        public void loadTask() {
            String objectName = getObjectName();
            String task = new PropertiesCommand()
        }

        public String getTask(){
            if(task == null) {
               // Get the task
            }
            return task;
        }

        public void setTask(String task) {
            this.task = task;
        }*/

        public ArrayList<String> getPredecessors()                  {  return predecessors; }
        public void setPredecessors(ArrayList<String> predecessors) {  this.predecessors = predecessors; }

        public ArrayList<String> getSuccessors()                    {  return successors;  }
        public void setSuccessors(ArrayList<String> successors)     {  this.successors = successors;  }

        public void loadSource() {
           this.source = attr.getAsByte(AttributeCommand.SOURCE);
        }

        public byte[] getSource() {
            if(source == null) {
                loadSource();
            }
            return source;

            //return new byte[0];
        }

        /**
         * We have the objectName, predecessors and successors. This method gets the rest of the information.
         * This is not done automatically in the above constructor, as all this data may not be necessary in every
         * instantiation of HistoryObject
         */


        public void populate() {
            // We need these strings : name, version, author (owner), comment, status, task
            // We also need these dates: created, modified

            loadName();
            loadVersion();
            loadOwner();
            loadComment();
            loadStatus();
            loadCreated();
            loadModified();
            loadRelease();

            populated = true;
        }

        public boolean isPopulated() {
            return populated;
        }

        public String toString() {
            return getObjectName();
        }

        public static final class Key {
            private final String id;

            public Key(String id) {
                if (id == null) {
                    throw new IllegalArgumentException();
                }
                this.id = id;
            }

            public int hashCode() {
                return id.hashCode();
            }

            public boolean equals(Object obj) {
                if (!(obj instanceof Key)) return false;
                return id.equals(((Key) obj).id);
            }
        }
    }
}
