// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.vcsUtil.VcsUtil;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;

import java.util.ArrayList;
import java.util.Map;
import java.io.File;

/**
 * Handles file move operations
 * @author Luke Stahlman
 */
public class MoveCommand extends AbstractCommand {
    private static final Logger LOG = Logger.getInstance("#com.telelogic.integration.intellij.cli.MoveCommand");

    protected   String taskID;
    public      boolean showSuccessMessage = false;
    protected   boolean delete = false;
    protected   VirtualFile commonDir;

    protected MoveCommand() {
        super();
    }

    protected MoveCommand(Project p) {
        super(p);
    }

    /**
     * Used to move an object
     * @param p The active IntelliJ project
     * @param currentLoc The current path
     * @param newLoc The new path
     * @param task The task this operation should be associated with
     * @throws VcsException If task is not set
     */
    public MoveCommand(Project p, VirtualFile currentLoc, VirtualFile newLoc, String task) throws VcsException {
        super(p);

        if(task != null) {
            taskID = task;
        }
        else {
            if(!new Tasks(project).ensureTaskSet()) {
                throw new VcsException("Error: No task is set.");
            }
            taskID = myConfig.getDefaultTask();
        }
        // Synergy doesn't want to see the full path to these files, so we will make it relative to the
        // project (if possible)
       if((commonDir = VfsUtil.getCommonAncestor(currentLoc, newLoc)) != null){

            String oldPath = currentLoc.getPath().replaceFirst(commonDir.getPath(),"./");
            String newPath = newLoc.getPath().replaceFirst(commonDir.getPath(),"./");

            execute(oldPath, newPath);
        }
        else{
            // Can't run the move operation
        }
    }

    /**
     *
     * @param from file spec
     * @param to destination dir
     */
    public void execute(String from, String to) {
        options = new ArrayList<String>();

        options.add(myConfig.getPath());

        options.add("move");
        options.add("-task");
        options.add(taskID);
        options.add(from);
        options.add(to);

        String[] commands = options.toArray(new String[options.size()]);
        LOG.info("Moving " + from + " to " + to);
        try {
            toolWindow.appendConsole("Starting Move Operation...");
            // Run command
            //process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            ProcessBuilder pb = new ProcessBuilder(commands);
            if(commonDir != null) {
                pb = pb.directory(new File(commonDir.getPath()));
            }

            Map<String, String> env = pb.environment();
            env.put("CCM_ADDR", myConfig.getCmEnvPath());

            process = pb.start();

            new HelperThread(this).start();

        } catch (Exception e) {
            new CMExceptionHandler(e);
        }
    }

    /**
     * String value of this class
     *
     * @return Name of the class
     */
    public String toString() {
        return "Move";
    }

}
