// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.project.Project;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * This handles all file/object properties checks and is used primarily to display file/object information to the user
 * and determine whether files can be checked in or out by users
 * @author Luke Stahlman
 */
public class PropertiesCommand extends AbstractCommand {
    private VirtualFile     file;
    final   int             QUIET = 0;
    final   int             DISPLAY = 1;
    private String[]        attributes;

    /**
     * PropertiesCommand Constructor
     * This is the primary constructor
     * @param f File to run the properties command against
     * @param p Active IDEA project
     */
    public PropertiesCommand(VirtualFile f, Project p) {
        super(p);
        file        = f;
    }

    /**
     * Custom PropertiesCommand Constructor
     * Used when specific attributes of a file/object are needed
     * @param f File to run the properties command against
     * @param p Current project
     * @param attributes Custom attributes that are needed from the command query
     */
    public PropertiesCommand(VirtualFile f, Project p, String[] attributes){
        super(p);
        file            = f;
        this.attributes = attributes;
    }

    public boolean isControlled() {
        if(get() == null) {
            return false;
        }
        return true;
    }

    /**
     * Runs the propertes command on the file passed via the constructor, then determines whether the file is checked
     * in. The current method of checking is looking at the status attribute. There are two statuses that are not
     * synonymous with being checked in; "working" and "prep".  This method only checks for "working" at this time
     * (developers should not encounter objects in the "prep" state.
     * @return Whether a file is checked in
     */
    public boolean isCheckedIn() {
        attributes = new String[]{"status:%status"};
        HashMap<String,String> result = this.get();

        return !result.get("status").contains("working");
    }

    /**
     * Same as the other isCheckedIn method, but instead checking the status in the properties parameter.
     * @param properties The file properties to check
     * @return Whether a file is checked in
     */
    public static boolean isCheckedIn(HashMap<String,String> properties) {
        return !properties.get("status").contains("working");
    }

    /**
     * Gets file properties and checks if the file is checked out.  See isCheckedIn() for details on what constitutes a
     * checked out file.
     * @return Whether a file is checked out
     */
    public boolean isCheckedOut() {
        attributes = new String[]{"status:%status", "owner:%owner"};
        HashMap<String,String> result = this.get();

        return result.get("status").contains("working");
    }

    /**
     * More specific than isCheckedOut(), this method checks if the file is checked out to the user logged in under the
     * current Synergy session.
     * @return boolean answer to the check
     */
    public boolean isCheckedOutToUser() {
        attributes = new String[]{"status:%status", "%owner"};
        HashMap<String,String> result = this.get();

        return (result.get("status").contains("working") && result.get("owner").contains(myConfig.getUser()));

    }

    /**
     * Currently the same as isCheckedOutToUser() but is here in case more specific checks need to be made when a user
     * wants to check in a file.
     * @return boolean Whether an object can be checked in
     */
    public boolean canBeCheckedIn() {
        attributes = new String[]{"status:%status", "%owner"};
        HashMap<String,String> result = this.get();

        return (result.get("status").contains("working") && result.get("owner").contains(myConfig.getUser()));
    }

    /**
     * This sets up and executes the properties command
     * @return HashMap<String,String> with desired file/object attributes
     */
    public HashMap<String,String> get() {
        Process p;
        BufferedReader br;
        ArrayList<String> options       = new ArrayList<String>();
        HashMap<String,String> result   = new HashMap<String,String>();

        options.add(myConfig.getPath());
        options.add("properties");
        options.add(file.getPath());

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Run command
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
            br = new BufferedReader(new InputStreamReader(p.getInputStream()));

            result = CommandHelper.parseProperties(br);

        } catch (IOException e) { new CMExceptionHandler(e); }

        return result;
    }

    /**
     * Gets a specific property for an object
     * @return String value of the property
     * @param property The property desired
     * @param objectName The object name
     */
    public String get(String property, String objectName) {
        Process             p;
        BufferedReader      br;
        ArrayList<String>   options     = new ArrayList<String>();
        String              result      = null;

        options.add(myConfig.getPath());
        options.add("properties");
        options.add("-f");
        options.add(property);
        options.add(objectName);

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Run command
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
            br = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String l;
            int lineCount = 0;
            while ((l = br.readLine()) != null) {
                String trimmedLine = l.trim();
                // If this isn't the first line of output, add a line break
                if(lineCount > 0) {
                    result += "\n";
                }

                // Add the output to the result string
                if(result == null) {
                    result = trimmedLine;
                }
                else {
                    result += trimmedLine;
                }

                lineCount++;
            }

            br.close();

        } catch (IOException e) { new CMExceptionHandler(e); }

        return result;
    }

    /**
     * This sets up and executes the properties command to gather a specific attribute. Useful when only one piece of
     * information is desired (eg the file status)
     * @param format The desired format to be sent to Synergy. Format should be "attributeName:%synergyattribute"
     * @return HashMap<String,String> with desired file/object attributes
     */
    @SuppressWarnings({"SameParameterValue"})
    public HashMap<String,String> get(String format) {
        Process p;
        BufferedReader br;
        ArrayList<String> options       = new ArrayList<String>();
        HashMap<String,String> result   = new HashMap<String,String>();

        options.add(myConfig.getPath());
        options.add("properties");
        options.add("-f");
        options.add(format);
        options.add(file.getPath());

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Run command
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
            br = new BufferedReader(new InputStreamReader(p.getInputStream()));

            result = CommandHelper.parseProperties(br);

        } catch (IOException e) { new CMExceptionHandler(e); }

        return result;
    }

    /**
     * This runs the default ccm properties command (no special attributes) and sends the output to the HelperThread
     * for display to console and dialog
     */
    public void display() {

        //Process p;
        //BufferedReader br;
        ArrayList<String> options   = new ArrayList<String>();
        String fileName             = file.getName();

        options.add(myConfig.getPath());
        options.add("properties");
        options.add(file.getPath());

        String[] commands = options.toArray(new String[options.size()]);

        try {
            toolWindow.appendConsole("Getting properties for " + fileName);
            // Run command
            process   = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
            //br  = new BufferedReader(new InputStreamReader(p.getInputStream()));
            //new HelperThread(project, toolWindow, br, HelperThread.PROPERTIES).start();
            new HelperThread(this).start();

        } catch (Exception e) { new CMExceptionHandler(e); }

    }

    /**
     * @see com.telelogic.integration.intellij.cli.AbstractCommand#toString()
     */
    public String toString() { return "Properties"; }
}
