// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.cm.CMPlugin;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMConstants;
import com.telelogic.integration.intellij.ui.CMToolWindow;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;

import java.util.ArrayList;
import java.util.HashMap;
import java.io.*;

/**
 * Handles the starting and stopping of CM sessions. Should generally only be started and stopped once per IDE session
 * @author Luke Stahlman
 */
public class Session {
    // --Commented out by Inspection (4/19/07 12:31 PM):private String          CCM_ADDR;
    private CMConfiguration myConfig;
    private CMToolWindow    toolWindow;
    private CMPlugin        plugin;
    private Project         project;

    /**
     * Constructor
     * @param p The active IDEA project
     */
    public Session(Project p)
    {
        project     = p;
        myConfig    = CMConfiguration.getInstance(project);
        plugin      = CMPlugin.getInstance(project);
        toolWindow  = plugin.toolWindow;
    }

    /**
     * Used to start both GUI and CLI sessions. The following arguments are passed to "ccm start":
     * <pre>
     * -nogui If a CLI session is desired
     * -m Multiple server mode
     * -d Database
     * -h Engine
     * -n User
     * -pw Password
     * </pre>
     * Output (input stream and error stream) is sent to the console and the progress bar is started.  The progress bar
     * is stopped by the tool window, which analyzes and displays the output.
     */

    public void start()
    {
        Process process;
        ArrayList<String> options = new ArrayList<String>();
        BufferedReader br;
        BufferedReader errorBr;
        HashMap<String,String> envVars = new HashMap<String,String>();

        // Set the datetime format for all date-related CLI output
        envVars.put("CCM_DATETIME_FMT", CMConstants.SessionVars.CCM_DATE_FORMAT);


        if(myConfig.checkSettings()) {
            // Build command: ccm start [-nogui] -m -d database -h engine -n user -pw password
            options.add(myConfig.getPath());
            options.add("start");
            if(!myConfig.isGuiSession){ options.add("-nogui"); }
            options.add("-m");
            options.add("-d");
            options.add(myConfig.getDB());
            options.add("-h");
            options.add(myConfig.getEngine());
            options.add("-n");
            options.add(myConfig.getUser());
            options.add("-pw");
            options.add(myConfig.getPassword());

            String[] commands = options.toArray(new String[options.size()]);

            try {
                if(myConfig.isGuiSession) { toolWindow.appendConsole("Starting CM GUI Session...");}
                else    { toolWindow.appendConsole("Starting CM non-GUI Session...");}

                // Run command and send output to the console tab of the tool window
                //p = CommandHelper.runCommand(commands);
                process = CommandHelper.runCommand(commands, envVars);

                CommandHelper.processStartCommand(project, process);
                //br = new BufferedReader(new InputStreamReader(p.getInputStream()));
                //errorBr = new BufferedReader(new InputStreamReader(p.getErrorStream()));

                //toolWindow.startProgressBar();
                //toolWindow.sendToConsole(br, p);
                //toolWindow.sendToConsole(errorBr, p);

            } catch (IOException e) { new CMExceptionHandler(e); }
        }
        else {
            Messages.showErrorDialog("Please check your CM settings before starting a session.", "Settings Error");
        }
    }

    /**
     * Stops the session.  This method could be improved by evaluating whether a session has truly stopped before
     * declaring SESSION_STARTED false
     */
    public void stop()
    {
        Process p;
        ArrayList<String> options = new ArrayList<String>();
        BufferedReader br;

        options.add(myConfig.getPath());
        options.add("stop");

        String[] commands = options.toArray(new String[options.size()]);

        try {
            toolWindow.appendConsole("Stopping Session...");
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            br = new BufferedReader(new InputStreamReader(p.getInputStream()));
            toolWindow.sendToConsole(br);

            
        } catch (IOException e) { new CMExceptionHandler(e); }

        // Change the view to show the start panel (synergy icon and buttons to start GUI/CLI sessions) and
        // tell config no session is started.  The CCM_ADDR path is also erased
        //toolWindow.showStart();
        plugin.sessionStopped();
        
        myConfig.setCmEnvPath("");
    }

    /**
     * Determines whether a CLI or GUI session is started
     * @param withGUI The boolean value to assign to GUI
     */
    public void setGUI(boolean withGUI)         { myConfig.isGuiSession = withGUI; }

// --Commented out by Inspection START (4/19/07 12:31 PM):
//    /**
//     * CCM_ADDR is an address used to access the Synergy session
//     * @return CCM_ADDR Environment variable
//     */
//    public String getCCM_ADDR()                 { return CCM_ADDR; }
// --Commented out by Inspection STOP (4/19/07 12:31 PM)

    //public OutputCollector getProcessListener() { return null; }     // Implements the CMCommand's getProcessListener method. Not used
}
