// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.ui.Messages;
import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.ui.StatusBarHandler;

import java.util.*;
import java.io.*;

/**
 * Provides task-based operations. Most methods are static
 */
public class Tasks extends AbstractCommand{

    /**
     * Main constructor. Not often used as most all methods can be called without instantiating the entire class.
     * @param p The active IDEA project
     */
    public Tasks(Project p){
        super(p);
    }

    /**
     * Gets all non-completed tasks assigned to the user
     * @return List of assigned tasks. Format: "Task Number", "Task Name/Synopsis" or "No tasks assigned"
     */
    public synchronized LinkedHashMap<String,String> getMyAssignedTasks(){
        LinkedHashMap<String,String> myTasks = new LinkedHashMap<String,String>();
        Process p;
        BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();
        String valDelimiter = " :::: ";

        //  Build command
        options.add(myConfig.getPath());
        options.add("task");
        options.add("-qu");
        options.add("-ts");
        options.add("all_my_assigned");
        options.add("-f");
        options.add("%displayname" + valDelimiter + "%task_synopsis");
        options.add("-u"); // Unnumbered


        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            // Parse result into hash map
            // The CLI format is xx) Task yy: <task desc>
            // This code removes the " xx) Task" and splits the result into an array
            // using the : as the delimiter
            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line;
            //int i = 1;
            while ((line = output.readLine()) != null) {
                // First check whether any results will be displayed. If not, stop the loop
                if(line.contains("No Tasks selected by the query")) {
                    break;
                }
                else {
                    if(line.contains("Waiting, CM Synergy command in progress")){
                        continue;
                    }
                    // Split on the valDelimiter, and put task->synopsis
                    String[] parsed = line.split(valDelimiter, 2);
                    myTasks.put(parsed[0],parsed[1]);
                }
            }

        } catch (IOException e) { new CMExceptionHandler(e); }

        return myTasks;
    }

    // TODO: DCM Support requires tasks to be treated as strings, not integers (e.g., Task US#155)

    /**
     * Gets the current task as determined by Synergy, then returns the parsed value of that task.
     * @param project The active IDEA project
     * @return The current task ID
     */
    public static String getCurrentTaskID(Project project) {
        String task = Tasks.getCurrentTask(project);
        return parseTaskID(task);

        /*String taskID = "";

        if(!task.contains("not set")) {
            String[] parsedTask = task.split(":",2);
            if(parsedTask.length > 1) {
                try {
                    taskID = parsedTask[0];
                }
                catch(Exception e) {
                    // Just in case something weird happened
                     new CMExceptionHandler(e);
                }
            }
        }

        return taskID; */
    }

    /**
     * Separates the task ID from the full task name.
     * This is compatible with DCM's task ID naming conventions
     * @param task The task name
     * @return The parsed task ID
     */
    public static String parseTaskID(String task) {
        String taskID = "";

        if(!task.contains("not set")) {
            String[] parsedTask = task.split(":",2);
            if(parsedTask.length > 1) {
                try {
                    taskID = parsedTask[0];
                }
                catch(Exception e) {
                    // Just in case something weird happened
                     new CMExceptionHandler(e);
                }
            }
        }

        return taskID;
    }

    /**
     * Gets the current task as determined by Synergy. This can be used on session startup as well as to check whether
     * the current task identified in IDEA is correct.
     * @param project The active IDEA project
     * @return The current task. Format: "Task xx: Task synopsis" or "The current task is not set."
     */
    public static String getCurrentTask(Project project) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);
        String myTask = "";
        Process p;
        BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("task");
        options.add("-default");

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line;
            while ((line = output.readLine()) != null) {
                myTask = line;
            }
        } catch (IOException e) { new CMExceptionHandler(e); }

        //if(!Tasks.parseTaskID(myTask).equals(myConfig.getDefaultTask())) {
            myConfig.setDefaultTask(parseTaskID(myTask));
            StatusBarHandler.getInstance(project).setTask(myTask);
        //}

        return myTask;
    }

    public boolean ensureTaskSet() {
        if(getCurrentTask(project).equals("The current task is not set.") || getCurrentTask(project).equals("")) {
            LinkedHashMap<String,String> tasks = getMyAssignedTasks();
            LinkedList<String> taskList = new LinkedList<String>();

            for(Map.Entry synopsis : tasks.entrySet()) {
                taskList.add(synopsis.getKey() + ": " + synopsis.getValue());
            }

            // Check if the user has any assigned tasks.  If not, display a message and return false.
            if(taskList.size() == 0) {
                Messages.showInfoMessage("This operation requires a current task, however no tasks are assigned to you.", "No Assigned Tasks");
                return false;
            }

            String[] list = taskList.toArray(new String[taskList.size()]);

            int choice = Messages.showChooseDialog("Please select a default task to set:", "Set Default Task", list, list[0], null);
            if(choice != -1) {
                setCurrentTask(parseTaskID(list[choice]), project);
                return true;
            }
            else {
                return false;
            }
        }
        else {
            return true;
        }
    }

    /**
     * Placeholder for a future featureset that will allow tasks to be created from within IDEA
     * @param withClassicClient Whether the classic client task dialog should be used
     */
    public void createTask(boolean withClassicClient) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);
        Process p;
        BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();
        HashMap<String,String> result = new HashMap<String,String>();

        if(withClassicClient) {

            //  Build command
            options.add(myConfig.getPath());
            options.add("task");
            options.add("-create");
            options.add("-g");

            String[] commands = options.toArray(new String[options.size()]);

            try {
                // Execute
                p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            } catch (IOException e) { new CMExceptionHandler(e); }
        }
    }

    /**
     * Gets the properties for a specified task
     * @param project The active IDEA project
     * @param taskID The task to get properties for
     * @return The output of the command as parsed by CommandHelper.parseTask()
     */
    public static HashMap<String,String> getTaskProperties(Project project, String taskID) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);
        Process p;
        BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();
        HashMap<String,String> result = new HashMap<String,String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("task");
        options.add("-show");
        options.add("i");
        options.add(taskID);

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));

            result = CommandHelper.parseTask(output);

        } catch (IOException e) { new CMExceptionHandler(e); }

        // Add an additional keyval if this is the current task
        if(result.get("Task").equals(myConfig.getDefaultTask())) {
            result.put("default","true");
        }

        return result;
    }

    /**
     * Gets the properties for a specified task
     * @param project The active IDEA project
     * @param taskID The task to get properties for
     * @return The output of the command as parsed by CommandHelper.parseTask()
     */
    public static HashMap<String,String> getTaskPropertiesOLD(Project project, String taskID) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);
        Process p;
        BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();
        HashMap<String,String> result = new HashMap<String,String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("task");
        options.add("-show");
        options.add("i");
        options.add(taskID);

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));

            result = CommandHelper.parseTask(output);

        } catch (IOException e) { new CMExceptionHandler(e); }

        // Add an additional keyval if this is the current task
        if(result.get("Task").equals(myConfig.getDefaultTask())) {
            result.put("default","true");
        }

        return result;
    }

    /**
     * Sets the current task based on the passed taskID
     * @param taskID The task to be set
     * @param project The active IDEA project
     */
    public static void setCurrentTask(String taskID, Project project) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);
        //Process p;
        //BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("task");
        options.add("-default");
        options.add(taskID);

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            Process process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            // Consume the process output
            // Error checking could be done here, but it's not necessary
            new AbstractOutputConsumer(project, process, false, false) {
                protected void findErrors(String line) {}
                protected void finished(ArrayList<String> result) {}
            };

            // Check if command is successful
            if(Tasks.getCurrentTaskID(project).matches(taskID)) {
                myConfig.setDefaultTask(taskID);
            }
            else {
               //TODO: place message here stating something went wrong
            }

        } catch (IOException e) { myConfig.setDefaultTask(Tasks.getCurrentTaskID(project)); }
    }

    /**
     * Sets the current task based on the passed taskID
     * @param taskID The task to be set
     * @param project The active IDEA project
     */
    public static void setCurrentTask2(String taskID, Project project) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);
        //Process p;
        //BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("task");
        options.add("-default");
        options.add(taskID);

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            /*p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line;
            while ((line = output.readLine()) != null) {
                //TODO: Catch errors?
            }*/

            if(Tasks.getCurrentTaskID(project) == taskID) {
                myConfig.setDefaultTask(taskID);
            }
            else {
               //TODO: place message here stating something went wrong
            }

        } catch (IOException e) { myConfig.setDefaultTask(Tasks.getCurrentTaskID(project)); }
    }

    /**
     * Associate a specified object or objects to a task
     * @param taskID The task to which the object(s) will be assigned
     * @param files The objects to be assigned
     * @param project The active IDEA project
     */
    public static void associateWithTask(String taskID, VirtualFile[] files, Project project) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);
        //Process p;
        //BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("task");
        options.add("-a");
        options.add(taskID);
        options.add("-obj");
        for(VirtualFile file : files) {
            options.add(file.getPath());
        }

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

           /* p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line;
            while ((line = output.readLine()) != null) {
                //TODO: Catch errors?
            }*/

        } catch (IOException e) { new CMExceptionHandler(e); }
    }

    /**
     * Gets objects associated with the specified task.
     * @param taskID The task to use when looking for associated objects
     * @param project The active IDEA project
     * @return The list of objects in format {name, {version, type, owner, task, fullname}}
     */
    public static Vector<Vector> getAssociatedObjects(String taskID, Project project) {

        CMConfiguration myConfig = CMConfiguration.getInstance(project);

        ArrayList<String> rawOutput = new ArrayList<String>();
        HashMap<String,HashMap<String,String>> results = new HashMap<String,HashMap<String,String>>();

        String attrDelimiter = ";;;;";
        String valDelimiter = "::::";

        Process p;
        BufferedReader output;
        ArrayList<String> options = new ArrayList<String>();

        //  Build command
        options.add(myConfig.getPath());
        options.add("task");
        options.add("-show");
        options.add("objects");
        options.add(taskID);
        options.add("-f");
        options.add(":;:;" + " name " + valDelimiter + " %name " +
                attrDelimiter +
                " version " + valDelimiter + " %version " +
                attrDelimiter +
                " type " + valDelimiter + " %type " +
                attrDelimiter +
                " owner " + valDelimiter + " %owner " +
                attrDelimiter +
                " task " + valDelimiter + " %task " +
                attrDelimiter +
                " state " + valDelimiter + " %status " +
                attrDelimiter +
                " fullname " + valDelimiter + " %fullname " +
                attrDelimiter +
                " CR " + valDelimiter + " % ");

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Execute
            p = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            output  = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line;

            while ((line = output.readLine()) != null) {
                if(!line.contains("Warning: Could not identify task")) {

                    // This takes out the "xx)" beginning of each line in the object listing
                    if(line.contains(":;:;")) {
                        rawOutput.add(line.split(":;:;",2)[1]);
                    }
                }
            }
        } catch (IOException e) { new CMExceptionHandler(e); }

        // TODO: Optimize this so the results get placed directly into the vector

        if(rawOutput.size() > 0) {
            // Parse the output
            String[] keyVal;

            String name;
            HashMap<String,String> attrs;

            for(String line : rawOutput) {
                name = "";
                attrs = new HashMap<String,String>();

                keyVal = line.split(attrDelimiter);

                if(keyVal.length > 0) {
                    for(String attribute : keyVal) {
                        String[] splitAttribute = attribute.split(valDelimiter,2);

                        // Make sure the split did what we wanted
                        if(splitAttribute.length == 2) {
                            // Catch the name attribute and add the other attributes to a hashmap
                            if(splitAttribute[0].contains(" name ")) {
                                name = splitAttribute[1].trim();
                            }
                            else {
                                attrs.put(splitAttribute[0].trim(), splitAttribute[1].trim());
                            }
                        }
                        results.put(name, attrs);
                    }
                }
            }
        }

        Vector<Vector> objects = new Vector<Vector>();

        for (String key : results.keySet()) {

            HashMap<String, String> attrs = results.get(key);

            Vector<String> object = new Vector<String>();
            object.addElement(key);
            object.addElement(attrs.get("version"));
            object.addElement(attrs.get("type"));
            object.addElement(attrs.get("owner"));
            object.addElement(attrs.get("state"));
            object.addElement(attrs.get("task"));
            object.addElement(attrs.get("fullname"));

            objects.addElement(object);
        }

        return objects;
    }

    /**
     * @see com.telelogic.integration.intellij.cli.AbstractCommand#toString()
     */
    public String toString() { return "Tasks"; }
}
