// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VirtualFileManager;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.vcs.rollback.DefaultRollbackEnvironment;
import com.intellij.openapi.vcs.rollback.RollbackEnvironment;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMPlugin;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Removes the checked-out file from the project and gets the best candidate to replace it with.
 * @author Luke Stahlman
 */
public class UndoCheckoutCommand extends AbstractCommand {

    /**
     * Main constructor used to gather project information and analyze the file(s) the user wants to run the undo
     * check out operation on.
     * @param f The file(s) to be part of this operation
     * @param p The active IDEA project
     */
    public UndoCheckoutCommand(VirtualFile[] f, Project p) {
        super(p);

        virtualFiles = removeInvalidFiles(f);

        if(virtualFiles.length > 0) {
            String dialogMessage = "The following files will be replaced by their predecessors\n\n";
            for(VirtualFile file : virtualFiles) {
                dialogMessage += "\t" + file.getName() + "\n";
            }
            dialogMessage += "\nThis operation cannot be undone. Are you sure you want to continue?";

            if(Messages.showYesNoDialog(project, dialogMessage, "Confirm Undo Check Out", null) == 0){
                toolWindow.focusOnConsole();
                undoCheckout(virtualFiles);
            }
        }
        else {
            Messages.showMessageDialog("Operation cannot be completed: file(s) not currently checked out.",
                    "Synergy Message", Messages.getInformationIcon());
        }
    }

    /**
     * Runs the undo checkout command. This is actually "ccm delete -replace", which removes the file from the work
     * area, then asks for the best version to replace the old file with.
     * @param files What files will be part of this command
     */
    private void undoCheckout(VirtualFile[] files){
        //Process p;
        //BufferedReader br;
        ArrayList<String> options = new ArrayList<String>();

        saveFiles(files);

        // build checkin command
        // ccm ci -c "comment"  <file> <file> <file>...
        options.add(myConfig.getPath());
        options.add("delete");
        options.add("-replace");

        for(VirtualFile file : files) {
            String path = file.getPath();
            String newPath = path.replace("/","\\");
            options.add(newPath);
            //options.add(file.getPath());
        }

        String[] commands = options.toArray(new String[options.size()]);

        try {
            toolWindow.appendConsole("Undoing Checkout...");
            // Run command
            process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
            //br = new BufferedReader(new InputStreamReader(p.getInputStream()));
            //new HelperThread(project, toolWindow, br, HelperThread.UNDOCHECKOUT, files).start();
            new HelperThread(this).start();

        } catch (Exception e) { new CMExceptionHandler(e); }

    }

    public VirtualFile[] removeInvalidFiles(VirtualFile[] files) {
        ArrayList<VirtualFile> goodFiles = new ArrayList<VirtualFile>();

        for (VirtualFile tmp : files) {
            boolean result = new PropertiesCommand(tmp, project).isCheckedOutToUser();
            if(result) {
                goodFiles.add(tmp);
            }
        }

        return goodFiles.toArray(new VirtualFile[goodFiles.size()]);
    }

    /**
     * @see com.telelogic.integration.intellij.cli.AbstractCommand#toString()
     */
    public String toString() { return "Undo Checkout"; }

}
