// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.project.Project;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;

import java.util.ArrayList;

public class UnuseCommand extends AbstractCommand {
    private String taskID;
    public boolean showSuccessMessage = false;
    private boolean delete = false;

    /**
     * Used to unuse (or delete) a number of objects from a project at once
     * @param files The objects to be added
     * @param p The active IntelliJ project
     * @param task The task this operation should be associated with
     * @param delete Whether the file should be removed from the DB
     */
    public UnuseCommand (VirtualFile[] files, Project p, String task, boolean delete) {
        super(p);
        this.delete = delete;
        virtualFiles = files;
        if(task != null) {
            taskID = task;
        }
        else {
            taskID = myConfig.getDefaultTask();
        }

        execute();
    }

    /**
     * Used to unuse (or delete) an object from a project at once
     * @param file The objects to be added
     * @param p The active IntelliJ project
     * @param task The task this operation should be associated with
     * @param delete Whether the file should be removed from the DB
     */
    public UnuseCommand(VirtualFile file, Project p, String task, boolean delete) {
        super(p);
        this.delete = delete;

        ArrayList<VirtualFile> fileArray = new ArrayList<VirtualFile>();
        fileArray.add(file);

        virtualFiles = fileArray.toArray(new VirtualFile[fileArray.size()]);

        if(task != null) {
            taskID = task;
        }
        else {
            taskID = myConfig.getDefaultTask();
        }

        execute();
    }

    public void execute() {
        options = new ArrayList<String>();

        // Make sure all files are saved first
        saveFiles(virtualFiles);

        options.add(myConfig.getPath());

        options.add("unuse");
        options.add("-t"); 
        if(delete) {
            options.add("-d");
        }
        options.add(taskID);

        for(VirtualFile file : virtualFiles) {
            options.add(file.getPath());
        }

        String[] commands = options.toArray(new String[options.size()]);

        try {
            toolWindow.appendConsole("Starting Unuse Operation...");
            // Run command
            process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());

            new HelperThread(this).start();

        } catch (Exception e) {
            new CMExceptionHandler(e);
        }
    }

    /**
     * String value of this class
     *
     * @return Name of the class
     */
    public String toString() {
        return "Unuse";
    }
}
