// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.ui.Messages;
import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.cm.CMPlugin;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.ui.dialogs.UseVersion;
import com.telelogic.integration.intellij.ui.CMToolWindow;

import java.util.*;
import java.io.File;

/**
 * Opens the UseVersion dialog and executes the "ccm use" command based on the user's choice
 * @author Luke Stahlman
 */
public class UseVersionCommand extends AbstractCommand {

    //private Project project;
    //private CMConfiguration config;
    private VirtualFile file;

    // --Commented out by Inspection (4/19/07 12:32 PM):String userSelection;

    /**
     * Main Constructor.  Gathers history data on the object and displays the UseVersion dialog
     * @param p The active IDEA project
     * @param f The file specified by the user for this operation
     */
    public UseVersionCommand(Project p, VirtualFile f) {
        super(p);
        //project = p;
        file = f;

        //config = CMConfiguration.getInstance(project);
        HashMap<String, HashMap<String, String>> fileHistory = History.getVersions(project, file);

        Vector<Vector<String>> versions = new Vector<Vector<String>>();

        // This allows the user to let Synergy choose the correct version
        // Only needed if the user already invoked Use Version and wants to undo the selection
        Vector<String> defaultVersion = new Vector<String>();
        defaultVersion.addElement("Use recommended");
        defaultVersion.addElement("");
        defaultVersion.addElement("");
        defaultVersion.addElement("");
        defaultVersion.addElement("");

        versions.addElement(defaultVersion);

        //Iterator historyIterator = fileHistory.keySet().iterator();
        for (String key : fileHistory.keySet()) {

            HashMap<String, String> attrs = fileHistory.get(key);

            Vector<String> object = new Vector<String>();
            object.addElement(attrs.get("name"));
            object.addElement(attrs.get("version"));
            object.addElement(attrs.get("state"));
            object.addElement(attrs.get("owner"));
            object.addElement(key);

            versions.addElement(object);
        }

        // Open dialog w/ JTable showing the versons to choose from
        // The JTable calls back if a selection was made
        new UseVersion(versions, this).display();
        
    }

    /**
     * Called by the UseVersionDialog when the user selects a file version to use
     * @param version The desired object version
     */
    public void use(String version) {
        //Process p;
        //BufferedReader br;
        ArrayList<String> options = new ArrayList<String>();
        CMToolWindow toolWindow = CMPlugin.getInstance(project).toolWindow;

        options.add(myConfig.getPath());
        options.add("use");
        if(version.contains("Use recommended")){
            options.add("-r");
            options.add(file.getName());
        }
        else {
            options.add(version);
        }

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Run command
            ProcessBuilder pb = new ProcessBuilder(commands);
            // Make sure we're in the proper directory
            VirtualFile parent = file.getParent();
            if(parent != null && parent.isValid()) {
                pb = pb.directory(new File(parent.getPath()));
            }
            Map<String, String> env = pb.environment();
            env.put("CCM_ADDR", myConfig.getCmEnvPath());

            process = pb.start();
            //process = CommandHelper.runCommand(commands, myConfig.getCmEnvPath());
            //br = new BufferedReader(new InputStreamReader(p.getInputStream()));

            toolWindow.appendConsole("Starting Use Version Operation for " + file.getName());

            ArrayList<VirtualFile> fileArray = new ArrayList<VirtualFile>();
            fileArray.add(file);

            virtualFiles = fileArray.toArray(new VirtualFile[fileArray.size()]);


            int exit = process.waitFor();
            if(exit == 0) {
                Messages.showInfoMessage(project, "Use version for " + version + " succeeded", "Use Version");
            }
            else {
                Messages.showInfoMessage(project, "An error occurred attempting to use " + version, "Use Version");
            }

            // Fix: File does not automatically refresh in view after use version
            for (VirtualFile file : virtualFiles) {
                file.refresh(true, false);
            }

        } catch (Exception e) { new CMExceptionHandler(e); }
    }

    /**
     * @see com.telelogic.integration.intellij.cli.AbstractCommand#toString()
     */
    public String toString() {
        return "Use Version";
    }
}
