// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli.helpers;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.ui.Messages;
import com.telelogic.integration.intellij.cm.CMUtil;
import com.telelogic.integration.intellij.ui.dialogs.ParallelsDialog;
import com.telelogic.integration.intellij.cli.AbstractCommand;

import java.util.ArrayList;

/**
 * Takes objects that have been listed as parallel and attempts to get them merged.
 * This is done via Show Parallels.
 */
public class ParallelHelper {

    Project                 myProject;
    ArrayList<VirtualFile>  parallelFiles;
    AbstractCommand         command;

    ArrayList<VirtualFile>  merged, partiallyMerged, notMerged;

    public ParallelHelper(Project p, ArrayList<String> parallelObjects, AbstractCommand caller) {
        myProject = p;
        parallelFiles = new ArrayList<VirtualFile>();
        command = caller;

        // Confirm whether to show parallels
        String message = "Parallel warnings were encountered during the operation.  Do you want to resolve them and retry " +
                "the operation?";
        if(Messages.showYesNoDialog(myProject, message, "Resolve Parallel Conflicts?", null) != 0) {
            return;
        }
        //Get VirtualFile for each objectName
        for(String object : parallelObjects) {
            ArrayList<VirtualFile> matchingFiles = CMUtil.getVirtualFilesByObjectName(myProject, object);

            // We don't care if there's multiple matches, as all of them should be identical
            // TODO: Take care that if one VirtualFile of multiple matches is updated, others will also be updated
            if(matchingFiles.size() > 0) {
                parallelFiles.add(matchingFiles.get(0));
            }
        }

        // All VirtualFile matches have been found.  Now do show parallels
        try {
            new ParallelsDialog(myProject, parallelFiles, this);
        } catch (VcsException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    /**
     * Called by parallels dialog on close reporting which files were merged and which were not
     * @param merged
     * @param partiallyMerged
     * @param notMerged
     */
    public void reportResults(ArrayList<VirtualFile> merged, ArrayList<VirtualFile> partiallyMerged, ArrayList<VirtualFile> notMerged) {
        // Get merged files from dialog
        this.merged          = merged;
        this.partiallyMerged = partiallyMerged;
        this.notMerged       = notMerged;
        
        processResults();
    }

    /**
     * Look at the reported results and consult the user regarding any lingering parallel conflicts
     * Run the command again or cancel if indicated by the user
     */
    private void processResults() {
        String message = "";

        if(partiallyMerged.size() > 0) {
            message += "The following files were merged but still have parallel conflicts:\n\n";
            for(VirtualFile file : partiallyMerged) {
                message += file.getName() + "\n";
            }
            message += "\n";

        }

        if(notMerged.size() > 0) {
            message += "The following files were not merged:\n\n";
            for(VirtualFile file : notMerged) {
                message += file.getName() + "\n";
            }
            message += "\n";
        }

        // If there is a message to display, display it within a Yes/No dialog. Cancel the retry operation if
        // 'no' is chosen
        if(!message.equals("")) {
            message += "Retrying the " + command.toString().toLowerCase() + " command will likely not succeed. Do you still want to retry?";
            if(Messages.showYesNoDialog(myProject, message, "Retry Action", null) != 0) {
                Messages.showInfoMessage(myProject, "Action cancelled", "Synergy Messege");
                return;
            }
        }

        command.retry();
    }
}
