// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cli.helpers;

import com.telelogic.integration.intellij.ui.CMToolWindow;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMPlugin;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.project.Project;

import javax.swing.*;
import java.io.*;
import java.awt.*;

/**
 * Parses a buffered reader (generally CLI command output) and displays it in the tool window's CLI output console.
 * It also searches for information from Synergy regarding the CCM_ADDR environment variable, which is required
 * by the plugin. When this variable is outputted by Synergy, the plugin can change to a started state and allow
 * the user access to session-dependent functionality.
 */
public class ReaderThread extends Thread {
    Project         project;
    BufferedReader  br;
    CMToolWindow    toolWindow;
    Process         process;
    boolean         warning = false;
    String          name;

    /**
     * Default constructor
     * @param br The CLI output to be processed
     */
    public ReaderThread(Project project, Process process, BufferedReader br, String name) {
        this.project = project;
        this.br = br;
        this.toolWindow = CMPlugin.getInstance(project).toolWindow;
        this.process = process;
        this.name = name;
    }

    /**
     * Processes the buffered reader and looks for the CCM_ADDR environment variable. When the variable is found,
     * it executes the RunStartupCommands thread. If it finds an error in the buffered reader output, it
     * assumes the session did not start and invokes the tool window's sessionStartFailure() method.
     */
    public void run() {
        this.setPriority(Thread.NORM_PRIORITY);
        String l;
        // This is for password errors. The counter is the number of tries, the maxTries is the maximum number of times
        // the plugin will ask for a password before destroying the process
        int counter = 0;
        int maxTries = 3;

        try {
            while ((l = br.readLine()) != null) {

                final String line = l;

                // Process the line and look for errors
                if(line.contains("CCM_ADDR")) {
                    new RunStartupCommands(project, line).start();
                }
                else if(line.contains("Warning:") || line.contains("Fatal:")) {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            toolWindow.sessionStartFailure();
                        }
                    });
                }
                else if(line.contains("then also check if ccm_ess is running.")) {
                    // This is the last time of a typical username/password error message.  The process should
                    // be stopped now
                    process.destroy();
                }
                // We don't want to show this line in the console window.  It indicates the session won't start,
                // so the process is also destroyed
                if(line.contains("password:")) {
                    //process.destroy();
                    break;
                    /*
                    BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(process.getOutputStream()));

                    String password = null;

                    if (counter < maxTries) {
                        password = Messages.showInputDialog("Unable to validate password. Please try again:",
                                                            "Password Verification", null);
                    }

                    if(counter >= maxTries || password == null) {
                        process.destroy();
                        toolWindow.sessionStartFailure();
                        break;
                    }

                    counter++;   */

           
                }
                // Send the line to the console window
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        toolWindow.appendConsole(line);
                    }
                });
            }
        } catch (IOException e) { new CMExceptionHandler(e); }

    }
}
