// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cm;

import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.components.State;
import com.intellij.openapi.components.Storage;
import com.intellij.openapi.components.PersistentStateComponent;
import com.intellij.openapi.project.Project;
//import com.intellij.openapi.util.JDOMExternalizable;
import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.util.WriteExternalException;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.wm.StatusBar;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.util.xmlb.XmlSerializerUtil;
import com.telelogic.integration.intellij.cli.PropertiesCommand;
import com.telelogic.integration.intellij.ui.StatusBarHandler;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.NonNls;
import org.jdom.Element;

import javax.swing.*;
import java.util.HashMap;
import java.io.File;

// Persistent state implementation
@State(name = CMConfiguration.COMPONENT_NAME, storages = {@Storage(id = "synergy", file = "$WORKSPACE_FILE$")})

/**
 * Contains all major plugin-wide configuration information. This includes session-specific config such as the CCM_ADDR
 * environment variable passed to the plugin by Synergy. All project-specific configuration that is not tied to specific
 * Synergy sessions are saved via JDOMExternalizable
 */
public class CMConfiguration implements ProjectComponent, PersistentStateComponent<Element> {
    public static final String COMPONENT_NAME = "CMConfiguration";
    private String username  = "";
    private String password  = "";
    private String database  = "";
    private String engine    = "";
    private String path      = "";
    private String cmEnvPath = "";
    private String cmDelim   = "-";
    public  boolean SESSION_STARTED = false;
    public  boolean isGuiSession    = false;
    private HashMap<String,String>   options = new HashMap<String,String>();
    private String optionTabCheckout = "modified";
    private String optionTabTask     = "properties";
    private String optionTabAdd      = "prompt";
    private String defaultTask = "";
    private VirtualFile selectedFile = null;
    public HashMap<String,String> selectedFileProperties = null;
    private HashMap<String,String> logFiles = new HashMap<String,String>();

    private JLabel currentTaskDisplay = new JLabel();

    //private static final Logger LOG = Logger.getInstance(CMConfiguration.class.getName());

    private Project myProject;

    /**
     * Main constructor
     * @param project The active IDEA project
     */
	public CMConfiguration(Project project) { myProject = project; }

    /**
     * Required by ProjectComponent
     */
    public void initComponent()     { }

    /**
     * Required by ProjectComponent
     */
    public void disposeComponent()  { }

    /**
     * Returns the name of the config component. Required by ProjectComponent and used by IDEA
     */
    @NotNull
    public String getComponentName() { return CMConfiguration.COMPONENT_NAME; }

    /**
     * Required by ProjectComponent
     */
    public void projectOpened() { }

    /**
     * Required by ProjectComponent
     */
    public void projectClosed() { }

    /**
     * Getter method
     * @return Username
     */
    public String       getUser()               { return username;          }

    /**
     * Getter method
     * @return Password
     */
	public String       getPassword()           { return password;          }

    /**
     * Getter method
     * @return Database path
     */
    public String       getDB()                 { return database;          }

    /**
     * Getter method
     * @return Engine host
     */
	public String       getEngine()             { return engine;            }

    /**
     * Getter method
     * @return Path to ccm.exe
     */
    public String       getPath()               { return path;              }

    /**
     * Getter method
     * @return The active IDEA project
     */
    public Project      getMyProject()          { return myProject;         }

    /**
     * Getter method
     * @return The path to the Synergy session
     */
    public String       getCmEnvPath()          { return cmEnvPath;         }

    /**
     * Getter method
     * @return Value of the task option selection (in the option tab)
     */
    public String       getOptionTabTask()      { return optionTabTask;     }

    /**
     * Getter method
     * @return Value of the add option selection (in the option tab)
     */
    public String       getOptionTabAdd()       { return optionTabAdd;      }

    /**
     * Getter method
     * @return Value of the checkout option selection (in the option tab)
     */
    public String       getOptionTabCheckout()  { return optionTabCheckout; }

    /**
     * Getter method
     * @return Default/current task
     */
    public String          getDefaultTask()        { return defaultTask;       }

    /**
     * Getter method
     * @return Selected file
     */
    public VirtualFile  getSelectedFile()       { return selectedFile;      }

    /**
     * Provides access to the options collection
     * @param key The name of the option desired
     * @return The desired value of the option
     */
    public String       getOption(String key) {
        if(options.containsKey(key)) {
            return options.get(key);
        }
        else {
            return null;
        }
    }

    //TODO: Find log file location
    /**
     * Looks for and returns the path of the logfiles being used by Synergy. This varies on every system according to
     * each user's Synergy configuration (ccm.ini)
     * @return A collection containing all log files with their filenames as keys
     */
    public HashMap<String,String> getLogFiles() {
        // Get home directory
        // Look in user home
        // If ccm.ini exists, read it, look for Home = ...
        // If there, get value
        // If not, do same in CM home
        

        // Check for log files
        logFiles.put("ccm_ui.log", "c:/cm/ccm_ui.log");

        return logFiles;
    }

    public String getCmDelim() {
        return cmDelim;
    }

    public void setCmDelim(String cmDelim) {
        this.cmDelim = cmDelim;
    }
    
    /**
     * Setter method
     * @param username Username
     */
    public void     setUser(String username)            { this.username = username; }

    /**
     * Setter method
     * @param password Password
     */
    public void     setPassword(String password)        { this.password = password; }

    /**
     * Setter method
     * @param database Database path
     */
	public void     setDB(String database)              { this.database = database; }

    /**
     * Setter method
     * @param engine Engine host
     */
	public void     setEngine(String engine)            { this.engine = engine;     }

    /**
     * Setter method
     * @param path Path to ccm.exe
     */
    public void     setPath(String path)                { this.path = path;         }

    /**
     * Setter method
     * @param path  Session path
     */
    public void     setCmEnvPath(String path)           { this.cmEnvPath = CMUtil.getCmEnvPath(path); }

    /**
     * Setter method
     * @param key Option name
     * @param val Option value
     */

    public void     setOption(String key, String val)   { options.put(key, val);    }

    /**
     * Setter method
     * @param val Value for the task radio button group in the options tab
     */
    public void     setOptionTabTask(String val)        { this.optionTabTask = val; }

    /**
     * Setter method
     * @param val Value for the add radio button group in the options tab
     */
    public void     setOptionTabAdd(String val)         { this.optionTabAdd = val;  }

    /**
     * Setter method
     * @param val Value for the checkout radio button group in the options tab
     */
    public void     setOptionTabCheckout(String val)    { this.optionTabCheckout = val; }

    /**
     * Setter method
     * @param val The default/current task ID
     */
    public void     setDefaultTask(String val)  { 
        this.defaultTask = val;
    }


    /**
     * Sets the current file selected in the IDEA editor. This method also gathers and stores information about the
     * file from Synergy if a session is active
     * @param val The selected file
     */
    public void     setSelectedFile(VirtualFile val)    {
        this.selectedFile = val;
        if(this.selectedFile != null && SESSION_STARTED) {
            this.selectedFileProperties = new PropertiesCommand(selectedFile, myProject).get();
        }
        else {
            this.selectedFileProperties = new HashMap<String,String>();
        }
        CMPlugin.getInstance(myProject).toolWindow.updateCurrentFileTab(val);
    }

    /**
     * Tells the plugin to refresh the information it has from Synergy regarding the currently-selected file. This is
     * useful when the properties of the file have been changed while the file is in focus in the editor. Examples are
     * when the file is checked in/out or when it is associated with a Synergy task.
     */
    public void     updateSelectedFileProperties() {
        this.selectedFileProperties = new PropertiesCommand(selectedFile, myProject).get();
        //TODO: Put this somewhere sane
        
    }

    /**
     * Gets and returns an instance of the configuration. This is called by almost all classes in the plugin as a means
     * to gather information about the Synergy session and access instances of other plugin classes
     * @param project The active IDAE project
     * @return An instance of CMConfiguration
     */
    public static   CMConfiguration getInstance(Project project) {
      return project.getComponent(CMConfiguration.class);
    }

    /**
     * Evaluates and returns whether a default/current Synergy task is set. This does not call the Synergy CLI, rather
     * it checks with the plugin config. Its purpose is to determine whether a CLI action can go ahead or if the user
     * needs to first choose a task to work under.
     * @return Whether a task is set
     */
    public boolean isTaskSet() {
        return (!getDefaultTask().equals(""));
    }

    /**
     * Checks the user-provided config information for completeness and correctness. It only looks at whether all fields
     * are completed and the path to Synergy points to a file named "ccm.exe". It does not verify any data such as
     * the database and host names with Synergy.
     * @return Whether the check passed
     */
    public boolean checkSettings() {
        if (!username.equals("") && !password.equals("") && !database.equals("") && !engine.equals("") && !path.equals("")) {
            File ccm = new File(path);
            if(ccm.exists() && ccm.getName().equals("ccm.exe")) {
                return true;
            }
        }
        return false;
    }

    /**
     * Checks whether the path to Synergy provided by the user is correct. If a problem is found, the type of problem is
     * returned in the form of a human-readable error message that can be displayed in a dialog. If no problem is found,
     * the message is null
     * @return The error message, if any error with the ccm.exe path is discovered
     */
    public String checkCmdPath() {
        String message;

        if (getPath().length() == 0) {
            message = "Path is not set";
        } else {
            File ccmPath = new File(getPath());

            if (!ccmPath.exists())          { message = "Path is incorrect";}
            else if (ccmPath.isDirectory()) { message = "Path is to a directory, not ccm.exe"; }
            else                            { message = "Path is correct"; }
        }

        return message;
    }

    /**
     * JDOMExternalizable Methods
     * These store and retrieve config data to/from the project's IWS file in XML format.
     * *//*
    public void readExternal(Element element) throws InvalidDataException {

        //DefaultJDOMExternalizer.readExternal(this, element);

        Element config = element.getChild("config");
        if(config != null) {
            if(config.getAttribute("username") != null)  { setUser(config.getAttribute("username").getValue());       }
            if(config.getAttribute("password") != null)  { setPassword (config.getAttribute("password").getValue());  }
            if(config.getAttribute("db") != null)        { setDB(config.getAttribute("db").getValue());               }
            if(config.getAttribute("engine") != null)    { setEngine(config.getAttribute("engine").getValue());       }
            if(config.getAttribute("path") != null)      { setPath(config.getAttribute("path").getValue());           }
            //if(config.getAttribute("addOptions") != null)   { config.getAttribute("options").;       }
        }

        Element options = element.getChild("options");
        if(options != null) {
            if(options.getAttribute("task") != null)        { setOptionTabTask(options.getAttribute("task").getValue()); }
            if(options.getAttribute("add") != null)         { setOptionTabAdd(options.getAttribute("add").getValue()); }
            if(options.getAttribute("checkout") != null)    { setOptionTabCheckout(options.getAttribute("checkout").getValue()); }

            if(options.getAttribute("defaultTask") != null) {
                try {setDefaultTask(options.getAttribute("defaultTask").getValue()); }
                catch (Exception e) { setDefaultTask(""); }
            }
        }
    }*/

    /**
     * Stores user options that aren't tied to a specific Synergy session
     * @param //element Used by IDEA to call this method, however it is not used by this implmentation
     * @throws WriteExternalException
     */
    /*
    public void writeExternal(Element element) throws WriteExternalException {
        //DefaultJDOMExternalizer.writeExternal(this, element);

        Element config = new Element("config");
            config = config.setAttribute("username",    getUser()       );
            config = config.setAttribute("password",    getPassword()   );
            config = config.setAttribute("db",          getDB()         );
            config = config.setAttribute("engine",      getEngine()     );
            config = config.setAttribute("path",        getPath()       );
        element.addContent(config);

        Element options = new Element("options");
            options = options.setAttribute("task", getOptionTabTask());
            options = options.setAttribute("add", getOptionTabAdd());
            options = options.setAttribute("checkout", getOptionTabCheckout());
            options = options.setAttribute("defaultTask", getDefaultTask());
        element.addContent(options);
        //DEBUG: Messages.showMessageDialog("writing", "jdom", Messages.getInformationIcon());
    }*/

    public Element getState() {
        Element element = new Element(CMConfiguration.COMPONENT_NAME);

        Element config = new Element("config");
            config = config.setAttribute("username",    getUser()       );
            config = config.setAttribute("password",    getPassword()   );
            config = config.setAttribute("db",          getDB()         );
            config = config.setAttribute("engine",      getEngine()     );
            config = config.setAttribute("path",        getPath()       );
        element.addContent(config);

        Element options = new Element("options");
            options = options.setAttribute("task", getOptionTabTask());
            options = options.setAttribute("add", getOptionTabAdd());
            options = options.setAttribute("checkout", getOptionTabCheckout());
            options = options.setAttribute("defaultTask", getDefaultTask());
        element.addContent(options);

        return element;
    }

    public void loadState(@NonNls Element element) {
        //XmlSerializerUtil.copyBean(state, this);
        Element config = element.getChild("config");
        if(config != null) {
            if(config.getAttribute("username") != null)  { setUser(config.getAttribute("username").getValue());       }
            if(config.getAttribute("password") != null)  { setPassword (config.getAttribute("password").getValue());  }
            if(config.getAttribute("db") != null)        { setDB(config.getAttribute("db").getValue());               }
            if(config.getAttribute("engine") != null)    { setEngine(config.getAttribute("engine").getValue());       }
            if(config.getAttribute("path") != null)      { setPath(config.getAttribute("path").getValue());           }
            //if(config.getAttribute("addOptions") != null)   { config.getAttribute("options").;       }
        }

        Element options = element.getChild("options");
        if(options != null) {
            if(options.getAttribute("task") != null)        { setOptionTabTask(options.getAttribute("task").getValue()); }
            if(options.getAttribute("add") != null)         { setOptionTabAdd(options.getAttribute("add").getValue()); }
            if(options.getAttribute("checkout") != null)    { setOptionTabCheckout(options.getAttribute("checkout").getValue()); }

            if(options.getAttribute("defaultTask") != null) {
                try {setDefaultTask(options.getAttribute("defaultTask").getValue()); }
                catch (Exception e) { setDefaultTask(""); }
            }
        }  
    }

}