// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cm;

import com.intellij.openapi.vcs.diff.DiffProvider;
import com.intellij.openapi.vcs.history.VcsRevisionNumber;
import com.intellij.openapi.vcs.history.VcsFileRevision;
import com.intellij.openapi.vcs.changes.ContentRevision;
import com.intellij.openapi.vcs.FilePath;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.diff.*;
import com.intellij.vcsUtil.VcsUtil;
import com.intellij.peer.PeerFactory;
import com.telelogic.integration.intellij.cli.AttributeCommand;
import com.telelogic.integration.intellij.cli.History;
import com.telelogic.integration.intellij.cli.PropertiesCommand;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;

public class CMDiffProvider implements DiffProvider {

    CMPlugin plugin;
    Project project;

    public CMDiffProvider(CMPlugin plugin) {
        this.plugin = plugin;
        this.project = plugin.getProject();
    }

    /**
     * This is the revision in the user's project
     * @param virtualFile
     * @return
     */
    @Nullable
    public VcsRevisionNumber getCurrentRevision(VirtualFile virtualFile) {
        String version = new AttributeCommand(project, virtualFile.getPath()).get(AttributeCommand.VERSION);
        return new CMRevisionNumber(version);
    }

    /**
     * This is the predecessor to the current revision
     * @param virtualFile
     * @return
     */
    @Nullable
    public VcsRevisionNumber getLastRevision(VirtualFile virtualFile) {
        LinkedHashMap<History.HistoryObject.Key, History.HistoryObject> historyData;
        ArrayList<String> predecessorList;
        History.HistoryObject currentVersion;
        String predecessor;
        FilePath path = VcsUtil.getFilePath(virtualFile.getPath());

        // Get the object name and version
        String objectName = new PropertiesCommand(virtualFile, project).get("%objectname", path.getPath());
        String version = new AttributeCommand(project, objectName).get(AttributeCommand.VERSION);

        // Get a list of revisions for this object
        historyData = History.getRevisionData(project, path);

        // Get the relevant revision based on the objectName
        currentVersion = historyData.get(new History.HistoryObject.Key(objectName));

        // Get the list of predecessors to the current version
        try {
            predecessorList = currentVersion.getPredecessors();
            //predecessors = (String[])currentVersion.getPredecessors().toArray();
        } catch(NullPointerException ex) {
            return null;
        }

        // Return the predecessor
        // This looks messy, but the reality is there are few scenarios where a working object will have more than one
        // predecessor. More than one predecessor indicates a merge operation and at that point it's impossible to
        // open a diff window where IDEA is expecting one predecessor

        if(!predecessorList.isEmpty()) {
            predecessor = predecessorList.get(0);

            History.HistoryObject predecessorData = historyData.get(new History.HistoryObject.Key(predecessor));

            return CMRevisionNumber.getInstance(predecessorData);
        }
        else {
            return null;
        }
    }

    /**
     * The content that corresponds to the revision number and file
     * @param vcsRevisionNumber
     * @param virtualFile
     * @return
     */
    @Nullable
    public ContentRevision createFileContent(VcsRevisionNumber vcsRevisionNumber, VirtualFile virtualFile) {
        return new CMContentRevision(project, VcsUtil.getFilePath(virtualFile.getPath()), vcsRevisionNumber);
    }

    public static void doSimpleDiffRequest(Project project, History.HistoryObject object1, History.HistoryObject object2) {
        SimpleDiffRequest diffRequest = new SimpleDiffRequest(project, "Synergy Compare");
        DiffTool diffTool = DiffManager.getInstance().getDiffTool();

        DiffContent c1 = new SimpleContent(new String(object1.getSource()));
        DiffContent c2 = new SimpleContent(new String(object2.getSource()));

        diffRequest.setContents(c1, c2);
        diffRequest.setContentTitles(object1.getObjectName(), object2.getObjectName());

        if(diffTool.canShow(diffRequest)) {
            diffTool.show(diffRequest);
        }
    }

    public static void doSimpleDiffRequest(Project project, VirtualFile file1, VirtualFile file2, String title) {
        SimpleDiffRequest diffRequest = SimpleDiffRequest.compareFiles(file1, file2, project, title);
        DiffTool diffTool = DiffManager.getInstance().getDiffTool();

        if(diffTool.canShow(diffRequest)) {
            diffTool.show(diffRequest);
        }
    }
}
