// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cm;

import com.intellij.openapi.vfs.*;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vcs.FilePath;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.progress.Task;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.application.ModalityState;
import com.intellij.vcsUtil.VcsUtil;
import com.intellij.vcsUtil.VcsRunnable;
import com.telelogic.integration.intellij.cli.*;

import java.io.IOException;
import java.io.File;
import java.util.ArrayList;
import java.lang.reflect.Array;
import java.awt.*;

import org.jetbrains.annotations.Nullable;

public class CMLocalFileOperationsHandler implements LocalFileOperationsHandler {

    Project myProject;
    CMPlugin myPlugin;

    public CMLocalFileOperationsHandler(Project p){
        myProject = p;
        myPlugin = CMPlugin.getInstance(myProject);
    }

    public CMLocalFileOperationsHandler(CMPlugin plugin){
        myPlugin = plugin;
        myProject = plugin.getProject();
    }

    // ccm move <oldpath> <newpath>
    // If virtualFile is a directory, go into the structure and look for any writable files. If any are found,
    // complain.  Build a list of these files, then do a ccm properties.
    // -uncontrolled: abort the move (or find a way to move the uncontrolled object along with all other files)
    // -associated with a different task: abort the move
    // May need to block something here when it comes to IDEA's handling of all objects under the directory
    public synchronized boolean move(final VirtualFile virtualFile, final VirtualFile virtualFile1) throws IOException {
        if(!VcsUtil.isFileForVcs(virtualFile, myProject, myPlugin)){
            return false;
        }

        final boolean[] success = new boolean[]{false};

        /*Task runner = new Task.Modal(myProject, "Performing Move", false) {
            public void run(ProgressIndicator indicator) {
                indicator.setIndeterminate(true);
                if(new PropertiesCommand(virtualFile, myProject).isControlled()) {
                    new MoveCommand(myProject, virtualFile, virtualFile1, null);
                    // TODO: need success check here... one that is synchronized
                    success[0] = virtualFile1.isInLocalFileSystem();
                }
                if(indicator.isRunning()) {
                    indicator.stop();
                }
            }
        };

        runner.queue();*/

        final Runnable runnable = new CMRunnable(0) {

            public void run() {

                try {
                    if(new PropertiesCommand(virtualFile, myProject).isControlled()) {
                        new MoveCommand(myProject, virtualFile, virtualFile1, null);
                        // TODO: need success check here... one that is synchronized
                        success[0] = virtualFile1.isInLocalFileSystem();
                    }
                }
                catch(VcsException e) {
                    Messages.showErrorDialog(e.getMessage(), "Rename Error");
                }
                finally {
                    stopIndicator();
                }
            }
        };

        ProgressManager.getInstance().runProcessWithProgressSynchronously(runnable, "Performing Rename", false, myProject);

        return success[0];
    }

    // If user wants to get the previous version: ccm delete -replace <file>
    // If user wants to remove the object from the project: ccm unuse <file>
    // If user wants to completely remove the object from the database: ccm unuse -delete <file>
    public boolean delete(VirtualFile virtualFile) throws IOException {
        if(!VcsUtil.isFileForVcs(virtualFile, myProject, myPlugin)){
            return false;
        }

        if(new PropertiesCommand(virtualFile, myProject).isControlled()) {

            ArrayList<FilePath> filePaths = new ArrayList<FilePath>();

            filePaths.add(VcsUtil.getFilePath(virtualFile.getPath()));

            String[] options = new String[]{
                    "Replace with predecessor",
                    "Remove from project",
                    "Remove from project and delete"};

            int choice = Messages.showChooseDialog(
                    "What would you like Synergy to do to this object after deletion?",
                    "Synergy: Deleting " + virtualFile.getName(),
                    options,
                    options[0], null);

            switch(choice){
                case 0:
                            new Delete(filePaths, myProject, null);
                            break;
                case 1:
                            new UnuseCommand(virtualFile, myProject, null, false);
                            break;
                case 2:
                            new UnuseCommand(virtualFile, myProject, null, true);
                            break;
                default:
                            return true; // On cancel, don't remove the file, but tell IDEA it was completed
            }
        }
        return false;
    }

    // Add another object to Synergy? Copy the object to another location in Synergy? Leave it alone?
    @Nullable
    public File copy(VirtualFile virtualFile, VirtualFile virtualFile1, String s) throws IOException {
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    // ccm move <oldpath> <newpath> [-task num]
    public boolean rename(final VirtualFile oldName, final String newName) throws IOException {

        if(!VcsUtil.isFileForVcs(oldName, myProject, myPlugin)){
            return false;
        }

        final boolean[] success = new boolean[]{false};

        //ReadonlyStatusHandler.getInstance(myProject).ensureFilesWritable(oldName);


        /*final Task task = new Task.Modal(myProject, "Performing Rename", false) {

            public void run(ProgressIndicator indicator) {

                indicator.setIndeterminate(true);
                try {
                PropertiesCommand properties = new PropertiesCommand(oldName, myProject);

                    if(properties.isControlled()) {
                        if(!properties.isCheckedOut()) {
                            // Check out the file first
                            new CheckOutCommand(myProject, true, false, oldName);
                        }
                    }
                    

                    if(new PropertiesCommand(oldName, myProject).isControlled()) {
                        new RenameCommand(myProject, oldName, newName, null);
                        success[0] = true;
                    }
                }
                catch(VcsException e) {
                    Messages.showErrorDialog(e.getMessage(), "Rename Error");
                }
            }
        };*/

        final Runnable runnable = new CMRunnable(0) {

            public void run() {

                try {
                PropertiesCommand properties = new PropertiesCommand(oldName, myProject);

                    if(properties.isControlled()) {
                        if(!properties.isCheckedOut()) {
                            // Check out the file first
                            new CheckOutCommand(myProject, true, false, oldName);
                        }
                    }


                    if(new PropertiesCommand(oldName, myProject).isControlled()) {
                        new RenameCommand(myProject, oldName, newName, null);
                        success[0] = true;
                    }
                }
                catch(VcsException e) {
                    Messages.showErrorDialog(e.getMessage(), "Rename Error");
                }
                finally {
                    stopIndicator();
                }
            }
        };

        ProgressManager.getInstance().runProcessWithProgressSynchronously(runnable, "Performing Rename", false, myProject);

/*
        final Runnable runner = new Runnable() {
            public void run() {

            }
        }*/

        /*if(ApplicationManager.getApplication().isDispatchThread()) {
            task.queue();
        }
        else {
            EventQueue.invokeLater(new Runnable() {
                public void run() {
                    task.queue();
                }
            });
        }*/

        return success[0];
    }

    // ccm create <path/to/new/file.ext> (if confirmed by user)
    public boolean createFile(VirtualFile parent, String newFile) throws IOException {
        if(!VcsUtil.isFileForVcs(parent, myProject, myPlugin)){
            return false;
        }

        //int choice = Messages.showYesNoDialog(myProject, "Do you want Synergy to control " + newFile + " ?", "Synergy", null);

        if(confirmAddToSynergy(newFile)) {
            // If the user wants the file under Synergy control
            return AddCommand.createFile(myProject, parent, newFile, null);
        }
        return false;
    }

    // ccm create -t <path/to/new/dir> (if confirmed by user)
    public boolean createDirectory(VirtualFile parent, String newDir) throws IOException {
        if(!VcsUtil.isFileForVcs(parent, myProject, myPlugin)){
            return false;
        }

        //int choice = Messages.showYesNoDialog(myProject, "Do you want Synergy to control " + newDir + " ?", "Synergy", null);

        if(confirmAddToSynergy(newDir)) {
            // If the user wants the dir under Synergy control
            return AddCommand.createDir(myProject, parent, newDir, null);

        }
        
        return false;
    }

    private boolean confirmAddToSynergy(String fileName) {
        String message = "Do you want Synergy to control " + fileName + " ?";
        String title = "Synergy";

        return (Messages.showYesNoDialog(myProject, message, title , null) == 0);
    }

}
