// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cm;

import com.intellij.openapi.vcs.merge.MergeProvider;
import com.intellij.openapi.vcs.merge.MergeData;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.vcs.AbstractVcsHelper;
import com.intellij.openapi.vcs.history.VcsRevisionNumber;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.ReadonlyStatusHandler;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.ui.DialogWrapper;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.application.ModalityState;
import com.intellij.openapi.diff.*;
import com.intellij.openapi.util.text.StringUtil;
import com.intellij.vcsUtil.VcsUtil;
import com.intellij.peer.PeerFactory;
import com.telelogic.integration.intellij.cli.AttributeCommand;
import com.telelogic.integration.intellij.cli.History;
import com.telelogic.integration.intellij.cli.PropertiesCommand;
import com.telelogic.integration.intellij.cli.RelateCommand;
import com.telelogic.integration.intellij.ui.dialogs.MergeCandidates;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;

/**
 * Handles merge operations
 */
public class CMMergeProvider implements MergeProvider {

    private Project project;
    private HashMap<VirtualFile,LinkedHashMap<History.HistoryObject.Key, History.HistoryObject>> fileData;
    private HashMap<VirtualFile,HashMap<String,String>> mergeSelections;
    private static final String ORIGINAL    = "o";
    private static final String LAST        = "l";
    private static final String CURRENT     = "c";

    public CMMergeProvider(final Project project) {
        this.project            = project;
        this.fileData           = new HashMap<VirtualFile,LinkedHashMap<History.HistoryObject.Key, History.HistoryObject>>();
        this.mergeSelections    = new HashMap<VirtualFile,HashMap<String,String>>();
    }

    /**
     * Populates a MergeData object with byte arrays representing the original, last and current files.  This is a 3-way
     * merge.
     * Original = The predecessor to the working object
     * Last = The object to be merged with
     * Current = The working object in the user's work area
     * @param virtualFile The file used in the merge operation
     * @return The merge data
     * @throws VcsException
     */
    @NotNull
    public MergeData loadRevisions(VirtualFile virtualFile) throws VcsException {
        final LinkedHashMap<History.HistoryObject.Key, History.HistoryObject> historyData
                = History.getRevisionData(project, VcsUtil.getFilePath(virtualFile.getPath()));
        final HashMap<String,String> mergeSelection = new HashMap<String,String>();   // file type -> objectName
        final MergeData mergeData = new MergeData();

        final String currentObjectName              = new PropertiesCommand(virtualFile, project).get("%objectname", virtualFile.getPath());
        final History.HistoryObject currentObject   = historyData.get(new History.HistoryObject.Key(currentObjectName));
        final ArrayList<String>     successors;
        final History.HistoryObject originalObject;
        final History.HistoryObject lastObject;

        // Add the history data for this file to a hashmap for future reference
        fileData.put(virtualFile, historyData);


        // We have the history information for the current object. Now we need to load the predecessor. We will get its
        // source and also its successors
        final ArrayList<String> predecessors = currentObject.getPredecessors();
        if (predecessors.size() == 1) {

            originalObject = historyData.get(new History.HistoryObject.Key(predecessors.get(0)));
            originalObject.loadSource();
            mergeData.ORIGINAL  = originalObject.getSource();

            successors = originalObject.getSuccessors();
            LinkedList<History.HistoryObject> candidateData = new LinkedList<History.HistoryObject>(); // All successor data except for successors in the working state

            // Insert all non-working successors into the candidateData list
            // Also remove the current object if it shows up in the candidate list.  This will happen if the current
            // object is not in the working state. We will eventually check this file out, but only after the user has
            // selected a candidate to merge with. If it was checked out now, code would be needed to undo the checkout
            // in the event the user cancels the merge before making a candidate selection.
            for(String successor : successors) {
                final History.HistoryObject candidateObject = historyData.get(new History.HistoryObject.Key(successor));
                if(!candidateObject.getStatus().equals("working") && !candidateObject.getObjectName().equals(currentObjectName)) {
                    candidateData.add(candidateObject);
                }
            }

            if(candidateData.size() < 1) {
                throw new VcsException("There are no merge candidates for this object.");
            }

            // Display a dialog with the candidates and get the selected candidate back
            // Catches whether a user cancelled the merge in the candidate selection dialog
            String selection = MergeCandidates.getSelection(virtualFile, candidateData, historyData);
            if(selection == null) {
                throw new VcsException("Merge cancelled");
            }

            // Load the selection data
            lastObject = historyData.get(new History.HistoryObject.Key(selection));
            lastObject.loadSource();
            mergeData.LAST = lastObject.getSource();
            mergeData.LAST_REVISION_NUMBER = CMRevisionNumber.getInstance(lastObject);

        }
        else {
            // Either the # of predecessors is 0 or > 1.  Either way, merging is going to be a problem.  Recommend using
            // Synergy to merge the object

            throw new VcsException("An error occurred trying to merge this object. Please use the Synergy client to " +
                    "resolve this issue.");
        }


        //Clear any read-only status on the CURRENT file
        if(!virtualFile.isWritable()) {
            ReadonlyStatusHandler.OperationStatus fileStatus = ReadonlyStatusHandler.getInstance(project).ensureFilesWritable(virtualFile);
            // If the CURRENT file is still read-only at this point, the user has rejected the clear read-only status dialog
            // Cancel the merge and inform the user why.
            if(fileStatus.hasReadonlyFiles()) {
                throw new VcsException("Merge results cannot be written to a read-only file.");
            }
            else {
                // The "current object" has changed since this method started.  We need to get the new objectname from
                // Synergy so it can be referenced as the successor during the final resolution of the merge operation
                String newObjectName = new PropertiesCommand(virtualFile, project).get("%objectname", virtualFile.getPath());
                assert(newObjectName != null);
                mergeSelection.put(CURRENT, newObjectName);
            }
        }
        else {
            mergeSelection.put(CURRENT, currentObjectName);
        }

        mergeSelection.put(ORIGINAL, originalObject.getObjectName());
        mergeSelection.put(LAST, lastObject.getObjectName());

        try {
            mergeData.CURRENT = virtualFile.contentsToByteArray();
        } catch (IOException e) {
            throw new VcsException("An error occurred retrieving the local file.");
        }

        mergeSelections.put(virtualFile, mergeSelection);

        return mergeData;
    }

    /**
     * Populates a MergeData object with byte arrays representing the original, last and current files.  This is a 3-way
     * merge. This is used when the user has already selected an object to merge with the current file
     * Original = The predecessor to the working object
     * Last = The object to be merged with
     * Current = The working object in the user's work area
     * @param virtualFile The file used in the merge operation
     * @param mergeObjectName The object name used as the original
     * @return The merge data
     * @throws VcsException
     */
    @NotNull
    public MergeData loadPreselectedMergeData(VirtualFile virtualFile, String mergeObjectName) throws VcsException {
        final LinkedHashMap<History.HistoryObject.Key, History.HistoryObject> historyData
                = History.getRevisionData(project, VcsUtil.getFilePath(virtualFile.getPath()));
        final HashMap<String,String> mergeSelection = new HashMap<String,String>();   // file type -> objectName
        final MergeData mergeData = new MergeData();

        final String currentObjectName              = new PropertiesCommand(virtualFile, project).get("%objectname", virtualFile.getPath());
        final History.HistoryObject currentObject   = historyData.get(new History.HistoryObject.Key(currentObjectName));
        final ArrayList<String>     successors;
        final History.HistoryObject originalObject;
        final History.HistoryObject lastObject;

        // Add the history data for this file to a hashmap for future reference
        fileData.put(virtualFile, historyData);


        // We have the history information for the current object. Now we need to load the predecessor. We will get its
        // source and also its successors
        final ArrayList<String> predecessors = currentObject.getPredecessors();
        if (predecessors.size() == 1) {

            originalObject = historyData.get(new History.HistoryObject.Key(predecessors.get(0)));
            originalObject.loadSource();
            mergeData.ORIGINAL  = originalObject.getSource();

            successors = originalObject.getSuccessors();
            LinkedList<History.HistoryObject> candidateData = new LinkedList<History.HistoryObject>(); // All successor data except for successors in the working state

            // Insert all non-working successors into the candidateData list
            // Also remove the current object if it shows up in the candidate list.  This will happen if the current
            // object is not in the working state. We will eventually check this file out, but only after the user has
            // selected a candidate to merge with. If it was checked out now, code would be needed to undo the checkout
            // in the event the user cancels the merge before making a candidate selection.
            for(String successor : successors) {
                final History.HistoryObject candidateObject = historyData.get(new History.HistoryObject.Key(successor));
                if(!candidateObject.getStatus().equals("working") && !candidateObject.getObjectName().equals(currentObjectName)) {
                    candidateData.add(candidateObject);
                }
            }

            if(candidateData.size() < 1) {
                throw new VcsException("There are no merge candidates for this object.");
            }

            // Display a dialog with the candidates and get the selected candidate back
            // Catches whether a user cancelled the merge in the candidate selection dialog
            String selection = MergeCandidates.getSelection(virtualFile, candidateData, historyData);
            if(selection == null) {
                throw new VcsException("Merge cancelled");
            }

            // Load the selection data
            lastObject = historyData.get(new History.HistoryObject.Key(selection));
            lastObject.loadSource();
            mergeData.LAST = lastObject.getSource();
            mergeData.LAST_REVISION_NUMBER = CMRevisionNumber.getInstance(lastObject);

        }
        else {
            // Either the # of predecessors is 0 or > 1.  Either way, merging is going to be a problem.  Recommend using
            // Synergy to merge the object

            throw new VcsException("An error occurred trying to merge this object. Please use the Synergy client to " +
                    "resolve this issue.");
        }


        //Clear any read-only status on the CURRENT file
        if(!virtualFile.isWritable()) {
            ReadonlyStatusHandler.OperationStatus fileStatus = ReadonlyStatusHandler.getInstance(project).ensureFilesWritable(virtualFile);
            // If the CURRENT file is still read-only at this point, the user has rejected the clear read-only status dialog
            // Cancel the merge and inform the user why.
            if(fileStatus.hasReadonlyFiles()) {
                throw new VcsException("Merge results cannot be written to a read-only file.");
            }
            else {
                // The "current object" has changed since this method started.  We need to get the new objectname from
                // Synergy so it can be referenced as the successor during the final resolution of the merge operation
                String newObjectName = new PropertiesCommand(virtualFile, project).get("%objectname", virtualFile.getPath());
                assert(newObjectName != null);
                mergeSelection.put(CURRENT, newObjectName);
            }
        }
        else {
            mergeSelection.put(CURRENT, currentObjectName);
        }

        mergeSelection.put(ORIGINAL, originalObject.getObjectName());
        mergeSelection.put(LAST, lastObject.getObjectName());

        try {
            mergeData.CURRENT = virtualFile.contentsToByteArray();
        } catch (IOException e) {
            throw new VcsException("An error occurred retrieving the local file.");
        }

        mergeSelections.put(virtualFile, mergeSelection);

        return mergeData;
    }

    /**
     * This method is called when a merge is completed.  Synergy will then relate this merged file with the conflictor
     * as its successor
     * @param virtualFile This is the resulting merged file
     */
    public void conflictResolvedForFile(VirtualFile virtualFile) {
        String successor    = mergeSelections.get(virtualFile).get(CURRENT);
        String predecessor  = mergeSelections.get(virtualFile).get(LAST);

        RelateCommand relate = new RelateCommand(project);
        relate.doRelate(predecessor, successor);
    }

    /**
     * Checks the file type in Synergy to determine whether the file is binary.  If a file is considered ascii, return
     * false.  Otherwise, return true.
     * @param virtualFile
     * @return Whether the file is binary
     */
    public boolean isBinary(VirtualFile virtualFile) {
        String sourceType = new AttributeCommand(project, virtualFile.getPath()).get(AttributeCommand.HANDLE_SOURCE_AS);

        if(sourceType.equals("ascii")) {
            return false;
        }

        return true;
    }

    public static void doMerge(final Project project, final ArrayList<VirtualFile> files) {

/*        ArrayList<VirtualFile> readOnlyFiles = new ArrayList<VirtualFile>();

        for(VirtualFile file : files) {
            if(!file.isWritable()) {
                readOnlyFiles.add(file);
            }
        }
        if(readOnlyFiles.size() > 0) {
            ReadonlyStatusHandler.getInstance(project).ensureFilesWritable(readOnlyFiles.toArray(VirtualFile.EMPTY_ARRAY));
        }*/
        AbstractVcsHelper.getInstance(project).showMergeDialog(files, new CMMergeProvider(project));
    }

    /**
     * Run a merge on a single object with its selected parallel
     * This skips the dialog showing a selection of files to merge
     */
    public static boolean doSingleMerge(Project project, VirtualFile currentFile, History.HistoryObject currentObj,
                                     History.HistoryObject lastObj, History.HistoryObject originalObj) {
        ActionButtonPresentation actionButton = new ActionButtonPresentation(true, "Merge", true);
        String leftText;
        String rightText;
        String originalContent;
        String newObjectName = null;

        // Make sure the file to be used as merge result is writable
        // TODO:It might be good here to check the status, then check out if read-only.
        // If the file was checked out, we need to notify the show parallels dialog of the new object name
        ReadonlyStatusHandler.OperationStatus fileStatus =
                ReadonlyStatusHandler.getInstance(project).ensureFilesWritable(currentFile);

        if(fileStatus.hasUpdatedFiles()) {
            // The file was checked out. Get the new object name
            PropertiesCommand properties = new PropertiesCommand(currentFile, project);
            HashMap<String,String> results = properties.get("objectName:%objectname");
            if(results.containsKey("objectName")) {
                newObjectName = results.get("objectName");
            }
        }

        // Get string contents for all
        if(!currentFile.isDirectory()) {
            try {
                leftText = StringUtil.convertLineSeparators(new String(currentFile.contentsToByteArray()));
                rightText = StringUtil.convertLineSeparators(new String(lastObj.getSource()));
                originalContent = StringUtil.convertLineSeparators(new String(currentFile.contentsToByteArray()));

                MergeRequest mergeRequest = PeerFactory.getInstance().getDiffRequestFactory()
                        .createMergeRequest(leftText, rightText, originalContent, currentFile, project, actionButton);

                mergeRequest.setVersionTitles(new String[]{originalObj.getObjectName() + " (In work area)", "Merge Result", lastObj.getObjectName() + " (Parallel)"});
                mergeRequest.setWindowTitle("Merge " + currentFile.getName());

                DiffTool diffTool = DiffManager.getInstance().getDiffTool();

                if(diffTool.canShow(mergeRequest)) {
                    diffTool.show(mergeRequest);
                    if(mergeRequest.getResult() == DialogWrapper.OK_EXIT_CODE) {
                        // The file has been merged.  Run the relate command
                        // If the current file was checked out during this operation, we need to run the relate with
                        // the new object as the successor.  This is tested by checking whether newObjectName has a value.
                        RelateCommand relate = new RelateCommand(project);
                        if(newObjectName != null) {
                            relate.doRelate(lastObj.getObjectName(), currentObj.getObjectName());
                        }
                        else {
                            relate.doRelate(lastObj.getObjectName(), newObjectName);
                        }
                        return true;
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return false;
    }
}