// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cm;

import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.util.WriteExternalException;
//import com.intellij.openapi.util.DefaultJDOMExternalizer; // IDEA 7.0: Use PersistentStateComponent
//import com.intellij.openapi.util.JDOMExternalizable; // IDEA 7.0: Use PersistentStateComponent
import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.options.Configurable;
import com.intellij.openapi.vcs.*;
import com.intellij.openapi.vcs.diff.DiffProvider;
import com.intellij.openapi.vcs.history.VcsHistoryProvider;
import com.intellij.openapi.project.Project;
//import com.intellij.openapi.localVcs.LocalVcsServices; // IDEA 7.0: Use LocalHistory
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VirtualFileManager;
import com.intellij.openapi.vfs.LocalFileSystem;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.progress.Task;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.application.ModalityState;
import com.intellij.openapi.wm.StatusBar;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.vcsUtil.VcsUtil;
import com.intellij.util.concurrency.SwingWorker;
import com.telelogic.integration.intellij.ui.CMConfigurable;
import com.telelogic.integration.intellij.cli.Session;
import com.telelogic.integration.intellij.cli.PropertiesCommand;
import com.telelogic.integration.intellij.cli.CheckOutCommand;
import com.telelogic.integration.intellij.cli.Tasks;
import com.telelogic.integration.intellij.ui.CMToolWindow;
import com.telelogic.integration.intellij.ui.StatusBarHandler;
import org.jdom.Element;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import java.util.Map;
import java.util.ArrayList;

/**
 * Main class for the plugin
 * @author Luke Stahlman
 */
public class CMPlugin extends AbstractVcs implements ProjectComponent, EditFileProvider {
    public static String COMPONENT_NAME="Synergy";
    public static String PLUGIN_NAME="Synergy Plugin";
    private Project             myProject;
    private CMConfiguration     myConfig;
    public  CMToolWindow        toolWindow;
    public  VcsHistoryProvider  historyProvider;
    //public  CMVFSListener       vfsListener;
    //private CMVirtualFileSystem fileSystem;
    private CMLocalFileOperationsHandler fileHandler;
    private StatusBar           statusBar;

    private boolean activated = false;
    private boolean projectOpen = false;


    /**
     * Constructor used by IntelliJ IDEA to instantiate the plugin
     * @param project Active project
     * @param localVcsServices IDEA version control system servies (not used yet)
     */
    /* Removed for IDEA 7.0 compatibility
    public CMPlugin(Project project, LocalVcsServices localVcsServices) {
        super(project);
        myProject = project;
    }  */

    /**
     * IDEA 7.0 compatibility
     * @param project
     */
    public CMPlugin(Project project) {
        super(project);
        myProject = project;
        fileHandler = new CMLocalFileOperationsHandler(this);
        //LocalFileSystem.getInstance().registerAuxiliaryFileOperationsHandler(fileHandler);
    }

    /**
     * Component initialization
     */
    public void initComponent() {
        StatusBarHandler.getInstance(myProject).setStatus(StatusBarHandler.NOSESSION);
        StatusBarHandler.hide();
    }

    /**
     * Component disposal
     */
    public void disposeComponent() {
        //myCheckinEnvironment = null;
        StatusBarHandler.remove();
    }

    /**
     * Returns the component name of the plugin as required by BaseComponent
     * @return The name
     */
    @NotNull
    public  String      getComponentName()      { return PLUGIN_NAME; }

    /**
     * Returns the plugin name as required by AbstractVCS
     * @return The name
     */
    public  String      getName()               { return COMPONENT_NAME; }

    /**
     * Returns an instance of this plugin pertaining to a specific IDEA project
     * @param project The active IDEA project
     * @return The plugin
     */
    public  static      CMPlugin    getInstance(Project project) {
        //return (CMPlugin)project.getComponent(COMPONENT_NAME);
        return project.getComponent(CMPlugin.class);
    }

    /**
     * Returns an instance of CMConfigurable as required by AbstractVcs
     * @return CMConfigurable
     */
    public Configurable getConfigurable() {
        return new CMConfigurable( myProject );
    }

    /**
     * Executes commands relevant to the opening of an IDEA project. This includes loading config settings and
     * creating file and file editor listeners
     */
    public void projectOpened() {
        final ProjectLevelVcsManager vcsManager = ProjectLevelVcsManager.getInstance(getProject());

        projectOpen = true;

        myConfig = CMConfiguration.getInstance(myProject);
        //toolWindow = new CMToolWindow(myProject);

        // **** MOVE TO STARTSESSION ****
        // Attach the file listeners
        //CMFileListener myFileListener=new CMFileListener( getProject());
        //CMFileEditorListener myFileEditorListener=new CMFileEditorListener(getProject());
        //FileEditorManager.getInstance(getProject()).addFileEditorManagerListener(myFileEditorListener);
        //LocalFileSystem.getInstance().addVirtualFileListener( myFileListener );
        CMSessionMonitor sessionMonitor = new CMSessionMonitor(myConfig, this);

        // Register the file operations handler
        //LocalFileSystem.getInstance().registerAuxiliaryFileOperationsHandler(fileHandler);

        if(activated) {
            showToolWindow();
            showStatus();
        }
    }

    /**
     * Executes commands relevant ot the closing of a project. This essentially makes sure Synergy sessions aren't
     * unintentionally orphaned by the closing of IDEA or one of its projects
     */
    public void projectClosed() {

        projectOpen = false;

        //toolWindow.unregisterToolWindow();
        if(myConfig.SESSION_STARTED)
        {
            if(Messages.showYesNoDialog("Would you like to stop the active Synergy session?",
                    "Stop Synergy Session",
                    Messages.getQuestionIcon()) == 0) {
                Session mySession = new Session(myProject);
                mySession.stop();
            }
        }

    }

    public void activate() {
        super.activate();

        activated = true;


        // Register the file operations handler
        LocalFileSystem.getInstance().registerAuxiliaryFileOperationsHandler(fileHandler);
        CMFileEditorListener myFileEditorListener=new CMFileEditorListener(getProject());
        FileEditorManager.getInstance(getProject()).addFileEditorManagerListener(myFileEditorListener);

        if(projectOpen) {
            showToolWindow();
            showStatus();
        }
    }

    public void deactivate() {

        super.deactivate();
        activated = false;

        // Unregister the file operations handler so Synergy isn't continuing to eval file operations in other projects
        // This is throwing an exception in come cases, so a try/catch is used.  However, no action needs to be taken
        // when this exception occurs.
        try {
            LocalFileSystem.getInstance().unregisterAuxiliaryFileOperationsHandler(fileHandler);
        } catch (Exception e) {
            // Dummy catch.  Nothing really needs to be done here.
        }

        hideToolWindow();
        hideStatus();
    }

    /**
     * Handles the unexpected loss of a Synergy session
     * @param showError Whether to display an error message to the user
     */
    public void sessionLost(boolean showError) {
        myConfig.SESSION_STARTED = false;
        toolWindow.sessionStopped();
        if(showError) {
             toolWindow.appendConsole("Plugin Warning: Synergy Session Lost");
            Messages.showErrorDialog("The Synergy session has been lost. Please start a new session", "Synergy Error");
        }
        StatusBarHandler.getInstance(myProject).setStatus(StatusBarHandler.NOSESSION);
    }

    /**
     * Returns the name of the plugin as seen by the user
     * @return The plugin name
     */
    public String getDisplayName()  { return "Synergy";      }

    /**
     * Returns the project associated with the plugin
     * @return The active IDEA project
     */
    public Project getProject()     { return myProject; }

    /**
     * Handles the graceful checkout of a file that a user begins to edit. This is called when a file (presumably in a
     * non-working state) is determined by IDEA to be in the read-only state in the file system.
     * @param files The file(s) to be checked out
     * @throws VcsException
     */
    public void editFiles(final VirtualFile[] files ) throws VcsException
    {
        final CMConfiguration config = CMConfiguration.getInstance(getProject());

        // determine if file is already checked out?
        if(config.SESSION_STARTED) {
            // TODO: make a do...while loop here to allow a user to select a task before checkout
           // if(config.isTaskSet()) {
                new CheckOutCommand(files, myProject);
           // }
        }
        else {
            /*int choice = Messages.showYesNoDialog("No Synergy session was found. Would you like to start a session?", "No Session Found", null);
            if(choice == 0) {
                
            }*/
            throw new VcsException("No Synergy session found.");
        }
    }

    /**
     * A method required by EditFileProvider, however its actual use by IDEA is currently unknown
     * @return The request text
     */
    public String getRequestText()    {  return "File(s) Checked Out"; }

    /**
     * Returns the EditFileProvider for this project. In this case, CMPlugin is the provider
     * @return This plugin class
     */
    public EditFileProvider getEditFileProvider() {
        return this;
    }

    /**
     * Returns whether a session has already been started
     * @return Whether a session is started
     */
    public boolean isSessionStarted() {
        return this.myConfig.SESSION_STARTED;
    }


    public VcsHistoryProvider getVcsHistoryProvider() {
        if(historyProvider == null) {
            historyProvider = new CMHistoryProvider(this);
        }
        return historyProvider;
    }

    @Nullable
    public DiffProvider getDiffProvider() {
        return new CMDiffProvider(this);
    }

    public void sessionStarted() {
        myConfig.SESSION_STARTED = true;
        toolWindow.sessionStarted();
        //fileHandler = new CMLocalFileOperationsHandler(myProject);
        Tasks.getCurrentTask(myProject);
    }

    public void sessionStopped() {
        myConfig.SESSION_STARTED = false;
        myConfig.setCmEnvPath("");
        toolWindow.sessionStopped();
    }

    public void sendToConsole(String message) {
        toolWindow.appendConsole(message);   
    }

    private void showToolWindow() {

        if(toolWindow == null) {
            SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                toolWindow = new CMToolWindow(myProject);
            }
            });
        }
        else {
            toolWindow.setVisible(true);
        }
    }

    private void hideToolWindow() {
        if(toolWindow != null) {
            toolWindow.setVisible(false);
        }
    }

    private void hideStatus() {
        StatusBarHandler.hide();
    }

    private void showStatus() {
        StatusBarHandler.show(); 
    }

}
