// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cm;

import com.telelogic.integration.intellij.cli.CommandHelper;

import javax.swing.*;
import java.util.ArrayList;
import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * Thread that checks whether a session has been lost
 */
public class CMSessionMonitor extends Thread {

    private CMConfiguration config;
    private CMPlugin plugin;

    /**
     * Main constructor
     * @param c The plugin config
     * @param p The active IDEA project
     */
    public CMSessionMonitor(CMConfiguration c, CMPlugin p) {
        plugin = p;
        config = c;
        this.setPriority(Thread.MIN_PRIORITY);
        this.start();
    }

    /**
     * Handles the execution of the thread.
     * It first checks if the plugin has started a session.  If it has, it invokes getStatus() to verify the session
     * still exists.  If it does not, it notifies the plugin the session has been lost.  This thread loops once per
     * minute and has a low thread priority to keep it from interfering with the dispatch thread.
     */
    @SuppressWarnings({"InfiniteLoopStatement"})
    public void run() {
        // This is an infinite loop by design. Perhaps it can be started and stopped based on whether a project is open
        do {
            if(config.SESSION_STARTED) {
                if(!getStatus()) {
                    // Session has been lost
                    SwingUtilities.invokeLater(new Runnable() {

                        /**
                         * Call the plugin's method that handles lost sessions. As this involves modifying the UI, it
                         * must be run as a thread
                         */
                        public void run() {
                            plugin.sessionLost(true);
                        }

                    });
                }
            }
            
            try {
                sleep(60000); // Sleep for 1 minute
            } catch (InterruptedException e) {
                new CMExceptionHandler(e);
            }

        } while(true);
    }

    /**
     * Invokes "ccm status" and checks for a CCM_ADDR that matches the session environment variable stored in the
     * plugin config.  If it can't find a match, then the session has been lost.
     * @return Whether the session started by the plugin still exists
     */
    private boolean getStatus() {
        Process p;
        BufferedReader br;
        ArrayList<String> options = new ArrayList<String>();
        boolean sessionExists = false;
        String line;
        String ccmAddr = config.getCmEnvPath();

        // build checkin command
        // ccm co -c "comment"  <file> <file> <file>...
        options.add(config.getPath());
        options.add("status");

        String[] commands = options.toArray(new String[options.size()]);

        try {
            // Run command
            p = CommandHelper.runCommand(commands, ccmAddr);
            br = new BufferedReader(new InputStreamReader(p.getInputStream()));

            while((line = br.readLine()) != null) {
                if(line.contains(ccmAddr)) {
                    sessionExists = true;
                }
            }

        } catch (Exception e) { new CMExceptionHandler(e); }

        return sessionExists;
    }

}
