// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.cm;

import com.intellij.vcsUtil.VcsUtil;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.Computable;
import com.intellij.openapi.util.IconLoader;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.psi.search.PsiShortNamesCache;
import com.intellij.psi.PsiManager;
import com.intellij.psi.PsiFile;
import com.telelogic.integration.intellij.cli.PropertiesCommand;
import com.telelogic.integration.intellij.cli.AttributeCommand;
import com.telelogic.integration.intellij.ui.CMMessages;

import javax.swing.*;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * Static utility methods. Much of this code is derived from VssUtil.java in the JetBrains VSS Integration, which is
 * included in the IntelliJ IDEA Plugin Development Package. This package can be found at:
 * http://www.jetbrains.com/idea/plugins/plugin_developers.html
 */

public class CMUtil extends VcsUtil {
    private static final VirtualFile[] ourEmptyVirtualFileArray = new VirtualFile[]{};

    /**
     * Shows error message with specified message text and title.
     * The parent component is the root frame.
     * @param project The active IDEA project
     * @param message The message to be displayed
     * @param title The dialog window title to be used
     */
    public static void showErrorMessage( final Project project, final String message, final String title )
    {
        if( ApplicationManager.getApplication().isDispatchThread() )
            Messages.showMessageDialog( project, message, title, Messages.getErrorIcon());
        else
            ApplicationManager.getApplication().invokeLater( new Runnable()
            {
                /** @see Runnable#run() */
                public void run() { Messages.showMessageDialog( project, message, title, Messages.getErrorIcon()); }
            });
    }

    /**
     * General utility to strip quotes from a string
     * @param str The string that may contain quotes
     * @return The string with any quotes escaped
     */
    public static String escapeQuotes(String str) {
        if (str == null) return str;
        int idx = str.indexOf('"');
        if (idx < 0) return str;
        StringBuffer buf = new StringBuffer(str);
        while (idx < buf.length()) {
            if (buf.charAt(idx) == '"') {
                buf.replace(idx, idx + 1, "\\\"");
                idx += 2;
            }
            else {
                idx += 1;
            }
        }
        return buf.toString();
    }

    /**
     * IDEA 7.0 Compatibility: DataContext changed to AnActionEvent
     * @return <code>VirtualFile</code> available in the current context.
     *         Returns not <code>null</code> if and only if exectly one file is available.
     * @param event The object containing information about the Virtual File
     */
    public static VirtualFile getVirtualFile(AnActionEvent event) {
        VirtualFile[] files = getVirtualFiles(event);
        if (files.length != 1) {
            return null;
        }
        else {
            return files[0];
        }
    }

    /**
     * IDEA 7.0 Compatibility: DataContext changed to AnActionEvent. 
     * @return <code>VirtualFile</code>s available in the current context.
     *         Returns empty array if there are no available files.
     * @param event The object containing information about the Virtual File
     */
    public static VirtualFile[] getVirtualFiles(AnActionEvent event) {
        VirtualFile[] virtualFiles = (VirtualFile[])event.getData(DataKeys.VIRTUAL_FILE_ARRAY);
        if (virtualFiles == null) {
            virtualFiles = ourEmptyVirtualFileArray;
        }
        return virtualFiles;
    }

    /**
     * Shows message in the status bar.
     * @param project The active IDEA project
     * @param message The status bar message to be displayed
     */
    public static void showStatusMessage(final Project project, final String message) {
        SwingUtilities.invokeLater(new Runnable() {
            /** @see Runnable#run() */
            public void run() {
                if (project.isOpen()) {
                    WindowManager.getInstance().getStatusBar(project).setInfo(message);
                }
            }
        });
    }

    /**
     * Get the path to the Virtual File
     * @param file The Virtual File
     * @return The file's path
     */
    public static String getPath(final VirtualFile file) {
        return ApplicationManager.getApplication().runReadAction(new Computable<String>() {
            /** @see com.intellij.openapi.util.Computable#compute() */
            public String compute() {
                return file.getPath();
            }
        });
    }

    /**
     * Reads the CLI output line containing the CCM_ADDR environment variable and returns that variable's value.
     * @param output The CLI output
     * @return The CCM_ADDR value
     */
    public static String getCmEnvPath(String output)
    {
        String path = "";

        String words[] = output.split(" ");

        for (String word : words) {
            if(word.contains(":"))
            {
                path = word;
            }
        }

        //Messages.showMessageDialog(path, "CCM_ADDR", null);

        return path;
    }

    /**
     * Gets the Synergy icon for UI use
     * @param large Whether the icon requested should be returned in 'large' form
     * @return The requested icon
     */
    public static Icon getIcon(boolean large) {
        if(large){
            return IconLoader.getIcon("/com/telelogic/ui/icons/synergy.png");
        }
        else {
            return IconLoader.getIcon("/com/telelogic/ui/icons/synergysm.png");
        }
    }

    /**
     * This method looks for a VirtualFile associated with the given object name.  Will return multiple matches, though
     * usually the returned array will be of size = 1
     * @param project IDEA Project
     * @param objectName Synergy object name
     * @return ArrayList of matching VirtualFiles (usually just 1)
     */
    public static ArrayList<VirtualFile> getVirtualFilesByObjectName(Project project, String objectName) {
        CMConfiguration myConfig = CMConfiguration.getInstance(project);
        String delim = myConfig.getCmDelim();

        ArrayList<VirtualFile>  matches = new ArrayList<VirtualFile>();
        //AttributeCommand        attr    = new AttributeCommand(project, objectName);

        //String name = attr.get(AttributeCommand.NAME);
        // Splits the objectName using the CM delimiter (file.ext,1.3.1 where delim = ,)
        String[] tokens = objectName.split(delim);
        String name = tokens[0];

        FileEditorManager editor        = FileEditorManager.getInstance(project);
        PsiShortNamesCache find         = PsiManager.getInstance(project).getShortNamesCache();
        PsiFile[] results               = find.getFilesByName(name);

        // Iterate through results
        for(PsiFile result : results) {
            VirtualFile file = result.getVirtualFile();
            HashMap<String, String> fileProperties = new PropertiesCommand(file, project).get("name:%objectname");

            if(fileProperties.get("name").startsWith(objectName)) {
                matches.add(file);
            }
        }

        return matches;
    }
}
