// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.ui;

import com.intellij.openapi.options.BaseConfigurable;
import com.intellij.openapi.options.ConfigurationException;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.TextFieldWithBrowseButton;
import com.intellij.openapi.wm.WindowManager;
import com.telelogic.integration.intellij.cm.CMConfiguration;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.imageio.ImageIO;
import java.io.IOException;
import java.io.File;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Handles the display of plugin settings as seen in Settings->VCS.
 */
public class CMConfigurable extends BaseConfigurable {
    private static final Logger LOG = Logger.getInstance(CMConfigurable.class.getName());

    private JPanel myPanel;
    private JTextField usernameField;
    private JTextField databaseField;
    private JTextField engineField;
    private JLabel passwordLabel;
    private JPasswordField passwordField;
    private TextFieldWithBrowseButton pathField;
    private JPanel configPanel;
    private Icon icon;

    private Project myProject;
    private CMConfiguration myConfig;

    /**
     * Main constructor
     * @param project The active IDEA project
     */
    public CMConfigurable(Project project) {
        myProject = project;
        myConfig = CMConfiguration.getInstance(myProject);
    }

    /**
     * Getter for the plugin's VCS display name
     * @return The display name
     */
    public String getDisplayName() { return "Synergy"; }

    /**
     * Not currently used in the plugin, but originally allowed the synergy icon to be displayed in the IDEA
     * settings frame
     * @return The plugin's icon
     */
    public Icon getIcon() {
        if(icon == null) {
            try {
                icon = new ImageIcon(
                        ImageIO.read(getClass().getClassLoader().getResourceAsStream("icons/synergy.png"))
                );
            } catch (IOException e) { LOG.error("Error while reading icon for Synergy", e); }
        }
        return icon;
    }

    /**
     * Requirement for implementing BaseConfigurable. No use in this plugin for now
     * @return The help topic?
     */
    // TODO: Look for a use
    public String getHelpTopic(){
        return "project.propCM";
    }

    /**
     * Runs code to be executed when the settings dialog is displayed
     * @return The CMConfigurable JComponent
     */
    public JComponent createComponent() {

        // This opens a file chooser when the browse button is clicked next to the path field.
        // Only accepts files named "ccm.exe"
        pathField.addActionListener(
                new ActionListener() {
                    /**
                     * Open the chooser and verify the file chosen is ccm.exe
                     * @param e Event data (not used)
                     */
                    public void actionPerformed(ActionEvent e) {
                        JFileChooser fileChooser;

                        if(!myConfig.getPath().equals("")){ fileChooser = new JFileChooser(myConfig.getPath()); }
                        else { fileChooser = new JFileChooser(); }

                        FileFilter[] filters = fileChooser.getChoosableFileFilters();

                        for (FileFilter filter : filters) {
                            fileChooser.removeChoosableFileFilter(filter);
                        }

                        fileChooser.addChoosableFileFilter(
                                new FileFilter() {
                                    /**
                                     * Determines whether the file selected by the user is likely to be the ccm.exe
                                     * file required to start a synergy session
                                     * @param f The file to be evaluated (what's wanted is ccm.exe)
                                     * @return Whether the file matches "ccm.exe"
                                     */
                                    public boolean accept(File f) {
                                        return f.isDirectory() || "ccm.exe".equalsIgnoreCase(f.getName());
                                    }

                                    /**
                                     * The description to be displayed to the user explaining what file the user should
                                     * be pointing the chooser to
                                     * @return The description
                                     */
                                    public String getDescription() { return "Path to Synergy executable (ccm.exe)"; }
                                }
                        );

                        if ( JFileChooser.APPROVE_OPTION != fileChooser.showOpenDialog(
                                        WindowManager.getInstance().suggestParentWindow(myProject))) {
                            return;
                        }

                        File selection = fileChooser.getSelectedFile();
                        pathField.setText(selection.getAbsolutePath());
                    }
                }
        );

        return myPanel;
    }

    /**
     * Gets the values from the settings form's textfields and modifies the plugin config using those values
     * @throws ConfigurationException
     */
    public void apply() throws ConfigurationException {
        myConfig.setUser(usernameField.getText().trim());
        myConfig.setPassword(new String(passwordField.getPassword()));
        myConfig.setDB(databaseField.getText().trim());
        myConfig.setEngine(engineField.getText().trim());
        myConfig.setPath(pathField.getText().trim());
    }

    /**
     * Resets the UI fields to their original values (if the user clicks 'cancel' after modifying any fields)
     */
    public void reset(){
        usernameField.setText   (myConfig.getUser()     );
        passwordField.setText   (myConfig.getPassword()); //TODO: Verify removing .toString() didn't create a bug
        databaseField.setText   (myConfig.getDB()       );
        engineField.setText     (myConfig.getEngine()   );
        pathField.setText       (myConfig.getPath()     );
    }

    /**
     * Release the class's panel from memory
     */
    public void disposeUIResources() { /*myPanel = null;*/ }

    /**
     * Determines whether any of the text fields were modified
     * @return Whether the settings were modified
     */
    public boolean isModified() {
        return  !usernameField.getText().trim().equals(myConfig.getUser()) ||
                !passwordField.getPassword().toString().equals(myConfig.getPassword()) ||
                !databaseField.getText().trim().equals(myConfig.getDB()) ||
                !engineField.getText().trim().equals(myConfig.getEngine()) ||
                !pathField.getText().trim().equals(myConfig.getPath());

    }

    public JPanel getConfigPanel() {
        this.createComponent();
        return configPanel;
    }
}
