// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.ui;

import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowManager;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.util.IconLoader;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.peer.PeerFactory;
import com.intellij.ui.content.*;
import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cli.Tasks;
import com.telelogic.integration.intellij.ui.tabs.*;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;
import java.io.BufferedReader;
import java.io.IOException;

/**
 * Handles the plugin's UI, which is represented as an IDEA tool window
 */
public class CMToolWindow {

    public CMConfiguration      myConfig;
    public  Project             myProject;
    private ToolWindowManager   twm;
    public  JProgressBar        sessionProgress;

    private Session sessionTab;
    private Options optionsTab;
    private MyTasks tasksTab;
    private CurrentTask currentTaskTab;
    private CurrentFile currentFileTab;
    private static final String TOOL_WINDOW_ID = "Synergy";

    private ContentManager twContentManager;

    private ToolWindow myToolWindow;

    private ContentFactory twContentFactory;
    Content sessionContent;
    Content myTasksContent;
    Content currTaskContent;
    Content currFileContent;

    /**
     * Main declaration. Sets up the tool window, its tabs, and creates a listener for tab focus changes
     * @param project The active IDEA project
     */
    public CMToolWindow(Project project) {
        myProject   = project;
        twm         = ToolWindowManager.getInstance(project);
        myConfig    = CMConfiguration.  getInstance(project);

        // Initialize UI
        myToolWindow=twm.registerToolWindow(TOOL_WINDOW_ID, true, ToolWindowAnchor.BOTTOM); // IDEA 7.0 Compatibility
        myToolWindow.setIcon(IconLoader.getIcon("icons/synergysm.png"));

        PeerFactory peerFactory = PeerFactory.getInstance();
        twContentFactory = peerFactory.getContentFactory();
        twContentManager = myToolWindow.getContentManager();

        sessionTab      = new Session(project);
        optionsTab      = new Options(project);
        tasksTab        = new MyTasks(project);
        currentTaskTab  = new CurrentTask(project);
        currentFileTab  = new CurrentFile(project);

        sessionContent = twContentFactory.createContent(sessionTab.getPanel(), "Session", false);
        myTasksContent = twContentFactory.createContent(tasksTab.getPanel(), "My Tasks", false);
        currTaskContent = twContentFactory.createContent(currentTaskTab.getPanel(), "Current Task", false);
        currFileContent = twContentFactory.createContent(currentFileTab.getPanel(), "Current File", false);

        twContentManager.addContent(sessionContent);
        twContentManager.addContent(myTasksContent);
        twContentManager.addContent(currTaskContent);
        twContentManager.addContent(currFileContent);

        twContentManager.addContentManagerListener(new ContentManagerListener() {
            public void contentAdded(ContentManagerEvent event) { }

            public void contentRemoved(ContentManagerEvent event) { }

            public void contentRemoveQuery(ContentManagerEvent event) { }

            public void selectionChanged(ContentManagerEvent event) {
                Component component = event.getContent().getComponent();
                if(component.equals(tasksTab.getPanel())) {
                    tasksTab.updateTaskList();
                }
                else if(component.equals(currentTaskTab.getPanel())) {
                    currentTaskTab.update();
                }
                else if(component.equals(currentFileTab.getPanel())) {
                    updateCurrentFileTab(myConfig.getSelectedFile());
                }
            }
        });

        /*parentPanel.addComponentListener(new ComponentListener() {
            public void componentResized(ComponentEvent event) { }

            public void componentMoved(ComponentEvent event) { }

            public void componentShown(ComponentEvent event) {
                Component component = event.getComponent();
                if(component.equals(tasksTab.getPanel())) {
                    tasksTab.updateTaskList();
                }
                else if(component.equals(currentTaskTab.getPanel())) {
                    currentTaskTab.update();
                }
                else if(component.equals(currentFileTab.getPanel())) {
                    updateCurrentFileTab(myConfig.getSelectedFile());
                }
            }

            public void componentHidden(ComponentEvent event) { }
        });*/
/*
        tabPane.addChangeListener(new ChangeListener() {
            *//**
             * Detects when a new tab is in focus.  If the new tab in focus is Status or Tasks, it refreshes the
             * respective tab
             * @param changeEvent Event data (not used in this case)
             *//*
            public void stateChanged(ChangeEvent changeEvent) {

                if(tabPane.getSelectedComponent().equals(tasksTab.getPanel())) {
                    tasksTab.updateTaskList();
                }
                else if(tabPane.getSelectedComponent().equals(currentTaskTab.getPanel())) {
                    currentTaskTab.update();
                }
                //TODO: Update
                else if(tabPane.getSelectedComponent().equals(currentFileTab.getPanel())) {
                    updateCurrentFileTab(myConfig.getSelectedFile());    
                }   
            }

        });*/

        if(myConfig.SESSION_STARTED) {
            sessionStarted();
        }
        else {
            sessionStopped();
        }
    }

    /**
     * Removes the tool window from user view
     */
    public void unregisterToolWindow() {
        twm.unregisterToolWindow(TOOL_WINDOW_ID);
    }

    public void setVisible(boolean visible) {
        myToolWindow.setAvailable(visible, null);
    }

    /**
     * Sends buffered readers to the ReaderThread where the output is extracted and displayed in the plugin's CLI
     * console output
     * @param br The buffered reader to be processed
     */
    public void sendToConsole(BufferedReader br) {
        new ReaderThread(br).start();
    }

    /**
     * Readies the UI for user interaction when a session starts. This includes enabling session-dependent tool window
     * tabs.
     */
    public void sessionStarted() {
        StatusBarHandler.getInstance(myProject).setTask(Tasks.getCurrentTask(myProject));
        sessionTab.disableStart();
        sessionTab.enableStop();
        enableAllTabs();

        // If a GUI session is not running, disable task creation
        // TODO: allow task creation without a GUI session
        tasksTab.setCreateTaskButtonEnabled(myConfig.isGuiSession);

        //TODO: Is this necessary?
        VirtualFile[] selectedFiles = FileEditorManager.getInstance(myProject).getSelectedFiles();

        if(selectedFiles.length > 0) {
            myConfig.setSelectedFile(selectedFiles[0]);
            //tabsPanelController.updateCurrentFileTab(selectedFiles[0]);
        }
        else {
            updateCurrentFileTab(myConfig.getSelectedFile());
        }

        stopProgressBar();
    }

    /**
     * Provides graceful fallback to a 'session stopped' state when something goes wrong attempting to start a session.
     */
    public void sessionStartFailure() {
        sessionStopped();
        stopProgressBar();

        SwingUtilities.invokeLater(new Runnable() {
            /**
             * Displays a start failure message dialog to the user.
             */
            public void run() {
                Messages.showErrorDialog("Session failed to start. Please check your plugin configuration.",
                        "Synergy Session Error");
            }
        });
    }

    /**
     * Looks for all tabs in the tool window's tab pane and enables each of them
     */
    public void enableAllTabs() {

        Content[] contents = twContentManager.getContents();
        for(Content c : contents) {
            c.getComponent().setEnabled(true);
        }
        /*
        for(int i = 0; i < tabPane.getTabCount(); i++) {
            tabPane.setEnabledAt(i, true);
        }*/
        //tabPane.repaint();
    }

    /**
     * Looks for all session-dependent tabs in the tool window's tab pane and disables them. This is done by disabling
     * any tab not explicitly excluded in the method's logic.
     */
    public void disableSessionTabs() {

        Content[] contents = twContentManager.getContents();

        for(Content c : contents) {
            if(!c.getComponent().equals(sessionTab.getPanel())) {
                c.getComponent().setEnabled(false);
            }
        }
/*
        for(int i = 0; i < tabPane.getTabCount(); i++) {
           // Disable everything except the session and options tabs
           if(!tabPane.getComponentAt(i).equals(sessionTab.getPanel()) &&
                   !tabPane.getComponentAt(i).equals(optionsTab.getPanel())) {
               tabPane.setEnabledAt(i, false);
           }
       }*/
        //tabPane.repaint();
    }

    /**
     * Tells the tasks tab to update itself with the latest task info from Synergy
     */
   public void updateTasksTab() {
       tasksTab.updateTaskList();
   }

    /**
     * Adds a line of text to the plugin's CLI output console
     * @param line The text to be added
     */
   public void appendConsole(String line) {
       sessionTab.appendConsole(line);
   }

    /**
     * Returns the tool window and its tabs to their stopped states
     */
   public void sessionStopped() {
        StatusBarHandler.getInstance(myProject).setStatus(StatusBarHandler.NOSESSION);
        disableSessionTabs();
        sessionTab.enableStart();
        sessionTab.disableStop();
   }

    /**
     * Directs the session tab to start showing the progress bar associated with starting a session
     */
    public void startProgressBar() {
        sessionTab.startProgressBar();
    }

    /**
     * Directs the session tab to stop showing the progress bar associated with starting a session
     */
    public void stopProgressBar() {
        sessionTab.stopProgressBar();
    }

    /**
     * Directs the current file tab to refresh its file properties with Synergy
     * @param file The file whose properties are to be displayed/updated in the current file tab
     */
    public void updateCurrentFileTab(VirtualFile file) {
        currentFileTab.update(file);
    }

    /**
     * Directs the current file tab to refresh its file properties with Synergy
     */
    public void updateCurrentFileTab() {
        currentFileTab.update();
    }

    /**
     * Puts in focus the tab containing the CLI output console. The current location of this console is within the
     * session tab
     */
    public void focusOnConsole() {
           //TODO: Commented out for refactoring
           // tabPane.setSelectedComponent(start);
        //tabPane.setSelectedComponent(sessionTab.getPanel());
        twContentManager.setSelectedContent(sessionContent);
    }

    /**
     * Parses a buffered reader (generally CLI command output) and displays it in the tool window's CLI output console.
     * It also searches for information from Synergy regarding the CCM_ADDR environment variable, which is required
     * by the plugin. When this variable is outputted by Synergy, the plugin can change to a started state and allow
     * the user access to session-dependent functionality.
     */
    class ReaderThread extends Thread {
        BufferedReader br;
        boolean warning = false;

        /**
         * Default constructor
         * @param br The CLI output to be processed
         */
        ReaderThread(BufferedReader br) {
            this.br = br;

        }

        /**
         * Processes the buffered reader and looks for the CCM_ADDR environment variable. When the variable is found,
         * it executes the RunStartupCommands thread. If it finds an error in the buffered reader output, it
         * assumes the session did not start and invokes the tool window's sessionStartFailure() method.
         */
        public void run() {
            this.setPriority(Thread.NORM_PRIORITY);
            String l;
           
            try {
                while ((l = br.readLine()) != null) {

                    final String line = l;

                    // Send the line to the console window
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            appendConsole(line);
                        }
                    });

                    // Process the line and look for errors
                    if(line.contains("CCM_ADDR")) {
                        new RunStartupCommands(line).start();
                    }
                    else if(line.contains("Warning:") || line.contains("Fatal:") || line.contains("password:")) {
                        sessionStartFailure();
                        warning = true;
                        break;
                        /*
                        if(!warning) {
                            sessionStartFailure();
                            warning = true;
                        }   */
                    }
                }
            } catch (IOException e) { new CMExceptionHandler(e); }
            /*if(warning) {
                sessionStartFailure();
            } */

        }
    }

    /**
     * Handles session loading via a thread so the IDEA UI doesn't freeze while session data is gathered for display
     * in the tool window.
     */
    class RunStartupCommands extends Thread {
        String ccmAddress;

        /**
         * Default constructor
         * @param ccmAddr The CCM_ADDR environment variable to be used
         */
        RunStartupCommands(String ccmAddr) {
            ccmAddress = ccmAddr;
            this.setPriority(Thread.NORM_PRIORITY);
        }

        /**
         * Sets the current task if there is one, then invokes the tool window's sessionStarted() method.
         */
        public void run() {

            myConfig.setCmEnvPath(ccmAddress);
            myConfig.SESSION_STARTED=true;

            if(!myConfig.getDefaultTask().equals("")) {
                Tasks.setCurrentTask(myConfig.getDefaultTask(), myProject);
            }
            sessionStarted();
        }
    }
}
