// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.ui;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.IconLoader;
import com.intellij.openapi.wm.StatusBar;
import com.intellij.openapi.wm.WindowManager;

import javax.swing.*;

public class StatusBarHandler {
    private static StatusBarHandler ourInstance;

    private Project         project;
    private StatusBar       statusBar;
    private JLabel          statusLabel;
    private Icon            synergyIcon = IconLoader.getIcon("icons/synergysm.png");
    public static String    NOSESSION = "No active session";
    public static String    NOTASK = "No current task";
    private static int      MAXCHARS = 100;

    private boolean         visible = false;

    synchronized public static StatusBarHandler getInstance(Project p) {
        if(ourInstance == null) {
            ourInstance = new StatusBarHandler(p);
        }
        return ourInstance;
    }

    synchronized public static void remove() {
        // Don't bother destroying anything that's nonexistant

        if(ourInstance != null) {
            // Two things are required: remove this component from the IDEA status bar, then nullify the instance
            ourInstance.removeFromStatusBar();
            ourInstance = null;
        }
    }

    synchronized public static void hide() {
        if(ourInstance != null) {
            ourInstance.removeFromStatusBar();
        }
    }

    synchronized public static void show() {
        if(ourInstance != null) {
            ourInstance.addToStatusBar();
        }
    }

    private StatusBarHandler(Project p) {
        project = p;
        statusLabel = new JLabel("", IconLoader.getIcon("icons/synergysm.png"), SwingConstants.LEFT);

        new StatusBarGetter().start();
    }

    /**
     * Tries to get the window manager's status bar. If it can't be found, return false
     * @return Whether the status bar was assigned
     */
    synchronized private boolean getStatusBar() {
        // Stop here if the status bar already exists
        if(statusBar != null) {
            return true;
        }
        else {
            statusBar = WindowManager.getInstance().getStatusBar(project);

            // It's very likely that when a project is opening, the status bar isn't returned by the WindowManager
            // We must still catch whether a null value is returned
            if(statusBar != null) {
                // We want to add the statusLabel only once
                // If addToStatusBar was already called (thus visible is true), it should be called again
                // now that the status bar is available
                if(visible) {
                    addToStatusBar();
                }
                return true;
            }
            // Tried but failed to get the status bar
            return false;
        }
    }

    private void removeFromStatusBar() {
        visible = false;
        // Sanity check.  No removing things from the status bar if we never got a status bar
        if(statusBar != null) {
            statusBar.removeCustomIndicationComponent(statusLabel);
        }
    }

    private void addToStatusBar() {

        visible = true;

        if(statusBar != null) {
            statusBar.addCustomIndicationComponent(statusLabel);
        }
    }

    public void setTask(String task) {
        getStatusBar();
        statusLabel.setText(textFormatter("Current Task: " + task));
        statusLabel.setToolTipText(task);
    }

    public void setStatus(String msg) {
        getStatusBar();
        statusLabel.setText(textFormatter(msg));
        statusLabel.setToolTipText(msg);
    }

    private String textFormatter(String text) {
        if(text.length() > MAXCHARS) {
            return text.substring(0, MAXCHARS-4) + "...";
        }
        return text;
    }

    private class StatusBarGetter extends Thread {
        public void run() {
            if(!getStatusBar()) {
                try {
                    sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
