// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.ui.dialogs;

import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.application.ApplicationManager;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/**
 * Displays a dialog used to gather comments from the user to be inputted into Synergy (e.g. when a task is completed).
 * @author Luke Stahlman
 */
public class CommentDialog extends JDialog {
    private JPanel contentPane;
    private JButton buttonOK;
    private JButton buttonCancel;
    private JTextArea comment;
    private String commentText = null;
    private boolean required = false;
    private boolean cancelled = true; // Changed to false in onOK()

    /**
     * Main constructor. Sets up the UI and button listeners
     */
    public CommentDialog() {
        setContentPane(contentPane);
        setModal(true);
        getRootPane().setDefaultButton(buttonOK);

        buttonOK.addActionListener(new ActionListener() {
            /** @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent) */
            public void actionPerformed(ActionEvent e) {
                onOK();
            }
        });

        buttonCancel.addActionListener(new ActionListener() {
            /** @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent) */
            public void actionPerformed(ActionEvent e) {
                onCancel();
            }
        });

// call onCancel() when cross is clicked
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            /** @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent) */
            public void windowClosing(WindowEvent e) {
                onCancel();
            }
        });

// call onCancel() on ESCAPE
        contentPane.registerKeyboardAction(new ActionListener() {
            /** @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent) */
            public void actionPerformed(ActionEvent e) {
                onCancel();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    }

    /**
     * Evaluates the comment entered by the user. If the comment field is blank, it checks whether a comment was
     * required. If so, it keeps the dialog open and tells the user to enter a comment. If no comment is required,
     * the user is asked whether a blank comment was intentional.
     */
    private void onOK() {

        commentText = comment.getText();

        // If the comment field is left blank, confirm that the user doesn't want to add any comments
        // The showYesNoDialog method returns 0 if the answer is yes
        if(commentText.equals("")) {
            if(required) {
                Messages.showWarningDialog("Comments are required to complete this operation", "Comment Warning");
            }
            else
            {
                if(Messages.showYesNoDialog("Are you sure you want to leave the comments blank?",
                            "No Comments Warning",null) == 0) {
                    cancelled = false;
                    dispose();
                }
            }
        }
        else {
            cancelled = false;
            dispose();
        }
    }

    /**
     * Disposes the dialog if the user clicks the cancel button or closes the window
     */
    private void onCancel() {
        dispose();
    }

    /**
     * Displays the dialog
     */
    public void showDialog() {
        pack();
        setLocationRelativeTo(null);
        setVisible(true);
    }

    /**
     * Getter
     * @return The comment entered by the user
     */
    public String getComment() {
        return commentText;
    }

    // Sets the current comment (Task description)
    public void setComment(String text) {
       comment.setText(text);
    }

    /**
     * Setter
     * @param r Whether a comment is required from the user
     */
    public void setRequired(boolean r) {
        required = r;
    }

    public boolean wasCancelled() {
        return cancelled;
    }

}
