// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.ui.dialogs;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vcs.VcsException;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.progress.Task;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.vcsUtil.VcsUtil;
import com.telelogic.integration.intellij.cli.*;
import com.telelogic.integration.intellij.cli.helpers.ParallelHelper;
import com.telelogic.integration.intellij.cm.CMMergeProvider;
import com.telelogic.integration.intellij.cm.CMDiffProvider;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.event.*;
import java.awt.*;
import java.util.*;

public class ParallelsDialog extends JDialog {
    private JPanel  contentPane;
    private JButton buttonOK;
    private JButton buttonCancel;
    private JList   objectList;
    private JTable  parallelTable;
    private JLabel  mergeTableTitle;
    private JPanel  objectPanel;
    private JPanel  mergeCandidatePanel;
    private JButton buttonCompare;
    private JButton buttonMerge;

    private Project             project;
    private static String       BASE_TABLE_TITLE = "Merge candidates for ";
    private static final int    ON_CLOSE_NOTHING = 0;
    private static final int    ON_CLOSE_RETRY = 1;
    private DefaultTableModel   tableModel;
    private Vector<VirtualFile> objectListData;
    private ParallelHelper      parallelHelper;

    // VirtualFile -> Merge Candidates
    final HashMap<VirtualFile, LinkedList<History.HistoryObject>> mergeCandidateData
            = new HashMap<VirtualFile, LinkedList<History.HistoryObject>>();

    // VirtualFile -> Object History
    final HashMap<VirtualFile, LinkedHashMap<History.HistoryObject.Key, History.HistoryObject>> fileHistoryData
            = new HashMap<VirtualFile, LinkedHashMap<History.HistoryObject.Key, History.HistoryObject>>();

    // ObjectName -> VirtualFile
    final HashMap<String, VirtualFile> objectNameToVirtualFile = new HashMap<String, VirtualFile>();

    // Keeps track of the files that have been merged in the dialog
    // This is for user with the ParallelHelper instance, if it exists
    final ArrayList<VirtualFile> filesMerged            = new ArrayList<VirtualFile>();
    final ArrayList<VirtualFile> filesPartiallyMerged   = new ArrayList<VirtualFile>();
    final ArrayList<VirtualFile> filesNotMerged         = new ArrayList<VirtualFile>();

    // Keep track of directories.  Currently they cannot be merged through the integration
    final ArrayList<VirtualFile> directories            = new ArrayList<VirtualFile>();

    public ParallelsDialog(final Project p, final ArrayList<VirtualFile> files) throws VcsException {
        new ParallelsDialog(p, files, null);
    }

    public ParallelsDialog(final Project p, final ArrayList<VirtualFile> files, ParallelHelper helper) throws VcsException {
        project = p;
        parallelHelper = helper;

        // Find directories (directory merge not yet supported)
        for(VirtualFile file : files) {
            if(file.isDirectory()) {
                directories.add(file);
            }
        }

        // If any directories were found, remove them from the list of files to be used in the show parallels operation
        // Display the directories that cannot be merged in a message dialog
        // If other non-dir objects exist, ask user if Show Parallels should continue. If not, show an info message with
        if(directories.size() > 0) {
            String title = "Show Parallels Warning";
            String message = "The following objects are directories and cannot be merged through this tool:\n";
            for(VirtualFile dir : directories) {
                message += "\n" + dir.getName();
                files.remove(dir);
            }
            if(files.size() > 0) {
                message += "\n\nWould you like to continue this operation for non-directory objects?";
                if(Messages.showYesNoDialog(project, message, title, null) != 0) {
                    return;
                }
            }
            else {
                Messages.showInfoMessage(project, message, title);
                return;
            }
        }

        // Gather all the history data for each VirtualFile, get their merge candidates, then show the parallels dialog
        final Task showParallels = new Task.Backgroundable(p, "Loading parallel information", false) {
            public void run(ProgressIndicator progressIndicator) {

                try {
                    Runnable runnable; // What will be executed once all the data is processed

                    // Iterate through the virtual file array and get any parallels (considered merge candidates)
                    for(VirtualFile file : files) {
                        LinkedHashMap<History.HistoryObject.Key, History.HistoryObject> history =
                                History.getRevisionData(p, VcsUtil.getFilePath(file.getPath()));

                        LinkedList<History.HistoryObject> candidates = History.getMergeCandidates(p, file, history);

                        if(candidates.size() > 0) {
                            filesNotMerged.add(file);
                            mergeCandidateData.put(file, candidates);
                            fileHistoryData.put(file, history);
                        }
                    }

                    // Show the dialog if there are merge candidates, otherwise display a message stating no candidates
                    // found.  Both actions are enclosed in a runnable
                    if(mergeCandidateData.size() > 0) {
                        runnable = new Runnable() {
                            public void run() {
                                showDialog();
                            }
                        };
                    }
                    else {
                        runnable = new Runnable() {
                            public void run() {
                                Messages.showMessageDialog("No parallels were found for the selected object(s)", "Show Parallels Result", null);
                            }
                        };

                    }

                    // Run the 'runnable' from above
                    ApplicationManager.getApplication().invokeLater(runnable);
                    
                } catch (VcsException exception) {
                    VcsUtil.showErrorMessage(project,
                            "An error was encountered attemping to show parallels: \n" + exception.getMessage(),
                            "Warning");
                }

            }
        };

        // Run the above task.  This was a little tricky to implement, as Java is very specific about this being on
        // the event dispatch thread. First try to run this normally through the ApplicationManager.  If that can't be
        // done (we're not in the dispatch thread), then create a new runnable and place it in the event queue
        if(ApplicationManager.getApplication().isDispatchThread()) {
            showParallels.queue();
        }
        else {
            EventQueue.invokeLater(new Runnable() {
                public void run() {
                    showParallels.queue();
                }
            });
        }

    }

    /*public ParallelsDialog(Project p, HashMap<VirtualFile, LinkedList<History.HistoryObject>> data, HashMap<VirtualFile, LinkedHashMap<History.HistoryObject.Key, History.HistoryObject>> history) throws VcsException {
        this.mergeCandidateData = data;
        this.fileHistoryData    = history;
        this.project            = p;

        showDialog();
    }*/

    /**
     * Sets up and displays the dialog UI
     */
    synchronized private void showDialog() {
        setUpListeners();
        setContentPane(contentPane);
        setModal(true);
        getRootPane().setDefaultButton(buttonOK);

        // Set up objectList view
        objectList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        // Get empty table data
        tableModel = ObjectHistoryTableModel.getBlankHistoryTableModel();

        // Set up table view
        parallelTable.setShowVerticalLines(true);
        parallelTable.setModel(tableModel);
        parallelTable.setShowVerticalLines(false);
        parallelTable.setColumnSelectionAllowed(false);
        parallelTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        parallelTable.setIntercellSpacing(new Dimension(2,0));

        // Set up list of objects with parallels (objectPanel)
        objectListData = new Vector<VirtualFile>(){};

        for(VirtualFile file : mergeCandidateData.keySet()) {
            String objectName = new PropertiesCommand(file, project).get("%objectname", file.getPath());
            if(objectName != null) {
                objectNameToVirtualFile.put(objectName, file);
            }
        }

        updateObjectList();
        enableButtons((parallelTable.getSelectedRow() != -1));

        // Set up and display UI
        setTitle("Showing Parallels");
        pack();
        setVisible(true);
    }

    /**
     * Handles the close action.  If there is an action to be performed on close, it calls that action
     */
    private void onClose() {
        setVisible(false);
        // Call retry() if necessary
        if(parallelHelper != null) {
            Runnable runnable = new Runnable() {
                public void run() {
                    parallelHelper.reportResults(filesMerged, filesPartiallyMerged, filesNotMerged);
                }
            };
            ApplicationManager.getApplication().invokeLater(runnable);
        }
        dispose();
    }

    /**
     * Display the dialog
     */
    public void display() {
        pack();
        setVisible(true);
    }

    /**
     * Add listeners for all relevant components in the dialog
     */
    private void setUpListeners() {

        // Call a merge dialog
        buttonMerge.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent event) {
                doMerge();
            }
        });

        // Call a compare dialog
        buttonCompare.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent event) {
                doCompare();
            }
        });

        parallelTable.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent event) {
                enableButtons((parallelTable.getSelectedRow() != -1));
            }
        });

        parallelTable.addKeyListener(new KeyAdapter() {
            public void keyTyped(KeyEvent event) {
                enableButtons((parallelTable.getSelectedRow() != -1));
            }
        });

        // Display the parallel candidates for the selected object
        objectList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent event) {
                updateTableModel();
            }
        });

        // Close
        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onClose();
            }
        });

// call onClose() when cross is clicked
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                onClose();
            }
        });

// call onClose() on ESCAPE
        contentPane.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onClose();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    }

    /**
     * Starts a diff request between the object selected in the list with the selected parallel
     */
    private void doCompare() {
        String selection = (String)objectList.getSelectedValue();
        VirtualFile file = objectNameToVirtualFile.get(selection);

        int row = parallelTable.getSelectedRow();
        if(row != -1) {
            // Get the object name, then its history object
            History.HistoryObject candidate = (History.HistoryObject)parallelTable.getValueAt(row, 0);
            // Messy... this should be a predecessor
            History.HistoryObject current = fileHistoryData.get(file).get(new History.HistoryObject.Key(selection));
            // Call a diff dialog from here
            CMDiffProvider.doSimpleDiffRequest(project, current, candidate);
        }
    }

    /**
     * Gets merge data and calls the merge provider.  Once merge is completed, the merged object is refreshed to update
     * its list of parallel objects (if any)
     */
    private void doMerge() {
        // Get the virtual file in the object list, then the selected history object
        String selection = (String)objectList.getSelectedValue();
        VirtualFile file = objectNameToVirtualFile.get(selection);

        int row = parallelTable.getSelectedRow();

        // Ensure a row has been selected, then run merge operations
        if(row != -1) {
            // Get the current object's history object
            History.HistoryObject currentObj = fileHistoryData.get(file).get(new History.HistoryObject.Key(selection));

            // Get the selected parallel candidate's object name, then its history object
            History.HistoryObject lastObj = (History.HistoryObject)parallelTable.getValueAt(row, 0);

            // Messy... this should be a predecessor
            History.HistoryObject originalObj = fileHistoryData.get(file).get(new History.HistoryObject.Key(selection));

            // Call a merge dialog from here
            // If it returns true, then the file was merged
            if(CMMergeProvider.doSingleMerge(project, file, currentObj, lastObj, originalObj)) {
                // Refresh the candidate file that was merged
                refreshMergedObject(selection);
            }
        }
    }

    /**
     * This is called when a file has been merged with a merge candidate.  Two things could be the case now: all
     * parallel conflicts are resolved (so the file should no longer be listed in the dialog), or there are still merge
     * candidates to be dealt with. Candidate data for the file is gathered by this method.  If there are no more
     * candidates, take the file out of the list to display and place it in the mergedFiles array.
     * @param objectName The object to be refreshed
     */
    private void refreshMergedObject(String objectName) {
        VirtualFile file = objectNameToVirtualFile.get(objectName);

        LinkedHashMap<History.HistoryObject.Key, History.HistoryObject> history =
                History.getRevisionData(project, VcsUtil.getFilePath(file.getPath()));

        try {
            LinkedList<History.HistoryObject> candidates = History.getMergeCandidates(project, file, history);

            if(candidates.size() > 0) {
                // There are still conflicts.  Keep the file in the parallel list, but refresh the candidates
                mergeCandidateData.put(file, candidates);
                filesPartiallyMerged.add(file);
            }
            else {
                // Add the file to mergedFiles and remove it from the list of parallels
                filesMerged.add(file);
                filesNotMerged.remove(file);
                filesPartiallyMerged.remove(file);

                mergeCandidateData.remove(file);
                objectNameToVirtualFile.remove(objectName);
            }

            updateObjectList();

            this.repaint();

        } catch (VcsException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    /**
     * This refreshes the object list data model to reflect any changes (e.g., All parallel conflicts have been resolved
     * for an object) and resets the list selecton
     */
    private void updateObjectList() {
        objectList.setListData(objectNameToVirtualFile.keySet().toArray());
        if(objectNameToVirtualFile.size() > 0) {
            objectList.setSelectedIndex(0);
        }
        else {
            objectList.clearSelection();
            clearCandidateTable();
        }

    }

    /**
     * Finds the selected value from the object list and populates the parallel candidates table with the respective
     * candidate data 
     */
    private void updateTableModel() {
        if(objectList.getSelectedValue() instanceof String) {
            // Get the current list selection
            //VirtualFile selection = (VirtualFile)objectList.getSelectedValue();

            VirtualFile selection = objectNameToVirtualFile.get(objectList.getSelectedValue());

            LinkedList<History.HistoryObject> linkedData = mergeCandidateData.get(selection);

            // Change the table to match the selection
            DefaultTableModel newModel = ObjectHistoryTableModel.getHistoryTableModel(linkedData);
            parallelTable.setModel(newModel);
            mergeTableTitle.setText(BASE_TABLE_TITLE + objectList.getSelectedValue());
            enableButtons(false);
        }
    }

    /**
     *  Clears the parallel candidates table
     */
    private void clearCandidateTable() {
        parallelTable.setModel(ObjectHistoryTableModel.getBlankHistoryTableModel());
    }

    public static void show(Project project, ArrayList<VirtualFile> files) {
        try {
            new ParallelsDialog(project, files);
        } catch (VcsException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    /**
     * Enable or disable the compare and merge buttons
     * @param enabled Whether the buttons should be enabled
     */
    private void enableButtons(boolean enabled) {
        buttonCompare.setEnabled(enabled);
        buttonMerge.setEnabled(enabled);
    }

    /**
     * Special renderer for the object list
     */
    private class ObjectListCellRenderer implements ListCellRenderer {
        public Component getListCellRendererComponent(JList jList, Object o, int i, boolean b, boolean b1) {
            return null;
        }
    }
}
