// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.ui.dialogs;

import com.telelogic.integration.intellij.cli.UseVersionCommand;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import java.awt.event.*;
import java.awt.*;
import java.util.Vector;

/**
 * Displays a dialog that allows a user to choose which version of a specific object should be placed in the work area
 * (IDEA project).
 */
public class UseVersion extends JDialog {
    private JPanel contentPane;
    private JButton buttonOK;
    private JButton buttonCancel;
    private JTable versionTable;
    private JScrollPane tableScrollPane;
    private UseVersionCommand command;
    private VersionTableModel tableModel;

    private Vector<Vector<String>>  data = new Vector<Vector<String>>();
    private Vector<String>          columns = new Vector<String>();

    /**
     * Main constructor
     * @param values Object versions the user can choose from
     * @param c The UseVersionCommand instance calling this class
     */
    public UseVersion(Vector<Vector<String>> values, UseVersionCommand c) {
        data = values;
        command = c;
        setContentPane(contentPane);
        setModal(true);
        getRootPane().setDefaultButton(buttonOK);
        setTitle("Use Version");

        buttonOK.addActionListener(new ActionListener() {
            /** @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent) */
            public void actionPerformed(ActionEvent e) {
                onOK();
            }
        });

        buttonCancel.addActionListener(new ActionListener() {
            /** @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent) */
            public void actionPerformed(ActionEvent e) {
                onCancel();
            }
        });

// call onCancel() when cross is clicked
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            /** @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent) */
            public void windowClosing(WindowEvent e) {
                onCancel();
            }
        });

// call onCancel() on ESCAPE
        contentPane.registerKeyboardAction(new ActionListener() {
            /** @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent) */
            public void actionPerformed(ActionEvent e) {
                onCancel();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

        versionTable.addMouseListener(new MouseAdapter() {
            /** @see java.awt.event.MouseAdapter#mouseClicked(java.awt.event.MouseEvent) */
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    onOK();
                }
            }
        });


        columns=new Vector<String>();
        columns.addElement("Name");
        columns.addElement("Version");
        columns.addElement("State");
        columns.addElement("Owner");

        tableModel = new VersionTableModel(values, columns);
        //tableModel.setDataVector(values, columnNames);

        versionTable.setModel(tableModel);
        versionTable.setShowVerticalLines(false);
        versionTable.setColumnSelectionAllowed(false);
        versionTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        versionTable.setIntercellSpacing(new Dimension(2,0));
    }

    /**
     * Disposes of the dialog and starts the execution of the UseVersion command
     */
    private void onOK() {
        setVisible(false);
        executeUse();
        dispose();
    }

    /**
     * Handle a cancel/window closing event (dispose of the dialog)
     */
    private void onCancel() {
        dispose();
    }

    /**
     * Display the dialog
     */
    public void display() {
        pack();
        setVisible(true);
    }

    /**
     * Execute the UseVersion command for the object version desired
     */
    private void executeUse() {
        int row = versionTable.getSelectedRow();
        String version = tableModel.getValueAt(row,4).toString();
        //String version = versionTable.getValueAt(row, 4).toString();
        command.use(version);
    }
 /*
    public static void main(String[] args) {
        UseVersion dialog = new UseVersion();
        dialog.pack();
        dialog.setVisible(true);
        System.exit(0);
    }*/

    /**
     * Provides a means to disable the user's ability to edit table cells in the version display
     */
    class VersionTableModel extends DefaultTableModel {
        public VersionTableModel(Vector<Vector<String>> values, Vector<String> columnNames) {
            data = values;
            columns = columnNames;
        }

        public String getColumnName(int i) {
            return columns.get(i);
        }

        public void setValueAt(Object o, int row, int column) {
            Vector<String> rowData = data.get(row);
            rowData.set(column, o.toString());
        }

        public Object getValueAt(int row, int column) {
            Vector<String> rowData = data.get(row);
            return rowData.get(column);
        }

        public int getColumnCount() {
            return columns.size();
        }

        public int getRowCount() {
            return data.size();
        }

        /**
         * Returns whether a cell is edited (always false in this implementation)
         * @param row Cell row
         * @param col Cell column
         * @return Whether the cell can be edited
         */
        public boolean isCellEditable(int row, int col) {
            return false;
        }

    }
}
