// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.ui.tabs;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import com.telelogic.integration.intellij.actions.AssociateWithTask;
import com.telelogic.integration.intellij.actions.AddAction;
import com.telelogic.integration.intellij.actions.CheckOutAction;
import com.telelogic.integration.intellij.actions.CheckInAction;
import com.telelogic.integration.intellij.cli.PropertiesCommand;
import com.telelogic.integration.intellij.cm.CMConfiguration;

import javax.swing.*;
import java.util.HashMap;

/**
 * Provides the Current File tab and its related functions.
 */
public class CurrentFile {
    private JPanel currentFile;
    private JTextField nameField;
    private JTextField versionField;
    private JTextField ownerField;
    private JTextField statusField;
    private JTextField createTimeField;
    private JTextField modifyTimeField;
    private JTextField platformField;
    private JTextField releaseField;
    private JTextField taskField;
    private JButton assocWithTaskButton;
    private JButton addButton;
    private JButton checkOutButton;
    private JButton checkInButton;

    private Project project;
    private CMConfiguration config;

    /**
     * Main constructor
     * @param project The active IDEA project
     */
    public CurrentFile(Project project) {
        this.project = project;
        config = CMConfiguration.getInstance(project);

        // Add listeners for the current file tab panel
        assocWithTaskButton.addActionListener(new AssociateWithTask(project));
        addButton.addActionListener(new AddAction(project));
        checkOutButton.addActionListener(new CheckOutAction(project));
        checkInButton.addActionListener(new CheckInAction(project));
    }

    /**
     * Gets the JPanel used for this tab
     * @return The CurrentFile tab
     */
    public JPanel getPanel() {
        return currentFile;
    }

    /**
     * Refreshes the information displayed in the panel to represent the current properties of the file passed to the
     * method
     * @param f The file to use in updating the panel
     */
    public void update(VirtualFile f) {
        new Populate(f, config.selectedFileProperties).start();
    }

    public void update() {
        // Get selected file
        VirtualFile selectedFile = config.getSelectedFile();
        // Update selectedFileProperties
        config.updateSelectedFileProperties();

        new Populate(selectedFile, config.selectedFileProperties).start();
    }

    private void createUIComponents() {
        // TODO: place custom component creation code here 
    }

    /**
     * Thread which gathers data for the file to be displayed in the panel, then inserts that data into the panel's UI
     */
    private class Populate extends Thread {
        VirtualFile file;
        HashMap<String,String> properties = null;

        /**
         * Main constructor.
         * @param f The file to use in updating the panel
         */
        public Populate(VirtualFile f) {
            file = f;
        }

        /**
         * Alternative constructor. Used when the properties of the current file are already known to the plugin
         * @param f The file referenced by the panel
         * @param p The properties corresponding to the file
         */
        public Populate(VirtualFile f, HashMap<String,String> p) {
            file = f;
            properties = p;
        }

        /**
         * Get file property data from synergy (if no property data exists), then insert the property values into the
         * panel's text fields
         */
        public void run() {
            this.setPriority(Thread.NORM_PRIORITY);

            // Only run the properties command in Synergy if no properties exist
            if(properties == null) {
                if(file != null) {
                    properties = new PropertiesCommand(file, project).get();
                    if(properties == null) {
                        properties = new HashMap<String,String>();
                    }
                }
                else {
                    properties = new HashMap<String,String>();
                }
            }

            /* Checks to see whether file properties were returned. If they weren't, that indicates the file is not
               under Synergy control. If under Synergy control, disable the 'add' button and display the file
               properties. If not under Synergy control, disable all buttons except the 'add' button. The idea here
               is to allow the user to run synergy operations relevant to the file being referenced by this panel */
            if(properties.containsKey("name")) {
                addButton.setEnabled(false);

                if(config.getDefaultTask().equals(properties.get("task"))) {
                    assocWithTaskButton.setEnabled(false);
                }
                else {
                    assocWithTaskButton.setEnabled(true);
                }

                if(PropertiesCommand.isCheckedIn(properties)) {
                    assocWithTaskButton.setEnabled(false);
                    checkOutButton.setEnabled(true);
                    checkInButton.setEnabled(false);
                }
                else {
                    checkOutButton.setEnabled(false);
                    checkInButton.setEnabled(true);
                }
            }
            else {
                addButton.setEnabled(true);
                checkOutButton.setEnabled(false);
                checkInButton.setEnabled(false);
                assocWithTaskButton.setEnabled(false);
            }

            nameField.setText(       properties.containsKey("name")          ? properties.get("name")        : "");
            statusField.setText(     properties.containsKey("status")        ? properties.get("status")      : "");
            ownerField.setText(      properties.containsKey("owner")         ? properties.get("owner")       : "");
            platformField.setText(   properties.containsKey("platform")      ? properties.get("platform")    : "");
            releaseField.setText(    properties.containsKey("release")       ? properties.get("release")     : "");
            createTimeField.setText( properties.containsKey("create_time")   ? properties.get("create_time") : "");
            modifyTimeField.setText( properties.containsKey("modify_time")   ? properties.get("modify_time") : "");
            taskField.setText(       properties.containsKey("task")          ? properties.get("task")        : "");
            versionField.setText(    properties.containsKey("version")       ? properties.get("version")     : "");

        }
    }
}
