// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.ui.tabs;

import com.telelogic.integration.intellij.cli.Tasks;
import com.telelogic.integration.intellij.cli.CompleteTaskCommand;
import com.telelogic.integration.intellij.cm.CMExceptionHandler;
import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.ui.dialogs.CommentDialog;
import com.telelogic.integration.intellij.ui.dialogs.TaskPropertiesDialog;
import com.telelogic.integration.intellij.ui.CMMessages;
import com.telelogic.integration.intellij.actions.ViewAssociatedObjectAction;
import com.telelogic.integration.intellij.actions.CreateTaskAction;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.ui.Splitter;
import com.intellij.openapi.ui.LabeledComponent;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.openapi.wm.StatusBar;

import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.border.TitledBorder;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.*;
import java.util.Vector;
import java.util.LinkedHashMap;
import java.util.HashMap;

/**
 * Provides the My Tasks tab and its related functions. These functions include listing all tasks in a JList,
 * displaying objects related to a selected task in a JTable, and providing the ability to view task properties and set
 * a selected task as the default for the active IDEA project.
 */
public class MyTasks {
    private JButton             refreshButton;
    private JLabel              currentTaskValue;
    private JButton             createTaskButton;
    private PopulateTaskList    taskUpdateThread;
    private JPopupMenu          taskPopupMenu;

    private JPanel              myTasks;
    private JPanel              buttons;
    private Splitter            splitter;
    private JScrollPane         tasksListScrollPane;
    private JScrollPane         assocObjectsScrollPane;
    private JList               taskList;
    private TaskListModel       taskListModel;
    private JTable              associatedObjectTable;
    private JPanel buttonPanel;
    private LabeledComponent    tasks;
    private LabeledComponent    objects;

    private Project             project;
    private CMConfiguration     config;
    private int                 totalTasks = 0;
    private String              previouslySelectedTask = null;
    private String              currentlySelectedTask = null;

    /**
     * Main constructor
     * @param project The active IDEA project
     */
    public MyTasks(Project project) {
        this.project = project;
        this.config = CMConfiguration.getInstance(project);

        myTasks = new JPanel(new BorderLayout());
        buttons = new JPanel(new FlowLayout());
        splitter = new Splitter();
        taskList = new JList();
        associatedObjectTable = new JTable();
        tasksListScrollPane = new JScrollPane(taskList);
        assocObjectsScrollPane = new JScrollPane(associatedObjectTable);

        buttons.add(createTaskButton);
        buttons.add(refreshButton);

        JPanel buttonsAndTasks = new JPanel(new BorderLayout());
        buttonsAndTasks.add(tasksListScrollPane, BorderLayout.CENTER);
        buttonsAndTasks.add(buttons, BorderLayout.SOUTH);

        tasks = new LabeledComponent();
        tasks.setText("Assigned Tasks");
        //tasks.add(tasksListScrollPane);
        tasks.add(buttonsAndTasks);

        objects = new LabeledComponent();
        objects.setText("Associated Objects");
        objects.add(assocObjectsScrollPane);


        myTasks.add(splitter);
        //myTasks.add(buttonPanel, BorderLayout.SOUTH);
        /*splitter.setFirstComponent(tasksListScrollPane);
        splitter.setSecondComponent(assocObjectsScrollPane);*/
        splitter.setFirstComponent(tasks);
        splitter.setSecondComponent(objects);
        splitter.setHonorComponentsMinimumSize(true);
        splitter.setProportion((float)0.25);

        //taskModel = new DefaultListModel();

        tasksListScrollPane.setMinimumSize(new Dimension(200,0));
        assocObjectsScrollPane.setMinimumSize(new Dimension(200,0));

        popupSetup();
        tableListenerSetup();

        refreshButton.addActionListener(new ActionListener() {
            /**
             * Updates the tasks tab when the "Refresh" button is clicked
             */
            public void actionPerformed(ActionEvent actionEvent) {
                updateTaskList();
            }
        });

        createTaskButton.addMouseListener(new CreateTaskAction(project));
    }

    /**
     * Returns this class's JPanel
     * @return The MyTasks JPanel
     */
    public JPanel getPanel() {
        return myTasks;
    }

    /**
     * Creates the task list popup menu and add listeners
     */
    private void popupSetup() {
        // PopulateTaskList the task popup menu
        taskPopupMenu = new JPopupMenu();
        JMenuItem menuItem=new JMenuItem("Set as Current Task");
        menuItem.addActionListener(new ActionListener() {
            /**
             * When "Set as current task" is selected by the user, tell the plugin to set the selected task as the
             * current/default task.
             * @param e Event data
             */
            public void actionPerformed(ActionEvent e) {
                setTask();
            }
        });
        taskPopupMenu.add(menuItem);

        menuItem = new JMenuItem("Complete Task");
        menuItem.addActionListener(new ActionListener() {
            /**
             * When "Complete Task" is selected by the user, tell the plugin to complete the selected task.
             * @param e Event data
             */
            public void actionPerformed(ActionEvent e) {
                completeTask();
            }
        });
        taskPopupMenu.add(menuItem);

        menuItem = new JMenuItem("Update Associated Objects");
        menuItem.addActionListener(new ActionListener() {
            /**
             * Tell the plugin to update/refresh the assoc. objects table.
             * @param e Event data
             */
            public void actionPerformed(ActionEvent e) {
                new PopulateAssociatedObjectsTable(currentlySelectedTask).run();
            }
        });
        taskPopupMenu.add(menuItem);

        menuItem = new JMenuItem("Task Properties");
        menuItem.addActionListener(new ActionListener() {
            /**
             * When "Complete Task" is selected by the user, tell the plugin to complete the selected task.
             * @param e Event data
             */
            public void actionPerformed(ActionEvent e) {
                // Call properties function
                getTaskProperties();
            }
        });
        taskPopupMenu.add(menuItem);

        // Listener for mouse clicks on the task list
        taskList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent event) {
                if(!event.getValueIsAdjusting()) {
                    if(totalTasks > 0) {
                        setSelectedTask();

                        try {
                            if(!currentlySelectedTask.equals(previouslySelectedTask)) {
                                new PopulateAssociatedObjectsTable(currentlySelectedTask).run();
                            }
                        } catch (Exception e) {
                            // Catches a nullpointerexception when a session is stopped
                            // As such, clear the assoc objects table
                            associatedObjectTable.setModel(new DefaultTableModel());
                        }
                    }
                }
            }
        });
        taskList.addMouseListener(new MouseAdapter() {

            /**
             * Handles what happens when an item in the task list is clicked. First, the clicked task is selected. If
             * the task was not previously selected, the objects associated with it are loaded. If the task is clicked
             * twice, the method looks at the user-selected options to determine whether to set the task as default or
             * to display the task properties. If the event was a right click, a popup menu is displayed.
             * @param e Event data
             */
            public void mouseClicked(MouseEvent e) {

                // This calls a method that sets the selected task to that which is closest to the mouse location
                // when the mouse was clicked.
                changeTaskSelection(e);

                // First check if there are any tasks assigned before processing the table selection
                if(totalTasks > 0) {
                    setSelectedTask();

                    /*if(!currentlySelectedTask.equals(previouslySelectedTask)) {
                        new PopulateAssociatedObjectsTable(currentlySelectedTask).run();
                    }*/

                    if(e.getClickCount() == 2) { // Could be used to set default task or show task properties. Otherwise, no usage
                        //JList list = (JList)e.getSource();
                        //int index = list.locationToIndex(e.getPoint());

                        // Sets the task as default if the user selected such functionality in the options tab
                        if(config.getOptionTabTask().equals("set")) {
                            setTask();
                        }
                        else {
                            getTaskProperties();
                        }
                    }
                    else if(e.isPopupTrigger() || e.getButton() == MouseEvent.BUTTON3)
                    {
                        taskPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                    }

                }
            }
        });

    }

    /**
     * Creates the listner and its functions for the Associated Objects table. There is only functionality for a dobule
     * click. Such an event will cause the plugin to search for the associated object in the user's IDEA project. If
     * the object is found, it is shown in the editor. If not, a message is displayed telling the user the object is
     * not in the project. This message could be replaced with a feature to open a read-only editor within IDEA
     * or in an external application (e.g. Notepad) to allow the user to view the object even if it's not in the active
     * IDEA project.
     */
    private void tableListenerSetup() {
        associatedObjectTable.addMouseListener(new MouseAdapter() {
            /**
             * Double-click listener functionality
             * @param e Event data
             */
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {

                    int row                 = associatedObjectTable.getSelectedRow();
                    int objectColumn        = 5;
                    int typeColumn          = 2;
                    int objectNameColumn    = 0;

                    for(int i = 0; i < associatedObjectTable.getColumnCount(); i++) {
                        if(associatedObjectTable.getColumnName(i).equals("Name")) {
                            objectNameColumn = i;
                        }
                        else if(associatedObjectTable.getColumnName(i).equals("Synergy Name")) {
                            objectColumn = i;
                        }
                        else if(associatedObjectTable.getColumnName(i).equals("Type")) {
                            typeColumn = i;
                        }
                    }
                    if(associatedObjectTable.getValueAt(row, typeColumn).equals("dir")) {
                        // Show message saying selection is a directory and can't be viewed
                        CMMessages.generalMessage("This object is a directory and cannot be opened in the editor");
                    }
                    else {
                        String object = associatedObjectTable.getValueAt(row, objectColumn).toString();
                        String objectName = associatedObjectTable.getValueAt(row, objectNameColumn).toString();
                        new ViewAssociatedObjectAction(project, object, objectName);
                    }
                }
            }
        });
    }

    /**
     * Converts the CLI output of a task listing into a task number. The format from the CLI is xx) Task yy: Synopsis.
     * The algorithm returns 'yy' as an integer
     * @param text The text to be parsed
     * @return The integer value of the task
     */
   /* private String parseTask(String text) {

        // Remove the description
        String task = text.split(":")[0];                    // "xx) Task yy"

        // Remove everything else but the task ID
        String taskID = task.split("Task")[1].trim();   // "yy"

        return taskID;
    }*/

    /**
     * Sets the task selected in the "My Assigned Tasks" list.
     */
    private void setSelectedTask() {

        previouslySelectedTask = currentlySelectedTask;
        currentlySelectedTask = (String)taskList.getSelectedValue();

        // Get selected value in the task list
        //String val = (String)taskList.getSelectedValue();

        //currentlySelectedTask = parseTask(val);
    }

    /**
     * Returns the task currently selected in the My Assigned Tasks list
     * @return the currently selected task
     */
    private String getSelectedTask() {
        return currentlySelectedTask;
    }

    /**
     * This locates the position (point) of the mouse in the tasks list, then sets the selected task index to the that
     * which is closest to the mouse
     * @param e Event data
     */
    private void changeTaskSelection(MouseEvent e) {
        JList list = (JList)e.getSource();
        int taskidx = list.locationToIndex(e.getPoint());
        list.setSelectedIndex(taskidx);
    }

    /**
     * Sets the taskList selection to a specific task if that task exists in the list
     * @param task The task to set
     */
/*
    public void changeTaskSelection(String task) {
        ListModel listModel = taskList.getModel();
        // Iterate through taskList and look for a match
        for(int i = 0; i < listModel.getSize(); i++) {
            String item = listModel.getElementAt(i).toString();
            if(parseTask(item).equals(task)) {
                taskList.setSelectedIndex(i);
            }
        }
    }
*/

    /**
     * Sets the current/default task to the currently-selected task in the task tab
     */
    //TODO: Fix momentary freeze in the UI when this method is run
    public void setTask() {
        // if(getSelectedTask() != Tasks.getCurrentTaskToInt(project)) {
        Tasks.setCurrentTask(getSelectedTask(), project);
        currentTaskValue.setText(Tasks.getCurrentTask(project));
        taskList.updateUI();
        //}
    }

    /**
     * Sets the current/default task to the value passed
     * @param task The task to be set
     */
    public void setTask(String task)
    {
        Tasks.setCurrentTask(task, project);
        currentTaskValue.setText(Tasks.getCurrentTask(project));
        taskList.updateUI();
    }

    /**
     * Handles confirmation of the task completion command and gathers a completion comment.  It then sends the task and
     * comment to the CompleteTaskCommand class, where the task itself is completed. The tasks tab is updated afterwards
     * to remove the completed task from the list of assigned tasks. If no comment is given by the user, the command is
     * not executed.
     */
    private void completeTask() {
        String task = getSelectedTask();

        HashMap<String,String> properties = new HashMap<String,String>();
        properties = Tasks.getTaskProperties(project, task);

        // Get comment
        if(Messages.showYesNoDialog("Are you sure you want to complete task " + task + "?", "Confirmation", null) == 0) {
            CommentDialog dialog = new CommentDialog();
            dialog.setTitle("Complete Task Comments");
            if(properties.containsKey("Description"))   { 
                dialog.setComment(properties.get("Description"));
            }

            dialog.setRequired(true);
            dialog.showDialog();

            if(!dialog.wasCancelled()) {
                String comment = dialog.getComment();

                if(comment != null) {
                    new CompleteTaskCommand(task, project, comment);
                    updateTaskList();
                }
            }
            else {
                Messages.showWarningDialog("Complete Task cancelled by user", "Complete Task Failed");
            }
        }
    }

    /**
     * Gets the properties for the task selected in the My Assigned Tasks list
     */
    private void getTaskProperties() {
        String task = getSelectedTask();
        TaskPropertiesDialog tp = new TaskPropertiesDialog(project, this, task);

        tp.showDialog();
    }

    /**
     * Starts a task update thread. This method checks to see if an update thread is already started. If not, the thread
     * is created then executred
     */
    public void updateTaskList() {
        if(taskUpdateThread == null || taskUpdateThread.getState() == Thread.State.TERMINATED) {
            taskUpdateThread = new PopulateTaskList();
            taskUpdateThread.start();
        }
    }

    public void updateAssociatedObjectsList() {
        new PopulateAssociatedObjectsTable(currentlySelectedTask).run();
    }

    public void setCreateTaskButtonEnabled(boolean isEnabled) {
        createTaskButton.setEnabled(isEnabled);
    }

    private void createUIComponents() {
        // TODO: place custom component creation code here
    }

    /**
     * Handles the updating of the My Assigned Tasks list.
     */
    private class PopulateTaskList extends Thread {

        /**
         * Sets up the task list, gathers the assigned tasks, then populates the list.
         */
        public void run() {
            this.setPriority(Thread.NORM_PRIORITY);

            ListCellRenderer renderer = new TaskListRenderer();

            Tasks taskHandler = new Tasks(project);
            LinkedHashMap<String,String> taskData = taskHandler.getMyAssignedTasks();
            //Vector taskVector = taskHandler.getMyAssignedTasks();

            //Object elements[] = taskVector.toArray();

            /*if(elements.length > 0) {
                if(elements[0].toString().contains("No tasks assigned")) {
                    totalTasks = 0;
                }
                else {
                    totalTasks = elements.length;
                }
            }*/

            //taskList.setListData(elements);
            taskListModel = new TaskListModel(taskData);
            taskList.setModel(taskListModel);
            //taskList.setListData(taskData);
            taskList.setCellRenderer(renderer);
            totalTasks = taskData.size();

            currentTaskValue.setText(Tasks.getCurrentTask(project));
        }
    }

    /**
     * Handles the visual rendering of the task list.
     */
    private class TaskListRenderer implements ListCellRenderer {
        DefaultListCellRenderer defaultRenderer = new DefaultListCellRenderer();

        /**
         * Renders task list items. The CLI output of the assigned tasks is parsed to remove some unneeded information
         * and the current task is rendered in bold.
         * @param list The JList to be rendered
         * @param value The list item to be rendered
         * @param index The item's position in the list
         * @param isSelected Whether the item is selected
         * @param cellHasFocus Whether the item is in focus
         * @return A JLabel with the rendered list item
         */
        public Component getListCellRendererComponent(JList list, Object value,
                                                      int index, boolean isSelected, boolean cellHasFocus) {

            JLabel renderer = (JLabel) defaultRenderer
                    .getListCellRendererComponent(list, value, index, isSelected,
                            cellHasFocus);

            Font defaultFont = defaultRenderer.getFont();
            Font displayFont = defaultFont;

            String strVal = (String)value;

            // Checks to see if the query returned anything
            if(strVal.contains("No tasks assigned")) {
                renderer.setText(strVal);
                displayFont = new Font(defaultFont.getFontName(), Font.ITALIC, defaultFont.getSize());
                //TODO: disable the ability to fire a mouse event here. Perhaps renderer.setEnabled(false)
            }
            else {
                // Remove the xx) from xx) Task yy: Synopsis
                /*String taskDisplay  = strVal.split("\\)", 2)[1];
                String task         = strVal.split(":")[0];
                String taskNumber   = task.split("Task")[1].trim();   // "yy"

                String taskID = "";

                // In case the results from the query are in an unexpected format, catch any number format exceptions
                // that could freeze IDEA
                try {
                    taskID = taskNumber;
                } catch (NumberFormatException e) {
                    new CMExceptionHandler(e);
                }
                taskID = value;
*/


                if(strVal.equals(config.getDefaultTask())) {
                    displayFont = new Font(defaultFont.getFontName(), Font.BOLD, defaultFont.getSize());
                }

                renderer.setText(strVal + ": " + taskListModel.getSynopsis(strVal));
            }
            renderer.setFont(displayFont);
            return renderer;
        }
    }

    /**
     * Handles the gathering and display of objects associated with a selected task
     */
    private class PopulateAssociatedObjectsTable extends Thread {

        private String taskID;
        private DefaultTableCellRenderer rightAlignRenderer;

        /**
         * Main constructor. Sets up the basic properties of the table
         * @param taskID The task whose associated objects are to be displayed
         */
        public PopulateAssociatedObjectsTable(String taskID) {
            this.taskID = taskID;
            //TitledBorder border=new TitledBorder("Objects Associated with Task " + taskID);
            objects.setText("Objects Associated with Task " + taskID);

            rightAlignRenderer = new DefaultTableCellRenderer();
            //assocObjectsScrollPane.setBorder(border);

            rightAlignRenderer.setHorizontalAlignment(JLabel.RIGHT);
        }

        /**
         * Gathers the associated objects from the Tasks class and places them into a table model.
         */
        public void run() {
            if(taskID != null) {
                // run tasks command
                Vector<Vector> objects = Tasks.getAssociatedObjects(taskID, project);
                Vector<String> columnNames;
                ObjectTableModel tableModel = new ObjectTableModel();

                columnNames = new Vector<String>();
                columnNames.addElement("Name");
                columnNames.addElement("Version");
                columnNames.addElement("Type");
                columnNames.addElement("Owner");
                columnNames.addElement("State");
                columnNames.addElement("Associated Tasks");
                columnNames.addElement("Synergy Name");
                columnNames.addElement("");

                tableModel.setDataVector(objects, columnNames);

                associatedObjectTable.setModel(tableModel);
                associatedObjectTable.setShowVerticalLines(false);
                associatedObjectTable.setColumnSelectionAllowed(false);
                associatedObjectTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
                associatedObjectTable.setIntercellSpacing(new Dimension(2,0));
                associatedObjectTable.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);

                // Align all text in columns to the right (except for the first column
                // This could be done by starting the iterator at 1, but stuck with 0 to make things less confusing
                for (int i = 0; i < associatedObjectTable.getColumnModel().getColumnCount(); i++) {
                    if(i != 0) {
                        associatedObjectTable.getColumnModel().getColumn(i).setCellRenderer(rightAlignRenderer);
                    }
                }
            }
        }

        /**
         * Custom table model. Its sole purpose is to disallow editing of the table by a user
         */
        private class ObjectTableModel extends DefaultTableModel {

            /**
             * Overrides the DefaulTableModel method to diable cell editing
             * @param row Row ID
             * @param col Column ID
             * @return Whether the cell can be edited
             */
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        }

    }


    private class TaskListModel extends AbstractListModel {
        private LinkedHashMap<String,String> data;
        private String[] keys;

        public TaskListModel(LinkedHashMap<String,String> map) {
            data = map;
            keys = map.keySet().toArray(new String[map.size()]);
        }

        public int getSize() {
            return keys.length;  //To change body of implemented methods use File | Settings | File Templates.
        }

        public String getElementAt(int i) {
            return keys[i];
        }

        public String getMappedValue(int i) {
            return data.get(getElementAt(i));
        }

        public String getSynopsis(String key) {
            return data.get(key);
        }
    }

}
