// Copyright 2006, 2007 Telelogic AB
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.telelogic.integration.intellij.ui.tabs;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.options.ConfigurationException;
import com.telelogic.integration.intellij.actions.StartSession;
import com.telelogic.integration.intellij.actions.StartGUI;
import com.telelogic.integration.intellij.actions.StopSession;
import com.telelogic.integration.intellij.cm.CMConfiguration;
import com.telelogic.integration.intellij.ui.CMConfigurable;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

/**
 * Provides the Session tab and its related functions
 */
public class Session {
    private JPanel sessionCtrlPanel;
    private JPanel buttonPanel;
    private JButton startCLIButton;
    private JButton startGUIButton;
    private JButton stopSessionButton;
    private JPanel synergyImagePanel;
    private JProgressBar sessionProgress;
    private JTabbedPane logTabs;
    private JPanel pluginMessagesPanel;
    private JTextArea consoleText;
    private JPanel start;
    private JPanel sessionSettings;
    private CMConfigurable  settings;

    private JButton applySettingsButton;
    private JButton resetSettingsButton;
    /**
     * Main constructor
     * @param project The active IDEA project
     */
    public Session(Project project) {
        //CMConfiguration config=CMConfiguration.getInstance(project);
        settings = new CMConfigurable(project);

        sessionProgress.setVisible(false);

        // Console Formatting
        consoleText.setEditable(false);
        consoleText.setBackground(Color.WHITE);
        consoleText.setMargin(new Insets(3,3,3,3));
        consoleText.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));

        // Add listeners for the Session tab panel
        startCLIButton.addActionListener(new StartSession(project));
        startGUIButton.addActionListener(new StartGUI(project));
        stopSessionButton.addActionListener(new StopSession(project));

        setupSettings();
    }

    /**
     * Returns the JPanel for the Session tab
     * @return This class's JPanel
     */
    public JPanel getPanel() {
        return start;
    }

    /**
     * Creates the Session Settings tab and sets up listeners for the "Apply" and "Revert" buttons
     */
    public void setupSettings() {
        sessionSettings = new JPanel(new BorderLayout());
        sessionSettings.setName("Session Settings");

        sessionSettings.setBorder(new EmptyBorder(2,2,2,2));

        applySettingsButton = new JButton("Apply");
        applySettingsButton.setSize(20, 80);
        resetSettingsButton = new JButton("Reset");
        resetSettingsButton.setSize(20, 80);

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(applySettingsButton);
        buttonPanel.add(resetSettingsButton);


        applySettingsButton.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if(e.getActionCommand().equals("Apply")) {
                            try {
                                settings.apply();
                            }catch(ConfigurationException ce) {}
                        }
                    }
                }
        );

        resetSettingsButton.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if(e.getActionCommand().equals("Reset")) {
                            settings.reset();
                        }
                    }
                }
        );

        // Get the VCS settings panel and "reset" it to show the current config
        JComponent settingsComponent = settings.getConfigPanel();
        settings.reset();

        // Add to the logTabs panel
        sessionSettings.add(settingsComponent, BorderLayout.CENTER);
        sessionSettings.add(buttonPanel, BorderLayout.SOUTH);
        sessionSettings.repaint();
        logTabs.add(sessionSettings.getName(), sessionSettings);

        // Refresh settings data when this tab panel is selected
        logTabs.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                JTabbedPane pane = (JTabbedPane)e.getSource();
                if(pane.getSelectedComponent().equals(sessionSettings)) {
                    settings.reset();
                }
            }
        });

    }

    /**
     * Begins the display of an indeterminate progress bar, indicating a Synergy session being started
     */
    public void startProgressBar() {
        sessionProgress.setVisible(true);
        sessionProgress.setIndeterminate(true);
    }

    /**
     * Ends the display of the progress bar, indicating that a session either started or failed to start
     */
    public void stopProgressBar() {
        sessionProgress.setVisible(false);
        sessionProgress.setIndeterminate(false);
        sessionProgress.setValue(sessionProgress.getMinimum());
    }

    /**
     * Adds a line of text to the bottom of the console panel.  Used primarily for command execution output
     * @param line The text to be added
     */
    public void appendConsole(String line)
    {
        consoleText.append(line + "\n");
        consoleText.setCaretPosition(consoleText.getDocument().getLength());
        consoleText.repaint();
    }

    /**
     * Enable the buttons used to start a CLI or GUI Synergy session
     */
    public void enableStart()   { startCLIButton.setEnabled(true);  startGUIButton.setEnabled(true); }

    /**
     * Disable the buttons used to start a CLI or GUI Synergy session
     */
    public void disableStart()  { startCLIButton.setEnabled(false); startGUIButton.setEnabled(false); }

    /**
     * Enable the button used to stop a CLI or GUI Synergy session
     */
    public void enableStop()    { stopSessionButton.setEnabled(true); }

    /**
     * Enable the button used to stop a CLI or GUI Synergy session
     */
    public void disableStop()   { stopSessionButton.setEnabled(false);}

    private void createUIComponents() {
        // TODO: place custom component creation code here
                // Get config panel and display it in the sessionSettings tab
        //JPanel sessionPanel = settings.getConfigPanel();
//        sessionSettings = new JPanel();

        //JComponent settingsComponent = settings.createComponent();
        //sessionSettings.add(settingsComponent);
        /*settingsComponent.setVisible(true);
        sessionSettings.setVisible(true);
        sessionSettings.add(startCLIButton);*/
       // sessionSettings.repaint();
    }
}
